package com.yizhi.library.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.library.application.param.RelateProjectParam;
import com.yizhi.library.application.vo.CaseLibraryVO;
import com.yizhi.library.application.vo.CaselibraryAuthorizeVO;
import com.yizhi.library.application.vo.ClassifyVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

/**
 * @author Ding
 * @className CaseLibraryClient
 * @description TODO
 * @date 2019/7/3
 **/

@FeignClient(name = "caselibrary", contextId = "CaseLibraryClient")
public interface CaseLibraryClient {

    /**
     * 获取活动管理列表
     *
     * @param name
     * @param state
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/caseLibrary/list")
    Page<CaseLibraryVO> getCaseLibraryList(@RequestParam(name = "name", required = false) String name,
                                           @RequestParam(name = "state", required = false) Integer state,
                                           @RequestParam(name = "pageNo") Integer pageNo,
                                           @RequestParam(name = "pageSize") Integer pageSize);

    /**
     * 新建案例活动
     *
     * @param caseLibraryVO
     * @return
     */
    @PostMapping("/caseLibrary/insert")
    Long insertCaseLibrary(@RequestBody CaseLibraryVO caseLibraryVO);

    /**
     * 新建案例活动获取维度列表
     *
     * @param caseLibraryId
     * @return
     */
    @GetMapping("/caseLibrary/getClassify")
    List<ClassifyVO> getClassify(@RequestParam(name = "caseLibraryId", required = false) Long caseLibraryId);

    /**
     * 案例活动审核对象导入
     *
     * @param caseLibraryId
     * @param url
     * @return
     */
    @GetMapping("/caseLibrary/importAuditor")
    String importAuditor(@RequestParam("caseLibraryId") Long caseLibraryId, @RequestParam("url") String url);

    /**
     * 案例活动必须提交人导入
     *
     * @param caseLibraryId
     * @param url
     * @return
     */
    @GetMapping("/caseLibrary/importScorer")
    String importScorer(@RequestParam("caseLibraryId") Long caseLibraryId, @RequestParam("url") String url);

    /**
     * 案例活动评分对象导入
     *
     * @param caseLibraryId
     * @param url
     * @return
     */
    @GetMapping("/caseLibrary/importSubmitter")
    String importSubmitter(@RequestParam("caseLibraryId") Long caseLibraryId, @RequestParam("url") String url);


    /**
     * 案例活动修改获取信息
     *
     * @param caseLibraryId
     * @return
     */
    @GetMapping("/caseLibrary/view")
    CaseLibraryVO viewCaseLibrary(@RequestParam("caseLibraryId") Long caseLibraryId);

    /**
     * 更新案例活动
     *
     * @param caseLibraryVO
     * @return
     */
    @PostMapping("/caseLibrary/update")
    boolean updateCaseLibrary(@RequestBody CaseLibraryVO caseLibraryVO);

    /**
     * 案例活动移除分类维度
     *
     * @param id
     * @return
     */
    @GetMapping("/caseLibrary/relatedClassify/delete")
    boolean deleteRelatedClassify(@RequestParam("id") Long id);

    /**
     * 案例活动发布上架
     *
     * @param caseLibraryId
     * @return
     */
    @GetMapping("/caseLibrary/release")
    boolean releaseCaseLibrary(@RequestParam("caseLibraryId") Long caseLibraryId);

    /**
     * 案例活动发布下架
     *
     * @param caseLibraryId
     * @return
     */
    @GetMapping("/caseLibrary/putDown")
    boolean putDownCaseLibrary(@RequestParam("caseLibraryId") Long caseLibraryId);

    /**
     * 案例活动草稿状态下删除
     *
     * @param caseLibraryId
     * @return
     */
    @GetMapping("/caseLibrary/delete")
    boolean deleteCaseLibrary(@RequestParam("caseLibraryId") Long caseLibraryId);

    /**
     * 案例活动关联培训项目
     *
     * @param relateProjectParam
     * @return
     */
    @PostMapping("/caseLibrary/relateProject")
    boolean relateProject(@RequestBody RelateProjectParam relateProjectParam);

    /**
     * 案例活动取消关联培训项目
     *
     * @param caseLibraryId
     * @return
     */
    @GetMapping("/caseLibrary/cancelRelateProject")
    boolean cancelRelateProject(@RequestParam("caseLibraryId") Long caseLibraryId);

    /**
     * 培训项目关联案例活动获取列表
     *
     * @param name
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/caseLibrary/getProjectList")
    Page<CaseLibraryVO> getProjectList(@RequestParam(value = "name", required = false) String name,
                                     @RequestParam("pageNo") Integer pageNo,
                                     @RequestParam("pageSize") Integer pageSize);

    @PostMapping("/caseLibrary/insertAuthorizes")
    boolean insertAuthorizes(@RequestBody List<CaselibraryAuthorizeVO> caseLibraryAuthorizeList);

    @GetMapping("/caseLibrary/getAuthorize")
    List<CaselibraryAuthorizeVO> getAuthorize(@RequestParam("caselibraryId") Long caselibraryId);

    /**
     * 获取案例活动信息
     * @param caselibraryId
     * @return
     */
    @GetMapping("/caseLibrary/getCaseLibrary")
    CaseLibraryVO getCaseLibrary(@RequestParam("caselibraryId") Long caselibraryId);

    /**
     * 管理端首页配置上传案例获取案例活动列表
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/caseLibrary/getHomeList")
    Page<CaseLibraryVO> getHomeList(@RequestParam("pageNo") Integer pageNo,
                                  @RequestParam("pageSize") Integer pageSize);

    /**
     * 规则搜索下拉框分页数据
     * @param field
     * @param value
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/caseLibrary/getPageByDrools")
    Page<DroolsVo> getPageByDrools(@RequestParam("field") String field,
                                   @RequestParam(value = "value", required = false) String value,
                                   @RequestParam("pageNo") Integer pageNo,
                                   @RequestParam("pageSize") Integer pageSize);
}
