package com.yizhi.library.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.library.application.vo.CommentReplyVO;
import com.yizhi.library.application.vo.CommentVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

/**
 * @author Ding
 * @className CommentManageClient
 * @description TODO
 * @date 2019/7/15
 **/

@FeignClient(name = "caselibrary", contextId = "CommentManageClient")
public interface CommentManageClient {

    /**
     * 获取评论管理列表
     *
     * @param studentCaseId
     * @param name
     * @param state
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/comment/manage/list")
    Page<CommentVO> getCommentManageList(@RequestParam(name = "studentCaseId", required = true) Long studentCaseId,
                                         @RequestParam(name = "name", required = false) String name,
                                         @RequestParam(name = "state", required = false) Integer state,
                                         @RequestParam("pageNo") Integer pageNo,
                                         @RequestParam("pageSize") Integer pageSize);

    /**
     * 下载评论信息
     *
     * @param studentCaseId
     * @param name
     * @param state
     * @return
     */
    @GetMapping("/comment/manage/downloadList")
    String downloadList(@RequestParam(name = "studentCaseId", required = true) Long studentCaseId,
                        @RequestParam(name = "name", required = false) String name,
                        @RequestParam(name = "state", required = false) Integer state);

    /**
     * 获取评论回复列表
     *
     * @param name
     * @param state
     * @param commentId
     * @return
     */
    @GetMapping("/comment/manage/replyList")
    List<CommentReplyVO> getCommentReplyList(@RequestParam(name = "name", required = false) String name,
                                             @RequestParam(name = "state", required = false) Integer state,
                                             @RequestParam("commentId") Long commentId);

    /**
     * 评论/回复下架
     *
     * @param id
     * @param type
     * @return
     */
    @GetMapping("/comment/manage/putDown")
    boolean putDown(@RequestParam(name = "id", required = true) Long id,
                    @RequestParam(name = "type", required = true) Integer type);

    /**
     * 评论/回复上架
     *
     * @param id
     * @param type
     * @return
     */
    @GetMapping("/comment/manage/release")
    boolean release(@RequestParam(name = "id", required = true) Long id,
                    @RequestParam(name = "type", required = true) Integer type);

    /**
     * 获取评论回复删除
     *
     * @param id
     * @param type
     * @return
     */
    @GetMapping("/comment/manage/delete")
    boolean delete(@RequestParam(name = "id", required = true) Long id,
                   @RequestParam(name = "type", required = true) Integer type);
}
