package com.yizhi.library.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.library.application.param.*;
import com.yizhi.library.application.vo.*;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Map;

/**
 * @author Ding
 * @className StudentCaseClient
 * @description TODO
 * @date 2019/7/9
 **/
@FeignClient(name = "caselibrary", contextId = "StudentCaseClient")
public interface StudentCaseClient {

    /**
     * pc端,移动端我的案例列表
     *
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/studentCase/myCaseLibrary/list")
    Page<CaseLibraryVO> myCaseLibraryList(@RequestParam("pageNo") Integer pageNo, @RequestParam("pageSize") Integer pageSize);

    /**
     * pc端,移动端我的案例记录总数
     *
     * @return
     */
    @GetMapping("/studentCase/myCaseLibrary/total")
    Integer myCaseLibraryTotal();

    /**
     * 学员提交案例获取案例库信息
     *
     * @param caseLibraryId
     * @return
     */
    @GetMapping("/studentCase/submit/prepare")
    StudentCaseVO prepareStudentCase(@RequestParam("caseLibraryId") Long caseLibraryId);

    /**
     * 学员提交案例
     *
     * @param studentCaseParam
     * @return
     */
    @PostMapping("/studentCase/submit")
    boolean submitStudentCase(@RequestBody StudentCaseParam studentCaseParam);

    /**
     * 获取学员案例提交记录列表
     *
     * @param caseLibraryId
     * @param endTime
     * @return
     */
    @GetMapping("/studentCase/getSubmitRecords")
    StudentCaseRecordListVO getSubmitRecords(@RequestParam("caseLibraryId") Long caseLibraryId,
                                             @RequestParam("endTime") String endTime);

    /**
     * 学员案例提交记录中提交
     *
     * @param studentCaseId
     * @return
     */
    @GetMapping("/studentCase/record/studentCaseRecordSubmit")
    boolean studentCaseRecordSubmit(@RequestParam("studentCaseId") Long studentCaseId);

    /**
     * 学员案例提交记录中撤回
     *
     * @param studentCaseId
     * @return
     */
    @GetMapping("/studentCase/record/studentCaseRecall")
    boolean studentCaseRecall(@RequestParam("studentCaseId") Long studentCaseId);

    /**
     * 学员案例记录查看或者编辑获得信息
     *
     * @param studentCaseId
     * @param type
     * @return
     */
    @GetMapping("/studentCase/record/studentCaseView")
    StudentCaseViewVO studentCaseView(@RequestParam(name = "studentCaseId") Long studentCaseId,
                                      @RequestParam(name = "type", required = false) Integer type);

    /**
     * 学员案例审核,评分获取案例列表
     *
     * @param type
     * @param name
     * @param pageSize
     * @param pageNo
     * @return
     */
    @GetMapping("/studentCase/getAuditList")
    Page<StudentCaseStudyVO> getAuditList(@RequestParam("type") Integer type,
                                          @RequestParam(value = "name", required = false) String name,
                                          @RequestParam("pageNo") Integer pageNo,
                                          @RequestParam("pageSize") Integer pageSize);

    /**
     * 学员案例学习获取案例信息/审核,评分获取案例信息
     *
     * @param studentCaseId
     * @param type
     * @return
     */
    @GetMapping("/studentCase/studyStudentCase")
    StudentCaseStudyVO studyStudentCase(@RequestParam("studentCaseId") Long studentCaseId,
                                        @RequestParam("type") Integer type,
                                        @RequestParam(name = "number", required = false) Integer number,
                                        @RequestParam(name = "name", required = false) String name);

    /**
     * 学员案例审核提交
     *
     * @param studentCaseAuditParam
     * @return
     */
    @PostMapping("/studentCase/audit")
    boolean studentCaseAudit(@RequestBody StudentCaseAuditParam studentCaseAuditParam);

    /**
     * 学员案例批量审核
     *
     * @param studentCaseAuditParam
     * @return
     */
    @PostMapping("/studentCase/batchAudit")
    boolean studentCaseBatchAudit(@RequestBody StudentCaseAuditParam studentCaseAuditParam);

    /**
     * 学员案例评分提交
     *
     * @param studentCaseAuditParam
     * @return
     */
    @PostMapping("/studentCase/score")
    boolean studentCaseScore(@RequestBody StudentCaseAuditParam studentCaseAuditParam);

    /**
     * 学员案例批量评分
     *
     * @param studentCaseAuditParam
     * @return
     */
    @PostMapping("/studentCase/batchScore")
    boolean studentCaseBatchScore(@RequestBody StudentCaseAuditParam studentCaseAuditParam);

    /**
     * 学员案例审核/评分导出案例
     *
     * @param studentCaseAuditParam
     * @return
     */
    @PostMapping("/studentCase/batchExport")
    String studentCaseBatchExport(@RequestBody StudentCaseAuditParam studentCaseAuditParam);

    /**
     * 首页配置获取列表
     *
     * @param homeListParam
     * @return
     */
    @PostMapping("/studentCase/homeList")
    Page<StudentCaseVO> getStudentCaseHomeList(@RequestBody HomeListParam homeListParam);

    /**
     * 首页配置获取学员案例信息
     *
     * @param studentCaseId
     * @return
     */
    @GetMapping("/studentCase/getStudentCaseDetails")
    StudentCaseStudyVO getStudentCaseDetails(@RequestParam("studentCaseId") Long studentCaseId);

    /**
     * 管理端修改学员案例新增可见范围用户,部门
     *
     * @param studentCaseAuthorizeList
     * @return
     */
    @PostMapping("/studentCase/insertAuthorizes")
    boolean insertAuthorizes(@RequestBody List<StudentCaseAuthorizeVO> studentCaseAuthorizeList);

    /**
     * 获取指定范围列表
     *
     * @param studentCaseId
     * @return
     */
    @GetMapping("/studentCase/getAuthorize")
    List<StudentCaseAuthorizeVO> getAuthorize(@RequestParam("studentCaseId") Long studentCaseId);

    /**
     * 获取我的收藏案例列表
     *
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/studentCase/getFavorites")
    Page<FavoriteVO> getFavorites(@RequestParam("pageNo") Integer pageNo,
                                  @RequestParam("pageSize") Integer pageSize,
                                  @RequestParam(value = "name", required = false) String name);

    /**
     * 首页获取案例点赞总数
     *
     * @param companyId
     * @param siteId
     * @param accountId
     * @return
     */
    @GetMapping("/studentCase/getFavoritesCount")
    Integer getFavoritesCount(@RequestParam("companyId") Long companyId,
                              @RequestParam("siteId") Long siteId,
                              @RequestParam("accountId") Long accountId);

    /**
     * 培训项目关联精选案例获取列表
     *
     * @param name
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/studentCase/getProjectList")
    Page<StudentCaseStudyVO> getProjectList(@RequestParam(value = "name", required = false) String name,
                                            @RequestParam("pageNo") Integer pageNo,
                                            @RequestParam("pageSize") Integer pageSize);

    /**
     * 精选案例搜索页获取列表
     *
     * @param selectionStudentCaseParam
     * @return
     */
    @PostMapping("/studentCase/selectionList")
    Page<FavoriteVO> getSelectionStudentCaseList(@RequestBody SelectionStudentCaseParam selectionStudentCaseParam);

    /**
     * 首页获取精选案例信息
     *
     * @param studentCaseId
     * @return
     */
    @GetMapping("/studentCase/getStudentCaseDetail")
    FavoriteVO getStudentCaseDetail(@RequestParam(value = "studentCaseId") Long studentCaseId);

    @GetMapping("/studentCase/VisibleRange")
    List<StudentCaseAuthorizeVO> VisibleRange(@RequestParam(value = "studentCaseId") Long studentCaseId);

    /**
     * 学员端审核案例修改关联分类
     * @param param
     * @return
     */
    @PostMapping("/studentCase/updateClassify")
    boolean modifyRlatedClassifys(@RequestBody UpdateCaseRelatedClassifyParam param);

    /**
     * 通知案例微服务该附件是视频/音频文件,等待转码
     * @param ids
     * @return
     */
    @PostMapping("/studentCase/verifyVedio")
    boolean verifyVedio(@RequestBody List<Long> ids);

    /**
     * 更新视频/音频转码url
     * @param map
     * @return
     */
    @PostMapping("/studentCase/updateTranscodeUrl")
    public boolean updateTranscodeUrl(@RequestBody Map<String, String> map);

    /**
     * 规则搜索下拉框分页数据
     * @param field
     * @param value
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/studentCase/getPageByDrools")
    Page<DroolsVo> getPageByDrools(@RequestParam("field") String field,
                                   @RequestParam(value = "value", required = false) String value,
                                   @RequestParam("pageNo") Integer pageNo,
                                   @RequestParam("pageSize") Integer pageSize);
}
