package com.yizhi.library.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.library.application.param.ExportAuditListParam;
import com.yizhi.library.application.vo.SelectionCaseListVO;
import com.yizhi.library.application.vo.StudentCaseManageListVO;
import com.yizhi.library.application.vo.StudentCaseStudyVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Map;

/**
 * @author Ding
 * @className StudentCaseManageClient
 * @description TODO
 * @date 2019/7/25
 **/
@FeignClient(name = "caselibrary", contextId = "StudentCaseManageClient")
public interface StudentCaseManageClient {

    /**
     * 所有学员案例管理列表/审核评分管理列表
     *
     * @param studentCaseName
     * @param accountName
     * @param state
     * @param pageNo
     * @param pageSize
     * @param caseLibraryId
     * @param orgName
     * @return
     */
    @GetMapping("/manage/studentCase/list")
    Page<StudentCaseManageListVO> studentCaseManageList(@RequestParam(name = "caseLibraryId", required = true) Long caseLibraryId,
                                                        @RequestParam(name = "studentCaseName", required = false) String studentCaseName,
                                                        @RequestParam(name = "accountName", required = false) String accountName,
                                                        @RequestParam(name = "state", required = false) Integer state,
                                                        @RequestParam(name = "orgName", required = false) String orgName,
                                                        @RequestParam(name = "pageNo", required = true) Integer pageNo,
                                                        @RequestParam(name = "pageSize", required = true) Integer pageSize);

    /**
     * 审核评分管理列表下载报表
     *
     * @param caseLibraryName
     * @param trainingProjectName
     * @param caseLibraryId
     * @param studentCaseName
     * @param accountName
     * @param state
     * @return
     */
    @GetMapping("/manage/studentCase/list/export")
    String exportStudentCaseManageList(@RequestParam(name = "caseLibraryName", required = false) String caseLibraryName,
                                       @RequestParam(name = "trainingProjectName", required = false) String trainingProjectName,
                                       @RequestParam(name = "caseLibraryId", required = true) Long caseLibraryId,
                                       @RequestParam(name = "studentCaseName", required = false) String studentCaseName,
                                       @RequestParam(name = "accountName", required = false) String accountName,
                                       @RequestParam(name = "state", required = false) Integer state);

    /**
     * 学员案例审核/评分获取学员案例信息
     *
     * @param studentCaseName
     * @param accountName
     * @param state
     * @param number
     * @param caseLibraryId
     * @param orgName
     * @return
     */
    @GetMapping("/manage/studentCase/auditOrScore")
    StudentCaseStudyVO studentCaseManageAuditOrScoreView(@RequestParam(name = "studentCaseId", required = true) Long studentCaseId,
                                                         @RequestParam(name = "caseLibraryId", required = false) Long caseLibraryId,
                                                         @RequestParam(name = "studentCaseName", required = false) String studentCaseName,
                                                         @RequestParam(name = "accountName", required = false) String accountName,
                                                         @RequestParam(name = "state", required = false) Integer state,
                                                         @RequestParam(name = "orgName", required = false) String orgName,
                                                         @RequestParam(name = "number", required = false) Integer number);

    /**
     * 导出待审核案例
     *
     * @param param
     * @return
     */
    @PostMapping("/manage/studentCase/auditList/export")
    String exportAuditList(@RequestBody ExportAuditListParam param);

    /**
     * 导入审核结果
     *
     * @param ossUrl
     * @return
     */
    @GetMapping("/manage/studentCase/auditList/import")
    String importAuditList(@RequestParam("ossUrl") String ossUrl);

    /**
     * 导出待评分案例
     *
     * @param param
     * @return
     */
    @PostMapping("/manage/studentCase/scoreList/export")
    String exportScoreList(@RequestBody ExportAuditListParam param);

    /**
     * 导入评分结果
     *
     * @param ossUrl
     * @return
     */
    @GetMapping("/manage/studentCase/scoreList/import")
    String importScoreList(@RequestParam("ossUrl") String ossUrl);

    /**
     * 精选案例管理列表
     *
     * @param studentCaseName
     * @param accountName
     * @param state
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/manage/studentCase/selectionList")
    Page<SelectionCaseListVO> getSelectionList(@RequestParam(name = "studentCaseName", required = false) String studentCaseName,
                                               @RequestParam(name = "accountName", required = false) String accountName,
                                               @RequestParam(name = "state", required = false) Integer state,
                                               @RequestParam("pageNo") Integer pageNo,
                                               @RequestParam("pageSize") Integer pageSize);

    /**
     * 精选案例管理列表下载报表
     *
     * @param studentCaseName
     * @param accountName
     * @param state
     * @return
     */
    @GetMapping("/manage/studentCase/selectionList/export")
    String selectionListExport(@RequestParam(name = "studentCaseName", required = false) String studentCaseName,
                               @RequestParam(name = "accountName", required = false) String accountName,
                               @RequestParam(name = "state", required = false) Integer state);

    /**
     * 举报记录列表
     *
     * @param studentCaseName
     * @param accountName
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/manage/studentCase/informList")
    Page<StudentCaseManageListVO> getInformList(@RequestParam(name = "studentCaseName", required = false) String studentCaseName,
                                                @RequestParam(name = "accountName", required = false) String accountName,
                                                @RequestParam("pageNo") Integer pageNo,
                                                @RequestParam("pageSize") Integer pageSize);

    /**
     * 举报记录列表导出
     *
     * @param studentCaseName
     * @param accountName
     * @return
     */
    @GetMapping("/manage/studentCase/informList/export")
    String informListExport(@RequestParam(name = "studentCaseName", required = false) String studentCaseName,
                            @RequestParam(name = "accountName", required = false) String accountName);

    /**
     * 举报记录查看
     *
     * @param informRecordId
     * @return
     */
    @GetMapping("/manage/studentCase/informList/view")
    StudentCaseStudyVO informListView(@RequestParam("informRecordId") Long informRecordId);

    /**
     * 案例管理列表获取记录数
     *
     * @param caseLibraryId
     * @return
     */
    @GetMapping("/manage/studentCase/list/getCount")
    Map<String, Object> getManageListCount(@RequestParam("caseLibraryId") Long caseLibraryId);

    /**
     * 学员案例下架
     *
     * @param studentCaseId
     * @return
     */
    @GetMapping("/manage/studentCase/putDown")
    boolean putDown(@RequestParam("studentCaseId") Long studentCaseId);

    /**
     * 学员案例上架
     *
     * @param studentCaseId
     * @return
     */
    @GetMapping("/manage/studentCase/release")
    boolean release(@RequestParam("studentCaseId") Long studentCaseId);

    /**
     * 学员案例删除
     * @param studentCaseId
     * @return
     */
    @GetMapping("/manage/studentCase/delete")
    boolean delete(@RequestParam("studentCaseId") Long studentCaseId);

    /**
     * 举报记录移除
     *
     * @param informRecordId
     * @return
     */
    @GetMapping("/manage/studentCase/informRecordRemove")
    boolean informRecordRemove(@RequestParam("informRecordId") Long informRecordId);
}
