package com.yizhi.library.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.library.application.param.CommentParam;
import com.yizhi.library.application.param.InformParam;
import com.yizhi.library.application.vo.CommentReplyVO;
import com.yizhi.library.application.vo.CommentVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * @author Ding
 * @className StudyCaseClient
 * @description TODO
 * @date 2019/7/12
 **/
@FeignClient(name = "caselibrary", contextId = "StudyCaseClient")
public interface StudyCaseClient {

    /**
     * 学习案例点赞
     *
     * @param studentCaseId
     * @return
     */
    @GetMapping("/studentCase/study/admire")
    boolean admireStudentCase(@RequestParam("studentCaseId") Long studentCaseId);

    /**
     * 学习案例取消点赞
     *
     * @param studentCaseId
     * @return
     */
    @GetMapping("/studentCase/study/cancelAdmire")
    boolean cancelAdmireStudentCase(@RequestParam("studentCaseId") Long studentCaseId);

    /**
     * 学习案例收藏
     *
     * @param studentCaseId
     * @return
     */
    @GetMapping("/studentCase/study/favorite")
    boolean favorite(@RequestParam("studentCaseId") Long studentCaseId);

    /**
     * 学习案例取消收藏
     *
     * @param studentCaseId
     * @return
     */
    @GetMapping("/studentCase/study/cancelFavorite")
    boolean cancelFavorite(@RequestParam("studentCaseId") Long studentCaseId);

    /**
     * 学习案例举报
     * @param informParam
     * @return
     */
    @PostMapping("/studentCase/study/inform")
    boolean inform(@RequestBody InformParam informParam);

    /**
     * 发表评论/回复
     * @param commentParam
     * @return
     */
    @PostMapping("/studentCase/study/comment")
    boolean comment(@RequestBody CommentParam commentParam);

    /**
     * 评论点赞
     * @param commentId
     * @return
     */
    @GetMapping("/studentCase/study/comment/admire")
    boolean admireComment(@RequestParam("commentId") Long commentId);

    /**
     * 评论取消点赞
     * @param commentId
     * @return
     */
    @GetMapping("/studentCase/study/comment/cancelAdmire")
    boolean cancelAdmireComment(@RequestParam("commentId") Long commentId);

    /**
     * 获取评论列表
     * @param studentCaseId
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/studentCase/study/comment/list")
    Page<CommentVO> getCommentList(@RequestParam("studentCaseId") Long studentCaseId,
                                 @RequestParam("pageNo") Integer pageNo,
                                 @RequestParam("pageSize") Integer pageSize);

    /**
     * 获取回复列表
     * @param commentId
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/studentCase/study/comment/replyList")
    Page<CommentReplyVO> getReplyList(@RequestParam("commentId") Long commentId,
                                      @RequestParam("pageNo") Integer pageNo,
                                      @RequestParam("pageSize") Integer pageSize);
}
