package com.yizhi.library.application.asynchronousTask;

import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.library.application.param.ExportAuditListParam;
import com.yizhi.library.application.util.Util;
import com.yizhi.library.application.vo.StudentCaseManageListVO;
import com.yizhi.library.application.service.IStudentCaseService;
import com.yizhi.system.application.system.remote.OrganizationClient;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author Ding
 * @className ExportAuditList
 * @description 导出待审核案例
 * @date 2019/7/29
 **/
@Component
public class ExportAuditList extends AbstractDefaultTask<Map, Map<String, Object>> {
    private static final Logger LOGGER = LoggerFactory.getLogger(ExportAuditList.class);

    @Autowired
    private IStudentCaseService studentCaseService;

    @Autowired
    private OrganizationClient organizationClient;

    @Autowired
    private Util util;

    @Override
    protected Map execute(Map<String, Object> map) {

        //从map中取信息
        RequestContext res = (RequestContext) map.get("res");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        Long taskId = (Long) map.get("taskId");
        ExportAuditListParam param = (ExportAuditListParam) map.get("param");

        //开始异步任务
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,
                res.getAccountId(), new Date(), res.getSiteId(), res.getCompanyId());
        working(taskContext);

        List<StudentCaseManageListVO> list = null;

        //获取导出数据
        try {
            list = studentCaseService.exportAuditList(param.getStudentCaseIds(), res);

            if (CollectionUtils.isEmpty(list)) {
                success(taskContext, "未找到数据", "");
            }
        } catch (Exception e) {
            LOGGER.error("获取数据过程中出现错误", e);
            fail(taskContext, "获取数据过程中出现错误");
        }


        //生成excel,上传阿里云返回url
        try {

            //获取案例活动及项目活动名称
            Map<String, Object> map1 = util.getCaseLibraryAndProjectName(null, param.getStudentCaseIds().get(0));

            String caseLibraryName = (String) map1.get("caseLibraryName");

            String trainingProjectName = (String) map1.get("trainingProjectName");


            //创建excel文件
            XSSFWorkbook wb = new XSSFWorkbook();
            //创建报表页
            XSSFSheet sheet = wb.createSheet("待审核案例列表");
            //创建表头
            XSSFRow row = sheet.createRow(0);


            //设置格式(水平垂直居中,加粗,字体大小14号)
            XSSFCellStyle style = wb.createCellStyle();
            style.setAlignment(HorizontalAlignment.CENTER);
            XSSFFont font = wb.createFont();
            font.setBold(true);
            font.setFontHeightInPoints((short) 14);
            style.setFont(font);


            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 7);
            //表头合并单元格
            sheet.addMergedRegion(cra);
            XSSFCell cell = row.createCell(0);

            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

            //填写表头名
            cell.setCellValue(caseLibraryName + "案例活动统计");

            //表头居中
            cell.setCellStyle(style);


            cra = new CellRangeAddress(1, 1, 0, 7);
            //第二行合并单元格合并单元格
            sheet.addMergedRegion(cra);
            row = sheet.createRow(1);
            row.createCell(0).setCellValue("所属项目: " + trainingProjectName);

            row = sheet.createRow(2);
            //重新设置格式(只加粗)
            style = wb.createCellStyle();
            font = wb.createFont();
            font.setBold(true);
            style.setFont(font);

            cell = row.createCell(0);
            cell.setCellStyle(style);
            cell.setCellValue("案例ID");

            cell = row.createCell(1);
            cell.setCellStyle(style);
            cell.setCellValue("案例标题");

            cell = row.createCell(2);
            cell.setCellStyle(style);
            cell.setCellValue("用户名");

            cell = row.createCell(3);
            cell.setCellStyle(style);
            cell.setCellValue("姓名");

            cell = row.createCell(4);
            cell.setCellStyle(style);
            cell.setCellValue("所在部门");

            cell = row.createCell(5);
            cell.setCellStyle(style);
            cell.setCellValue("提交时间");

            cell = row.createCell(6);
            cell.setCellStyle(style);
            cell.setCellValue("审核人");

            cell = row.createCell(7);
            style = wb.createCellStyle();
            font = wb.createFont();
            font.setBold(true);
            font.setColor(Font.COLOR_RED);
            style.setFont(font);
            cell.setCellStyle(style);
            cell.setCellValue("审核结果");


            //组装数据
            if (CollectionUtils.isNotEmpty(list)) {
                for (int i = 0; i < list.size(); i++) {
                    StudentCaseManageListVO listVO = list.get(i);

                    row = sheet.createRow(i + 3);
                    row.createCell(0).setCellValue(listVO.getStudentCaseId().toString());
                    row.createCell(1).setCellValue(listVO.getStudentCaseTitle());
                    row.createCell(2).setCellValue(listVO.getAccountName());
                    row.createCell(3).setCellValue(listVO.getAccountFullName() == null ?
                            "--" : listVO.getAccountFullName());

                    //获取部门及所有父级部门名称
                    List<String> orgNames = organizationClient.getChiefOrgNames(listVO.getOrgId(), res.getCompanyId());
                    if (CollectionUtils.isNotEmpty(orgNames)) {
                        row.createCell(4).setCellValue(Util.orgNamesToString(orgNames));
                    } else {
                        row.createCell(4).setCellValue("--");
                    }

                    SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy/M/dd HH:mm:ss");

                    row.createCell(5).setCellValue(sdf1.format(listVO.getSubmitTime()));

                    //审核人,评分人如果有多个则用逗号隔开
                    if (StringUtils.isNotBlank(listVO.getAuditorFullNames())) {
                        row.createCell(6).setCellValue(listVO.getAuditorFullNames());
                    } else {
                        row.createCell(6).setCellValue("--");
                    }
                }
            }

            String fileName = caseLibraryName + "待审核案例列表" + sdf.format(new Date()) + ".xlsx";
//            String requestPath = "/Users/dingxiaowei/aaa";
            String requestPath = FileConstant.SAVE_PATH;
            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }

            String path = requestPath + "/" + fileName;
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                wb.write(os);
                //阿里云返回url
                String uploadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功导出" + list.size() + "条记录", uploadUrl);
            } catch (Exception e2) {
                e2.printStackTrace();
                fail(taskContext, "导出待审核案例写入过程中发生错误");
                LOGGER.error("导出待审核案例写入过程中发生错误");
            } finally {
                if (os != null) {
                    os.close();
                }
                if (wb != null) {
                    wb.close();
                }
                if (file != null) {
                    file.delete();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "导出待审核案例下载发生错误");
            LOGGER.error("导出待审核案例下载发生错误，请查看日志");
        }


        return null;
    }
}
