package com.yizhi.library.application.asynchronousTask;

import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.library.application.domain.StudentCase;
import com.yizhi.library.application.vo.CommentVO;
import com.yizhi.library.application.service.ICommentService;
import com.yizhi.library.application.service.IStudentCaseService;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author Ding
 * @className ExportCommentList
 * @description TODO
 * @date 2019/7/16
 **/
@Component
public class ExportCommentList extends AbstractDefaultTask<Map, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExportCommentList.class);

    @Autowired
    private ICommentService commentService;

    @Autowired
    private IStudentCaseService studentCaseService;

    @Override
    protected Map execute(Map<String, Object> map) {

        //从map中取信息
        RequestContext res = (RequestContext) map.get("res");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        Long taskId = (Long) map.get("taskId");
        Long studentCaseId = (Long) map.get("studentCaseId");
        Integer state = (Integer) map.get("state");
        String name = (String) map.get("name");

        //开始异步任务
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,
                res.getAccountId(), new Date(), res.getSiteId(), res.getCompanyId());
        working(taskContext);

        List<CommentVO> commentVOList = new ArrayList<>();
        StudentCase studentCase = null;

        //获取导出数据
        try {
            studentCase = studentCaseService.selectById(studentCaseId);

            commentVOList = commentService.downloadList(res, studentCaseId, name, state);
        } catch (Exception e) {
            LOGGER.error("获取数据过程中出现错误", e);
            fail(taskContext, "获取数据过程中出现错误");
        }

        //生成excel,上传阿里云返回url
        try {
            if (null == studentCase || CollectionUtils.isEmpty(commentVOList)) {
                success(taskContext, "未找到数据", "");
            }


            //创建excel文件
            XSSFWorkbook wb = new XSSFWorkbook();
            //创建报表页
            XSSFSheet sheet = wb.createSheet("案例评论记录导出");
            //创建表头
            XSSFRow row = sheet.createRow(0);


            XSSFCellStyle style = wb.createCellStyle();
            style.setAlignment(HorizontalAlignment.CENTER);

            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 6);
            //表头合并单元格
            sheet.addMergedRegion(cra);
            XSSFCell cell = row.createCell(0);

            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

            //填写表头名
            cell.setCellValue(studentCase.getTitle() + "案例评论记录" + sdf.format(new Date()));

            //表头居中
            cell.setCellStyle(style);

            row = sheet.createRow(1);
            row.createCell(0).setCellValue("评论/回复人");
            row.createCell(1).setCellValue("姓名");
            row.createCell(2).setCellValue("评论对象");
            row.createCell(3).setCellValue("评论内容");
            row.createCell(4).setCellValue("提交时间");
            row.createCell(5).setCellValue("点赞数");
            row.createCell(6).setCellValue("状态");

            if (CollectionUtils.isNotEmpty(commentVOList)) {
                for (int i = 0; i < commentVOList.size(); i++) {
                    CommentVO commentVO = commentVOList.get(i);

                    row = sheet.createRow(i + 2);
                    row.createCell(0).setCellValue(commentVO.getCommenterUserName());
                    row.createCell(1).setCellValue(commentVO.getCommenterFullName());
                    row.createCell(2).setCellValue(commentVO.getReplyAccountFullName() == null ?
                            "--" : commentVO.getReplyAccountFullName());
                    row.createCell(3).setCellValue(commentVO.getContent());

                    SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy/M/dd HH:mm:ss");

                    row.createCell(4).setCellValue(sdf1.format(commentVO.getCreateTime()));
                    row.createCell(5).setCellValue(commentVO.getAdmireNum() == null ? "--" : commentVO.getAdmireNum().toString());
                    row.createCell(6).setCellValue(commentVO.getState() == 1 ? "已上架" : "已下架");
                }
            }

            String fileName = "案例评论记录" + sdf.format(new Date()) + ".xlsx";
//            String requestPath = "/Users/dingxiaowei/aaa";
            String requestPath = FileConstant.SAVE_PATH;
            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }

            String path = requestPath + "/" + fileName;
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                wb.write(os);
                //阿里云返回url
                String uploadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功导出" + commentVOList.size() + "条记录", uploadUrl);
            } catch (Exception e2) {
                e2.printStackTrace();
                fail(taskContext, "案例评论记录写入过程中发生错误");
                LOGGER.error("案例评论记录写入过程中发生错误");
            } finally {
                if (os != null) {
                    os.close();
                }
                if (wb != null) {
                    wb.close();
                }
                if (file != null) {
                    file.delete();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "案例评论记录下载发生错误");
            LOGGER.error("案例评论记录下载发生错误，请查看日志");
        }


        return null;
    }
}
