package com.yizhi.library.application.asynchronousTask;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.library.application.domain.CaseLibraryRelatedClassify;
import com.yizhi.library.application.util.Util;
import com.yizhi.library.application.vo.StudentCaseManageListVO;
import com.yizhi.library.application.vo.StudentRelatedClassifyVO;
import com.yizhi.library.application.service.IStudentCaseService;
import com.yizhi.system.application.system.remote.OrganizationClient;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author Ding
 * @className ExportStudentCaseManageList
 * @description 审核评分管理列表下载报表
 * @date 2019/7/29
 **/
@Component
public class ExportStudentCaseManageList extends AbstractDefaultTask<Map, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExportStudentCaseManageList.class);

    @Autowired
    private IStudentCaseService studentCaseService;

    @Autowired
    private OrganizationClient organizationClient;

    @Autowired
    private Util util;

    @Override
    protected Map execute(Map<String, Object> map) {

        //从map中取信息
        RequestContext res = (RequestContext) map.get("res");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        Long taskId = (Long) map.get("taskId");
        Long caseLibraryId = (Long) map.get("caseLibraryId");
        Integer state = (Integer) map.get("state");
        String studentCaseName = (String) map.get("studentCaseName");
        String accountName = (String) map.get("accountName");

        //开始异步任务
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,
                res.getAccountId(), new Date(), res.getSiteId(), res.getCompanyId());
        working(taskContext);

        List<StudentCaseManageListVO> list = null;

        //获取导出数据
        try {
            Integer pageNo = 1;
            Integer pageSize = Integer.MAX_VALUE;
            Page<StudentCaseManageListVO> page = studentCaseService.studentCaseManageList(caseLibraryId, studentCaseName, accountName,
                    state, null, pageNo, pageSize, res, 2);

            if (null != page) {
                list = page.getRecords();
            }

            if (CollectionUtils.isEmpty(list)) {
                success(taskContext, "未找到数据", "");
            }
        } catch (Exception e) {
            LOGGER.error("获取数据过程中出现错误", e);
            fail(taskContext, "获取数据过程中出现错误");
        }


        //生成excel,上传阿里云返回url
        try {

            //获取案例活动及项目活动名称
            Map<String, Object> map1 = util.getCaseLibraryAndProjectName(caseLibraryId, null);

            String caseLibraryName = (String) map1.get("caseLibraryName");

            String trainingProjectName = (String) map1.get("trainingProjectName");

            List<CaseLibraryRelatedClassify> relatedClassifyList = (List<CaseLibraryRelatedClassify>) map1.get("relatedClassify");

            //创建excel文件
            XSSFWorkbook wb = new XSSFWorkbook();
            //创建报表页
            XSSFSheet sheet = wb.createSheet("案例活动统计");
            //创建表头
            XSSFRow row = sheet.createRow(0);


            //设置格式(水平垂直居中,加粗,字体大小14号)
            XSSFCellStyle style = wb.createCellStyle();
            style.setAlignment(HorizontalAlignment.CENTER);
            XSSFFont font = wb.createFont();
            font.setBold(true);
            font.setFontHeightInPoints((short) 14);
            style.setFont(font);

            Integer n = 0;
            if (CollectionUtils.isNotEmpty(relatedClassifyList)) {
                n = relatedClassifyList.size();
            }


            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 8 + n);
            //表头合并单元格
            sheet.addMergedRegion(cra);
            XSSFCell cell = row.createCell(0);

            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

            //填写表头名
            cell.setCellValue(caseLibraryName + "案例活动统计");

            //表头居中
            cell.setCellStyle(style);


            cra = new CellRangeAddress(1, 1, 0, 8 + n);
            //第二行合并单元格合并单元格
            sheet.addMergedRegion(cra);
            row = sheet.createRow(1);
            row.createCell(0).setCellValue("所属项目: " + trainingProjectName);

            row = sheet.createRow(2);
            //重新设置格式(只加粗)
            style = wb.createCellStyle();
            font = wb.createFont();
            font.setBold(true);
            style.setFont(font);

            cell = row.createCell(0);
            cell.setCellStyle(style);
            cell.setCellValue("案例标题");

            cell = row.createCell(1);
            cell.setCellStyle(style);
            cell.setCellValue("用户名");

            cell = row.createCell(2);
            cell.setCellStyle(style);
            cell.setCellValue("姓名");

            cell = row.createCell(3);
            cell.setCellStyle(style);
            cell.setCellValue("所在部门");

            cell = row.createCell(4);
            cell.setCellStyle(style);
            cell.setCellValue("所在组织架构");

            cell = row.createCell(5);
            cell.setCellStyle(style);
            cell.setCellValue("提交时间");

            cell = row.createCell(6);
            cell.setCellStyle(style);
            cell.setCellValue("审核人");

            cell = row.createCell(7);
            cell.setCellStyle(style);
            cell.setCellValue("评分人");

            cell = row.createCell(8);
            cell.setCellStyle(style);
            cell.setCellValue("状态");

            if (n > 0) {
                for (int i = 0; i < n; i++) {
                    cell = row.createCell(9 + i);
                    cell.setCellStyle(style);
                    cell.setCellValue(relatedClassifyList.get(i).getClassifyName());
                }
            }


            //组装数据
            if (CollectionUtils.isNotEmpty(list)) {
                for (int i = 0; i < list.size(); i++) {
                    StudentCaseManageListVO listVO = list.get(i);

                    row = sheet.createRow(i + 3);
                    row.createCell(0).setCellValue(listVO.getStudentCaseTitle());
                    row.createCell(1).setCellValue(listVO.getAccountName());
                    row.createCell(2).setCellValue(listVO.getAccountFullName() == null ?
                            "--" : listVO.getAccountFullName());
                    row.createCell(3).setCellValue(listVO.getOrgName() == null ?
                            "--" : listVO.getOrgName());

                    //获取部门及所有父级部门名称
                    List<String> orgNames = organizationClient.getChiefOrgNames(listVO.getOrgId(), res.getCompanyId());
                    if (CollectionUtils.isNotEmpty(orgNames)) {
                        row.createCell(4).setCellValue(Util.orgNamesToString(orgNames));
                    } else {
                        row.createCell(4).setCellValue("--");
                    }

                    SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy/M/dd HH:mm:ss");

                    row.createCell(5).setCellValue(sdf1.format(listVO.getSubmitTime()));

                    //审核人,评分人如果有多个则用逗号隔开
                    if (StringUtils.isNotBlank(listVO.getAuditorFullNames())) {
                        row.createCell(6).setCellValue(listVO.getAuditorFullNames());
                    } else {
                        row.createCell(6).setCellValue("--");
                    }

                    if (StringUtils.isNotBlank(listVO.getScorerFullNames())) {
                        row.createCell(7).setCellValue(listVO.getScorerFullNames());
                    } else {
                        row.createCell(7).setCellValue("--");
                    }

                    row.createCell(8).setCellValue(listVO.getState() == null ? "--" : listVO.getState());

                    if (n > 0) {
                        List<StudentRelatedClassifyVO> studentRelatedClassifyVOS = listVO.getRelatedClassifyList();
                        for (int j = 0; j < n; j++) {
                            CaseLibraryRelatedClassify caseLibraryRelatedClassify = relatedClassifyList.get(j);
                            Long rootId = caseLibraryRelatedClassify.getClassifyId();
                            String name = null;
                            if (null!=rootId){
                                if (CollectionUtils.isNotEmpty(studentRelatedClassifyVOS)){
                                    for (StudentRelatedClassifyVO studentRelatedClassifyVO : studentRelatedClassifyVOS){
                                        if (studentRelatedClassifyVO.getRootId().equals(rootId)){
                                            name = studentRelatedClassifyVO.getClassifyName();
                                            break;
                                        }
                                    }
                                }
                            }
                            row.createCell(9 + j).setCellValue(name==null?"--":name);
                        }
                    }
                }
            }

            String fileName = caseLibraryName + "案例活动统计" + sdf.format(new Date()) + ".xlsx";
//            String requestPath = "/Users/dingxiaowei/aaa";
            String requestPath = FileConstant.SAVE_PATH;
            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }

            String path = requestPath + "/" + fileName;
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                wb.write(os);
                //阿里云返回url
                String uploadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功导出" + list.size() + "条记录", uploadUrl);
            } catch (Exception e2) {
                e2.printStackTrace();
                fail(taskContext, "案例活动统计写入过程中发生错误");
                LOGGER.error("案例活动统计写入过程中发生错误",e2);
            } finally {
                if (os != null) {
                    os.close();
                }
                if (wb != null) {
                    wb.close();
                }
                if (file != null) {
                    file.delete();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "案例活动统计下载发生错误");
            LOGGER.error("案例活动统计下载发生错误，请查看日志",e);
        }

        return null;
    }

}
