package com.yizhi.library.application.asynchronousTask;

import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.library.application.domain.StudentCase;
import com.yizhi.library.application.mapper.StudentCaseMapper;
import com.yizhi.library.application.service.IStudentCaseService;
import com.yizhi.library.application.util.DownloadOssUrl;
import com.yizhi.library.application.util.Util;
import org.apache.commons.collections.CollectionUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author Ding
 * @className ImportAuditList
 * @description 导入审核结果
 * @date 2019/7/29
 **/
@Component
public class ImportAuditList extends AbstractDefaultTask<Map, Map<String, Object>> {
    private static final Logger LOGGER = LoggerFactory.getLogger(ImportAuditList.class);

    @Autowired
    private IStudentCaseService studentCaseService;

    @Autowired
    private StudentCaseMapper studentCaseMapper;


    @Override
    protected Map execute(Map<String, Object> map) {
        RequestContext res = (RequestContext) map.get("res");
        Long taskId = (Long) map.get("taskId");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        String url = (String) map.get("url");

        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, res.getAccountId(), new Date(), res.getSiteId(), res.getCompanyId());
        working(taskContext);

        //解析url
        Workbook workbook = DownloadOssUrl.getWorkBook(url);

        if (null == workbook) {
            LOGGER.error("创建workbook对象失败");
            fail(taskContext, "创建workbook对象失败");
        }

        Sheet sheet = workbook.getSheetAt(0);
        Row row;
        Cell cell0;
        Cell cell7;

        int lastRowNum = sheet.getLastRowNum();


        List<StudentCase> studentCaseList = new ArrayList<>();
        Integer errNum = 0;
        for (int i = 3; i <= lastRowNum; i++) {
            row = sheet.getRow(i);
            cell0 = row.getCell(0);
            cell7 = row.getCell(7);
            Long id = null;
            Integer auditState = null;
            String result = null;

            if (null != cell0) {
                id = Util.getLongFromCell(cell0);
            }

            if (null != cell7) {
                result = Util.getStringFromCell(cell7);
            }

            if (null != id) {
                if (null != result) {
                    if (result.equals("通过")) {
                        auditState = 1;
                    } else if (result.equals("不通过")) {
                        auditState = 2;
                    }
                }
                if (null != auditState) {
                    StudentCase studentCase = new StudentCase();
                    studentCase.setId(id);
                    studentCase.setAuditState(auditState);
                    studentCaseList.add(studentCase);
                } else {
                    taskDetail(taskId, "第" + i + 1 + "行格式有误");
                    errNum++;
                }
            } else {
                taskDetail(taskId, "第" + i + 1 + "行格式有误");
                errNum++;
            }


        }
        Date date = new Date();
        //导入数据
        if (CollectionUtils.isNotEmpty(studentCaseList)) {
            //获取通过审核后上架规则
            Map<String, Object> map1 = studentCaseMapper.getExamine(studentCaseList.get(0).getId());
            Integer examine = (Integer) map1.get("examine");
            for (StudentCase studentCase : studentCaseList) {
                if (studentCase.getAuditState() == 1) {
                    if (examine == 1) {
                        studentCase.setState(2);
                    } else if (examine == 2) {
                        studentCase.setState(3);
                    }
                    studentCase.setScoreState(0);
                } else if (studentCase.getAuditState() == 2) {
                    studentCase.setState(3);
                }
                studentCase.setAuditTime(date);
            }
            studentCaseService.updateBatchById(studentCaseList);
            success(taskContext, "成功导入" + studentCaseList.size() + "条," + "失败" + errNum + "条", url);
        } else {
            success(taskContext, "成功导入0条", url);
        }


        return null;
    }
}
