package com.yizhi.library.application.asynchronousTask;

import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.library.application.domain.MustSubmit;
import com.yizhi.library.application.service.IMustSubmitService;
import com.yizhi.library.application.util.DownloadOssUrl;
import com.yizhi.library.application.vo.ImportVO;
import com.yizhi.system.application.model.AccountNameForIdsModel;
import com.yizhi.system.application.system.remote.AccountClient;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * @author Ding
 * @className ImportSubmitter
 * @description TODO
 * @date 2019/7/8
 **/

@Component
public class ImportSubmitter extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(ImportSubmitter.class);

    @Autowired
    private AccountClient accountClient;

    @Autowired
    private IMustSubmitService mustSubmitService;


    @Override
    protected String execute(Map<String, Object> map) {

        RequestContext res = (RequestContext) map.get("res");
        Long taskId = (Long) map.get("taskId");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        Long caseLibraryId = (Long) map.get("caseLibraryId");
        String url = (String) map.get("url");

        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, res.getAccountId(), new Date(), res.getSiteId(), res.getCompanyId());
        working(taskContext);

        //解析url
        Workbook workbook = DownloadOssUrl.getWorkBook(url);

        if (null == workbook) {
            LOGGER.error("创建workbook对象失败");
            fail(taskContext, "创建workbook对象失败");
        }

        Sheet sheet = workbook.getSheetAt(0);
        Row row;
        Cell cell;

        int lastRowNum = sheet.getLastRowNum();

        //需要获取用户id的name list
        List<String> names = new ArrayList<>();
        //需要获取用户id的name list带行数信息
        List<ImportVO> importVOList = new ArrayList<>();

        Integer errNum = 0;

        //格式有误的行数list
//        List<Integer> errorRowList = new ArrayList<>();
        //未查找到用户id的行数list
//        List<Integer> noIdRowList = new ArrayList<>();
        for (int i = 2; i <= lastRowNum; i++) {
            String userName = null;
            row = sheet.getRow(i);
            if (null != row) {
                //获取第一列
                cell = row.getCell(0);
                if (null != cell) {
                    if (cell.getCellTypeEnum().equals(CellType.STRING)) {
                        userName = cell.getStringCellValue();
                    } else if (cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
                        cell.setCellType(CellType.STRING);
                        userName = cell.getStringCellValue();
                    }
                    if (StringUtils.isNotBlank(userName)) {
                        names.add(userName.trim().toLowerCase());

                        ImportVO importVO = new ImportVO();
                        importVO.setRowNum(i + 1);
                        importVO.setName(userName.trim());
                        importVOList.add(importVO);
                    } else {
                        errNum++;
                        //内容为空或者格式有误
//                        errorRowList.add(i + 1);
                        Integer rowNum = i + 1;
                        taskDetail(taskId, "第" + rowNum + "行用户名格式有误");
                    }
                }
            }
        }

        Map<String, Long> nameIdMap = new HashMap<>();
        //根据用户名找出对应存在的用户id
        if (CollectionUtils.isNotEmpty(names)) {
            AccountNameForIdsModel model = new AccountNameForIdsModel(names, res);
            nameIdMap = accountClient.getAccountIdByNames(model, res.getCompanyId(), res.getSiteId(), "caseLibrary");
        } else {
            success(taskContext, "成功导入0条,未检索到用户", url);
        }

        //准备组装并导入的list
        List<MustSubmit> mustSubmitList = new ArrayList<>();
        //查询到用户id,组装数据准备导入
        if (null != nameIdMap && nameIdMap.size() > 0) {
            for (ImportVO importVO : importVOList) {
                Long accountId = nameIdMap.get(importVO.getName().toLowerCase());
                if (null != accountId) {
                    MustSubmit mustSubmit = new MustSubmit();
                    mustSubmit.setCaseLibraryId(caseLibraryId);
                    mustSubmit.setAccountId(accountId);
//                    mustSubmit.setState(1);
                    mustSubmit.setCreateById(res.getAccountId());
                    mustSubmit.setCreateByName(res.getAccountName());
                    mustSubmit.setCreateTime(new Date());
                    mustSubmit.setUpdateById(res.getAccountId());
                    mustSubmit.setUpdateByName(res.getAccountName());
                    mustSubmit.setUpdateTime(new Date());
                    mustSubmit.setCompanyId(res.getCompanyId());
                    mustSubmit.setSiteId(res.getSiteId());
                    mustSubmit.setOrgId(res.getOrgId());
                    mustSubmitList.add(mustSubmit);
                } else {
                    errNum++;
                    taskDetail(taskId, "第" + importVO.getRowNum() + "行未检索到用户");
                }
            }
            if (CollectionUtils.isNotEmpty(mustSubmitList)) {
                if (mustSubmitService.insertList(mustSubmitList)) {
                    success(taskContext, "成功导入" + mustSubmitList.size() + "条" + "失败" + errNum + "条", url);
                } else {
                    fail(taskContext, "导入失败,网络环境有误");
                }
            } else {
                success(taskContext, "成功导入0条,未检索到用户", url);
            }

        } else {
            success(taskContext, "成功导入0条,未检索到用户", url);
        }


        return null;
    }
}
