package com.yizhi.library.application.asynchronousTask;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.library.application.domain.CaseLibraryDimensionGrade;
import com.yizhi.library.application.mapper.CaseLibraryDimensionGradeMapper;
import com.yizhi.library.application.mapper.StudentCaseDimensionGradeRecordMapper;
import com.yizhi.library.application.service.IStudentCaseService;
import com.yizhi.library.application.vo.SelectionCaseListVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author Ding
 * @className SelectionListExport
 * @description 精选案例管理列表下载报表
 * @date 2019/7/30
 **/
@Component
public class SelectionListExport extends AbstractDefaultTask<Map, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(SelectionListExport.class);

    @Autowired
    private IStudentCaseService studentCaseService;
    @Autowired
    private CaseLibraryDimensionGradeMapper caseLibraryDimensionGradeMapper;
    @Autowired
    private StudentCaseDimensionGradeRecordMapper studentCaseDimensionGradeRecordMapper;

    @Override
    protected Map execute(Map<String, Object> map) {
        //从map中取信息
        RequestContext res = (RequestContext) map.get("res");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        Long taskId = (Long) map.get("taskId");
        String studentCaseName = (String) map.get("studentCaseName");
        String accountName = (String) map.get("accountName");
        Integer state = (Integer) map.get("state");

        //开始异步任务
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,
                res.getAccountId(), new Date(), res.getSiteId(), res.getCompanyId());
        working(taskContext);

        List<SelectionCaseListVO> list = null;

        //获取导出数据
        try {
            Integer pageNo = 1;
            Integer pageSize = Integer.MAX_VALUE;
            Page<SelectionCaseListVO> page = studentCaseService.getSelectionList(studentCaseName, accountName, state, pageNo, pageSize, res);
            if (null != page) {
                list = page.getRecords();
            }
            if (CollectionUtils.isEmpty(list)) {
                success(taskContext, "未找到数据", "");
            }
        } catch (Exception e) {
            LOGGER.error("获取数据过程中出现错误", e);
            fail(taskContext, "获取数据过程中出现错误");
        }

        //生成excel,上传阿里云返回url
        try {

            //创建excel文件
            XSSFWorkbook wb = new XSSFWorkbook();
            //创建报表页
            XSSFSheet sheet = wb.createSheet("精选案例清单");
            //创建表头
            XSSFRow row = sheet.createRow(0);


            //设置格式(水平垂直居中,加粗,字体大小14号)
            XSSFCellStyle style = wb.createCellStyle();
            style.setAlignment(HorizontalAlignment.CENTER);
            XSSFFont font = wb.createFont();
            font.setBold(true);
            font.setFontHeightInPoints((short) 14);
            style.setFont(font);


            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 6);
            //表头合并单元格
            sheet.addMergedRegion(cra);
            XSSFCell cell = row.createCell(0);

            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
            Date date = new Date();

            //填写表头名
            cell.setCellValue("精选案例清单" + sdf.format(date));

            //表头居中
            cell.setCellStyle(style);

            row = sheet.createRow(1);
            //重新设置格式(只加粗)
            style = wb.createCellStyle();
            font = wb.createFont();
            font.setBold(true);
            style.setFont(font);

            cell = row.createCell(0);
            cell.setCellStyle(style);
            cell.setCellValue("名称");

            cell = row.createCell(1);
            cell.setCellStyle(style);
            cell.setCellValue("所属案例活动");

            cell = row.createCell(2);
            cell.setCellStyle(style);
            cell.setCellValue("用户名");

            cell = row.createCell(3);
            cell.setCellStyle(style);
            cell.setCellValue("作者");

            cell = row.createCell(4);
            cell.setCellStyle(style);
            cell.setCellValue("提交时间");

            cell = row.createCell(5);
            cell.setCellStyle(style);
            cell.setCellValue("得分");

            cell = row.createCell(6);
            cell.setCellStyle(style);
            cell.setCellValue("状态");

            // 多维度打分
            Integer maxDimensionNum = caseLibraryDimensionGradeMapper.getMaxDimensionGradeNum(res.getSiteId());
            int indexNum = 0;
            if (maxDimensionNum != null) {
                maxDimensionNum = maxDimensionNum * 2;
                int x = 0;
                for (int i = 1; i <= maxDimensionNum; i++) {
                    x = i % 2;
                    cell = row.createCell(6 + i);
                    cell.setCellStyle(style);
                    if (x == 0) {
                        cell.setCellValue("纬度" + indexNum + "得分");
                    } else {
                        indexNum++;
                        cell.setCellValue("纬度" + indexNum + "名称");
                    }
                }
            }

            //组装数据
            if (org.apache.commons.collections.CollectionUtils.isNotEmpty(list)) {
                for (int i = 0; i < list.size(); i++) {
                    SelectionCaseListVO listVO = list.get(i);
                    LOGGER.info("listVO : +   "  + JSON.toJSONString(listVO));

                    row = sheet.createRow(i + 2);
                    row.createCell(0).setCellValue(listVO.getTittle());
                    row.createCell(1).setCellValue(listVO.getCaseLibraryName());
                    row.createCell(2).setCellValue(listVO.getAccountName());
                    row.createCell(3).setCellValue(listVO.getAccountFullName() == null ?
                            "--" : listVO.getAccountFullName());

                    SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy/M/dd HH:mm:ss");
                    row.createCell(4).setCellValue(sdf1.format(listVO.getSubmitTime()));

                    row.createCell(6).setCellValue(listVO.getState());

                    if (maxDimensionNum != null) {
                        // 多维度打分
                        CaseLibraryDimensionGrade grade = new CaseLibraryDimensionGrade();
                        grade.setDeleted(0);
                        grade.setCaseLibraryId(listVO.getCaseLibraryId());
                        EntityWrapper<CaseLibraryDimensionGrade> ew = new EntityWrapper<>(grade);
                        ew.orderBy("sort");
                        Double score;
                        Double allScore = 0.00;
                        CaseLibraryDimensionGrade caseLibraryDimensionGrade = null;
                        int x = 0;
                        int index = 0;
                        int num = 0;
                        int listIndex = 0;
                        if (listVO.getScoreRule() != null && listVO.getScoreRule() == 4) {
                            List<CaseLibraryDimensionGrade> dimensionGrades = caseLibraryDimensionGradeMapper.selectList(ew);
                            if (CollectionUtils.isNotEmpty(dimensionGrades)) {
                                num = dimensionGrades.size() * 2;
                                for (int j = 1; j <= num; j++) {
                                    x = j % 2;
                                    if (x == 1) {
                                        caseLibraryDimensionGrade = dimensionGrades.get(listIndex);
                                        listIndex++;
                                    }
                                    score = studentCaseDimensionGradeRecordMapper.getAverageScore(listVO.getAccountId(), caseLibraryDimensionGrade.getId(), listVO.getId());
                                    index = 6 + j;
                                    if (x == 0) {
                                        if (score != null) {
                                            allScore = allScore + score;
                                        }
                                        row.createCell(index).setCellValue(score == null ? "" : String.valueOf(NumberUtils.toScaledBigDecimal(score, 2, RoundingMode.HALF_UP)));
                                    } else {
                                        row.createCell(index).setCellValue(caseLibraryDimensionGrade.getName());
                                    }
                                }
                            }
                        }
                        row.createCell(5).setCellValue(allScore == null ? "--" : allScore.toString());
                    } else {
                        row.createCell(5).setCellValue(listVO.getScore() == null ? "--" : listVO.getScore().toString());
                    }
                }
            }

            String fileName = "精选案例清单" + sdf.format(new Date()) + ".xlsx";
//            String requestPath = "/Users/dingxiaowei/aaa";
            String requestPath = FileConstant.SAVE_PATH;
            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }

            String path = requestPath + "/" + fileName;
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                wb.write(os);
                //阿里云返回url
                String uploadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功导出" + list.size() + "条记录", uploadUrl);
            } catch (Exception e2) {
                e2.printStackTrace();
                fail(taskContext, "精选案例记录导出写入过程中发生错误");
                LOGGER.error("精选案例记录导出写入过程中发生错误");
            } finally {
                if (os != null) {
                    os.close();
                }
                if (wb != null) {
                    wb.close();
                }
                if (file != null) {
                    file.delete();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "精选案例记录导出下载发生错误");
            LOGGER.error("精选案例记录导出下载发生错误，请查看日志");
        }


        return null;
    }
}
