package com.yizhi.library.application.asynchronousTask;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.library.application.domain.StatisticStudentCase;
import com.yizhi.library.application.domain.StudentCase;
import com.yizhi.library.application.mapper.StudentCaseMapper;
import com.yizhi.library.application.service.IStatisticStudentCase;
import com.yizhi.library.application.service.IStatisticStudentCaseAccount;
import com.yizhi.library.application.service.IStudentCaseService;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.AccountVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.datasource.DataSourceUtils;
import org.springframework.stereotype.Component;

import javax.sql.DataSource;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class StatisticStudentCaseAbs extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(StatisticStudentCaseAbs.class);

    final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");

	final SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	@Autowired
	private IStatisticStudentCaseAccount statisticStudentCaseAccount;

	@Autowired
	private IdGenerator idGenerator;

	@Autowired
	private IStatisticStudentCase statisticStudentCase;

	@Autowired
	private AccountClient accountClient;

	@Autowired
	private IStudentCaseService studentCaseService;

	@Autowired
	private DataSource dataSource;

	@Autowired
	private  StudentCaseMapper studentCaseMapper;

	// 字段分割符
	private final static String FIELD_STERMINATED = "','";
	// 行分割符
	private final static String LINE_STERMINATED = "'\r\n'";

	private final static String FENGEFU = ","; // tab键
	private final static String HUANHANGFU = "\r\n"; // 换行

	private StringBuilder strBuilder = new StringBuilder();

	private final static String STUDENT_CASE_DETAIL = "(id,student_case_id,title,account_id,account_name,full_name,time,state,up_num,inform_num,study_num,company_id,site_id,org_id,create_time)";
	private final static String STUDENT_CASE_DETAIL_EXECUTE_SQL = "load data local infile ''  into table statistic_student_case_detail fields terminated by "
			+ FIELD_STERMINATED + "enclosed by '' escaped by 'N'" + " lines terminated by " + LINE_STERMINATED + " "
			+ STUDENT_CASE_DETAIL;

	@Override
	protected String execute(Map<String, Object> param) {
        Long taskId = idGenerator.generate();
        Long t1 = System.currentTimeMillis();
        /**
         * 走异步任务
         */
        Date curDateWork = new Date();
        TaskContext taskContext = new TaskContext(taskId, sim.format(curDateWork) + " 号案例报表分析前一天的数据", 999L, curDateWork);
        working(taskContext);
        String error=null;
        Integer dealSuccess = 0;
        List<String> listError = new ArrayList<String>();
		List<StudentCase> list = statisticStudentCaseAccount.getStudentCases();
		List<StatisticStudentCase> detailList = new ArrayList<StatisticStudentCase>();

		//每次跑批校正学员案例表中的点赞数(丁小伟加)
		studentCaseService.checkAdmireNum();

		for(StudentCase s:list) {
			try {
				StatisticStudentCase studentCase = new StatisticStudentCase();
				studentCase.setStudentCaseId(s.getId());
				EntityWrapper<StatisticStudentCase> wrapper = new EntityWrapper<StatisticStudentCase>(studentCase);


				statisticStudentCase.delete(wrapper);
	            taskDetail(taskId, "***** 案例统计 删除："+s.getId()+" 数据*****");
	            LOGGER.info("***** 案例统计删除："+s.getId()+" 数据*****");
	            if (s.getState()==1||s.getState()==2||s.getState()==3) {
					studentCase = statisticStudentCaseAccount.getStudentCase(s.getId());

					//每次跑批校正学员案例表中的点赞数(丁小伟加)
//		            studentCaseService.checkAdmireNum();
					Integer num = studentCaseMapper.getAdmireNum(s.getId());
					s.setAdmireNum(Long.valueOf(num));
					studentCaseService.updateById(s);

					taskDetail(taskId, "***** 案例统计开始："+s.getId()+" *****");
					LOGGER.info("***** 案例统计开始："+s.getId()+" *****");
					studentCase.setId(idGenerator.generate());
					
					studentCase.setCreateTime(curDateWork);
					if (studentCase.getAccountId()==null) {
						continue;
					}
					AccountVO accountVO = accountClient.findById(studentCase.getAccountId());
					studentCase.setAccountName(accountVO.getName());
					studentCase.setFullName(null==accountVO.getFullName()||""==accountVO.getFullName()?"——":accountVO.getFullName());
					detailList.add(studentCase);
					dealSuccess++;
				}
			} catch (Exception e) {
                LOGGER.info("案例按案例统计入库:" + s.getId() + "异常,跳过操作");
                error="案例按案例统计入库:" + s.getId() + "异常,跳过操作";
                listError.add(error);
				e.printStackTrace();
                taskDetail(taskId, "案例按案例统计入库处理过程中出现异常" + s.getId());
			}
		}
		insertBatchStatisticsCaseDetail(detailList);
        Long t2 = System.currentTimeMillis();
        Long t = (t2 - t1) / 1000;
        Integer count=dealSuccess+listError.size();
        success(taskContext, "执行成功,耗时:" + t + "s", "");
        taskDetail(taskId, "总数量:" + count + "。处理时间ms:" + (t2 - t1) + ",处理成功:" + dealSuccess + ",处理失败:" + listError.size());
        LOGGER.info("***** 案例按案例统计入库结束 ,数量:" + count + "。处理时间ms:" + (t2 - t1) + ",处理成功:" + dealSuccess + ",处理失败:" + listError.size() + " *****");
		return "success";
	}

	public boolean insertBatchStatisticsCaseDetail(List<StatisticStudentCase> listStatisticsCase) {

		Connection conn = null;
		InputStream in = null;
		PreparedStatement preStatement = null;
		com.mysql.jdbc.PreparedStatement mysqlPreStatement = null;
		try {
			// sql编译
			conn = DataSourceUtils.getConnection(dataSource);
			preStatement = conn.prepareStatement(STUDENT_CASE_DETAIL_EXECUTE_SQL);
			mysqlPreStatement = preStatement.unwrap(com.mysql.jdbc.PreparedStatement.class);
			in = switchStreamStatisticsCaseDetail(listStatisticsCase);
			mysqlPreStatement.setLocalInfileInputStream(in);
			// 执行
			preStatement.executeQuery();
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		} finally {
			// 大对象手动显示的设置为空保证GC时被释放
			if (in != null) {
				in = null;
			}

			try {
				// 关闭数据库连接
				if (mysqlPreStatement != null) {
					mysqlPreStatement.close();
				}
				if (preStatement != null) {
					preStatement.close();
				}
				if (conn != null) {
					DataSourceUtils.releaseConnection(conn, dataSource);
				}
			} catch (Exception e) {
				e.printStackTrace();
				return false;
			}
		}

		return true;
	}

	public InputStream switchStreamStatisticsCaseDetail(List<StatisticStudentCase> list) {
		for (StatisticStudentCase sc : list) {
			strBuilder.append(idGenerator.generate());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getStudentCaseId());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getTitle());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getAccountId());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getAccountName());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getFullName());
			strBuilder.append(FENGEFU);

			if (sc.getTime() != null) {
				strBuilder.append(format.format(sc.getTime()));
			}
			strBuilder.append(FENGEFU);
			strBuilder.append(sc.getState());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getUpNum());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getInformNum());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getStudyNum());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getCompanyId());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getSiteId());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getOrgId());
			strBuilder.append(FENGEFU);

			if (sc.getCreateTime() != null) {
				strBuilder.append(format.format(sc.getCreateTime()));
			}
			strBuilder.append(FENGEFU);
			strBuilder.append(HUANHANGFU);
		}
		byte[] bytes = strBuilder.toString().getBytes();
		InputStream in = new ByteArrayInputStream(bytes);

		// 大对象手动显示的设置为空保证GC时被释放
		bytes = null;

		// 清空之前的内容
		strBuilder.delete(0, strBuilder.length());

		return in;
	}
}
