package com.yizhi.library.application.asynchronousTask;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.library.application.domain.StatisticStudentCaseAccount;
import com.yizhi.library.application.domain.StatisticStudentCaseOrg;
import com.yizhi.library.application.service.IStatisticStudentCaseAccount;
import com.yizhi.library.application.service.IStatisticStudentCaseOrg;
import com.yizhi.system.application.system.remote.OrganizationClient;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.system.remote.SiteClient;
import com.yizhi.system.application.vo.ReportAccountRespVO;
import com.yizhi.system.application.vo.ReportRangeAccountReqVO;
import com.yizhi.system.application.vo.SiteVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.datasource.DataSourceUtils;
import org.springframework.stereotype.Component;

import javax.sql.DataSource;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.text.SimpleDateFormat;
import java.util.*;

@Component
public class StatisticStudentCaseAccountAbs extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(StatisticStudentCaseAccountAbs.class);

    final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");

    final SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Autowired
    private ReportClient reportClient;

    @Autowired
    private IStatisticStudentCaseAccount statisticStudentCaseAccount;

    @Autowired
    private OrganizationClient organizationClient;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private SiteClient siteClient;

    @Autowired
    private IStatisticStudentCaseOrg statisticStudentCaseOrg;

    @Autowired
    private DataSource dataSource;

    // 字段分割符
    private final static String FIELD_STERMINATED = "','";
    // 行分割符
    private final static String LINE_STERMINATED = "'\r\n'";

    private final static String FENGEFU = ","; // tab键
    private final static String HUANHANGFU = "\r\n"; // 换行

    private StringBuilder strBuilder = new StringBuilder();

    private final static String STUDENT_CASE_ACCOUNT = "(id,account_id,account_name,full_name,position,org_id,org_name,org_names,must_submit_num,submit_num,case_num,ups,up_num,informs,inform_num,study_num,status,audit_num,no_audit_num,no_score_num,score_num,create_time,company_id,site_id)";
    private final static String STUDENT_CASE_ACCOUNT_EXECUTE_SQL = "load data local infile ''  into table statistic_student_case_account fields terminated by "
            + FIELD_STERMINATED + "enclosed by '' escaped by 'N'" + " lines terminated by " + LINE_STERMINATED + " "
            + STUDENT_CASE_ACCOUNT;

    private final static String STUDENT_CASE_ORG = "(id,org_id,org_name,submit_num,audit_num,no_audit_num,no_score_num,score_num,case_num,up_num,study_num,create_time,company_id,site_id)";
    private final static String STUDENT_CASE_ORG_EXECUTE_SQL = "load data local infile ''  into table statistic_student_case_org fields terminated by "
            + FIELD_STERMINATED + "enclosed by '' escaped by 'N'" + " lines terminated by " + LINE_STERMINATED + " "
            + STUDENT_CASE_ORG;

    @Override
    protected String execute(Map<String, Object> param) {
        Long taskId = idGenerator.generate();
        Long t1 = System.currentTimeMillis();
        /**
         * 走异步任务
         */
        Date curDateWork = new Date();

        TaskContext taskContext = new TaskContext(taskId, sim.format(curDateWork) + " 号案例用户报表分析前一天的数据", 999L, curDateWork);
        working(taskContext);
        String error = null;
        Integer dealSuccess = 0;
        List<String> listError = new ArrayList<String>();
        List<Long> siteIds = statisticStudentCaseAccount.getSiteIds();
        List<StatisticStudentCaseAccount> accountList = new ArrayList<StatisticStudentCaseAccount>();
        for (Long siteId : siteIds) {
            StatisticStudentCaseAccount entity = new StatisticStudentCaseAccount();
            entity.setSiteId(siteId);
            EntityWrapper<StatisticStudentCaseAccount> wrapper = new EntityWrapper<StatisticStudentCaseAccount>(entity);
            statisticStudentCaseAccount.delete(wrapper);
            taskDetail(taskId, "***** 案例统计用户 删除：" + siteId + " 数据*****");
            LOGGER.info("***** 案例统计用户 删除：" + siteId + " 数据*****");
            ReportRangeAccountReqVO rangeAccountReqVO = new ReportRangeAccountReqVO();
            rangeAccountReqVO.setSiteId(siteId);
            List<ReportAccountRespVO> accounts = reportClient.getRangeAccounts(rangeAccountReqVO);
            Map<Long, String> orgNameMap = new HashMap<>(16);
            for (ReportAccountRespVO vo : accounts) {
                try {
                    entity = statisticStudentCaseAccount.getAccountExportVO(vo.getUserId(), siteId);
                    taskDetail(taskId, "***** 案例统计开始 用户：" + vo.getUserId() + " *****");
                    LOGGER.info("***** 案例统计开始 用户：" + vo.getUserId() + " *****");
                    entity.setId(idGenerator.generate());
                    entity.setAccountId(vo.getUserId());
                    entity.setAccountName(vo.getUserName());
                    entity.setFullName(null == vo.getUserFullName() || "" == vo.getUserFullName() ? "——" : vo.getUserFullName());
                    if (0 == vo.getStatus()) {
                        entity.setStatus("禁用");
                    } else {
                        entity.setStatus("启用");
                    }
                    entity.setPosition(null == vo.getPosition() || "" == vo.getPosition() ? "——" : vo.getPosition());
                    List<Long> orgIds = new ArrayList<Long>();
//					orgIds.add(vo.getOrgId());
//					Map<Long, String> orgName = organizationClient.getParentNameWithCurrentName(orgIds);
                    entity.setOrgId(vo.getOrgId());
                    entity.setOrgName(vo.getOrgName());
                    String orgNames = "";
                    if (null != orgNameMap.get(vo.getOrgId())) {
                        entity.setOrgNames(orgNameMap.get(vo.getOrgId()));
                    } else {
                        List<String> orgNameList = new ArrayList<>();
                        orgNameList = organizationClient.getChiefOrgNames(vo.getOrgId(), vo.getCompanyId());
                        if (CollectionUtils.isNotEmpty(orgNameList)) {
                            for (String s : orgNameList) {
                                if (StringUtils.isBlank(orgNames)) {
                                    orgNames = orgNames + s;
                                } else {
                                    orgNames = orgNames + "/" + s;
                                }
                            }
                            entity.setOrgNames(orgNames);
                            orgNameMap.put(vo.getOrgId(), orgNames);
                        }
                    }

                    entity.setSiteId(siteId);
                    entity.setCompanyId(vo.getCompanyId());
                    entity.setCreateTime(curDateWork);
                    accountList.add(entity);
                    dealSuccess++;
                } catch (Exception e) {
                    LOGGER.info("案例按用户统计入库:" + vo.getUserId() + "异常,跳过操作");
                    error = "案例按用户统计入库:" + vo.getUserId() + "异常,跳过操作";
                    listError.add(error);
                    e.printStackTrace();
                    taskDetail(taskId, "案例按用户统计处理过程中出现异常" + vo.getUserId());
                }
            }

        }
        insertBatchStatisticsCaseAccount(accountList);
        try {
            Thread.sleep(120000);
        } catch (InterruptedException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
        List<StatisticStudentCaseOrg> orgList = new ArrayList<StatisticStudentCaseOrg>();
        List<SiteVO> list = siteClient.selectByIds(siteIds);
        for (SiteVO vo : list) {
            StatisticStudentCaseOrg entity = new StatisticStudentCaseOrg();
            entity.setSiteId(vo.getId());
            EntityWrapper<StatisticStudentCaseOrg> wrapper = new EntityWrapper<StatisticStudentCaseOrg>(entity);
            statisticStudentCaseOrg.delete(wrapper);
            taskDetail(taskId, "***** 案例统计部门 删除：" + vo.getId() + " 数据*****");
            LOGGER.info("***** 案例统计部门删除：" + vo.getId() + " 数据*****");
            Set<Long> orgs = organizationClient.getOrgIdsInSite(vo.getId(), vo.getCompanyCode(), vo.getCode());
            for (Long orgId : orgs) {
                try {
                    entity = statisticStudentCaseAccount.getOrgExportVO(orgId, vo.getId());
                    taskDetail(taskId, "***** 案例统计开始 部门：" + orgId + " *****");
                    LOGGER.info("***** 案例统计开始 部门：" + orgId + " *****");
                    if (null != entity) {
                        entity.setId(idGenerator.generate());
                        entity.setCreateTime(curDateWork);
                        orgList.add(entity);
                        dealSuccess++;
                    }
                } catch (Exception e) {
                    LOGGER.info("案例按部门统计入库:" + orgId + "异常,跳过操作");
                    error = "案例按部门统计入库:" + orgId + "异常,跳过操作";
                    listError.add(error);
                    e.printStackTrace();
                    taskDetail(taskId, "案例按部门统计处理过程中出现异常" + orgId);
                }
            }
        }
        insertBatchStatisticsCaseOrg(orgList);
        Long t2 = System.currentTimeMillis();
        Long t = (t2 - t1) / 1000;
        Integer count = dealSuccess + listError.size();
        success(taskContext, "执行成功,耗时:" + t + "s", "");
        taskDetail(taskId, "总数量:" + count + "。处理时间ms:" + (t2 - t1) + ",处理成功:" + dealSuccess + ",处理失败:" + listError.size());
        LOGGER.info("***** 案例统计入库结束 ,数量:" + count + "。处理时间ms:" + (t2 - t1) + ",处理成功:" + dealSuccess + ",处理失败:" + listError.size() + " *****");
        return "success";
    }

    public boolean insertBatchStatisticsCaseAccount(List<StatisticStudentCaseAccount> listStatisticsCase) {

        Connection conn = null;
        InputStream in = null;
        PreparedStatement preStatement = null;
        com.mysql.jdbc.PreparedStatement mysqlPreStatement = null;
        try {
            // sql编译
            conn = DataSourceUtils.getConnection(dataSource);
            preStatement = conn.prepareStatement(STUDENT_CASE_ACCOUNT_EXECUTE_SQL);
            mysqlPreStatement = preStatement.unwrap(com.mysql.jdbc.PreparedStatement.class);
            in = switchStreamStatisticsCaseAccount(listStatisticsCase);
            mysqlPreStatement.setLocalInfileInputStream(in);
            // 执行
            preStatement.executeQuery();
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        } finally {
            // 大对象手动显示的设置为空保证GC时被释放
            if (in != null) {
                in = null;
            }

            try {
                // 关闭数据库连接
                if (mysqlPreStatement != null) {
                    mysqlPreStatement.close();
                }
                if (preStatement != null) {
                    preStatement.close();
                }
                if (conn != null) {
                    DataSourceUtils.releaseConnection(conn, dataSource);
                }
            } catch (Exception e) {
                e.printStackTrace();
                return false;
            }
        }

        return true;
    }

    public InputStream switchStreamStatisticsCaseAccount(List<StatisticStudentCaseAccount> list) {
        for (StatisticStudentCaseAccount sc : list) {
            strBuilder.append(idGenerator.generate());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getAccountId());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getAccountName());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getFullName());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getPosition());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getOrgId());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getOrgName());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getOrgNames());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getMustSubmitNum());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getSubmitNum());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getCaseNum());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getUps());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getUpNum());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getInforms());
            strBuilder.append(FENGEFU);
            strBuilder.append(sc.getInformNum());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getStudyNum());
            strBuilder.append(FENGEFU);
            strBuilder.append(sc.getStatus());
            strBuilder.append(FENGEFU);
            strBuilder.append(sc.getAuditNum());
            strBuilder.append(FENGEFU);
            strBuilder.append(sc.getNoAuditNum());
            strBuilder.append(FENGEFU);
            strBuilder.append(sc.getNoScoreNum());
            strBuilder.append(FENGEFU);
            strBuilder.append(sc.getScoreNum());
            strBuilder.append(FENGEFU);
            if (sc.getCreateTime() != null) {
                strBuilder.append(format.format(sc.getCreateTime()));
            }

            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getCompanyId());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getSiteId());
            strBuilder.append(FENGEFU);
            strBuilder.append(HUANHANGFU);

        }
        byte[] bytes = strBuilder.toString().getBytes();
        InputStream in = new ByteArrayInputStream(bytes);

        // 大对象手动显示的设置为空保证GC时被释放
        bytes = null;

        // 清空之前的内容
        strBuilder.delete(0, strBuilder.length());

        return in;
    }

    public boolean insertBatchStatisticsCaseOrg(List<StatisticStudentCaseOrg> listStatisticsCase) {

        Connection conn = null;
        InputStream in = null;
        PreparedStatement preStatement = null;
        com.mysql.jdbc.PreparedStatement mysqlPreStatement = null;
        try {
            // sql编译
            conn = DataSourceUtils.getConnection(dataSource);
            preStatement = conn.prepareStatement(STUDENT_CASE_ORG_EXECUTE_SQL);
            mysqlPreStatement = preStatement.unwrap(com.mysql.jdbc.PreparedStatement.class);
            in = switchStreamStatisticsCaseOrg(listStatisticsCase);
            mysqlPreStatement.setLocalInfileInputStream(in);
            // 执行
            preStatement.executeQuery();
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        } finally {
            // 大对象手动显示的设置为空保证GC时被释放
            if (in != null) {
                in = null;
            }

            try {
                // 关闭数据库连接
                if (mysqlPreStatement != null) {
                    mysqlPreStatement.close();
                }
                if (preStatement != null) {
                    preStatement.close();
                }
                if (conn != null) {
                    DataSourceUtils.releaseConnection(conn, dataSource);
                }
            } catch (Exception e) {
                e.printStackTrace();
                return false;
            }
        }

        return true;
    }

    public InputStream switchStreamStatisticsCaseOrg(List<StatisticStudentCaseOrg> list) {
        for (StatisticStudentCaseOrg sc : list) {
            strBuilder.append(idGenerator.generate());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getOrgId());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getOrgName());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getSubmitNum());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getAuditNum());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getNoAuditNum());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getNoScoreNum());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getScoreNum());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getCaseNum());
            strBuilder.append(FENGEFU);


            strBuilder.append(sc.getUpNum());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getStudyNum());
            strBuilder.append(FENGEFU);

            if (sc.getCreateTime() != null) {
                strBuilder.append(format.format(sc.getCreateTime()));
            }

            strBuilder.append(FENGEFU);


            strBuilder.append(sc.getCompanyId());
            strBuilder.append(FENGEFU);

            strBuilder.append(sc.getSiteId());
            strBuilder.append(FENGEFU);
            strBuilder.append(HUANHANGFU);

        }
        byte[] bytes = strBuilder.toString().getBytes();
        InputStream in = new ByteArrayInputStream(bytes);

        // 大对象手动显示的设置为空保证GC时被释放
        bytes = null;

        // 清空之前的内容
        strBuilder.delete(0, strBuilder.length());

        return in;
    }
}
