package com.yizhi.library.application.asynchronousTask;

import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.toolkit.StringUtils;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.library.application.util.DownloadOssUrl;
import com.yizhi.library.application.util.Util;
import com.yizhi.library.application.vo.AccessoryUrlVO;
import com.yizhi.library.application.vo.StudentCaseVO;
import com.yizhi.library.application.service.IStudentCaseService;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.util.application.zip.ZipUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author Ding
 * @className StudentCaseBatchExport
 * @description 学员案例审核/评分导出案例
 * @date 2019/7/23
 **/
@Component
public class StudentCaseBatchExport extends AbstractDefaultTask<Map, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(StudentCaseBatchExport.class);

    @Autowired
    private IStudentCaseService studentCaseService;

    @Autowired
    private AccountClient accountClient;

    @Autowired
    private Util util;

    @Override
    protected Map execute(Map<String, Object> map) {
        //从map中取信息
        RequestContext res = (RequestContext) map.get("res");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        Long taskId = (Long) map.get("taskId");
        List<Long> idList = (List<Long>) map.get("idList");

        //开始异步任务
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,
                res.getAccountId(), new Date(), res.getSiteId(), res.getCompanyId());
        working(taskContext);

        List<StudentCaseVO> studentCaseVOList = null;

        //获取导出数据
        try {
            studentCaseVOList = studentCaseService.studentCaseBatchExport(idList);
        } catch (Exception e) {
            LOGGER.error("获取数据过程中出现错误", e);
            fail(taskContext, "获取数据过程中出现错误");
        }

        //生成excel,上传阿里云返回url
        try {
            if (CollectionUtils.isEmpty(studentCaseVOList)) {
                success(taskContext, "未找到数据", "");
            }

            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
            //新建压缩文件夹
            String zipName = "案例批量导出" + sdf.format(new Date());
//            String requestPath = "/Users/dingxiaowei/aaa" + File.separator + zipName;
            String requestPath = FileConstant.SAVE_PATH + File.separator + zipName;
//            String requestPath = "C:\\Users\\lenovo\\Desktop"+ File.separator + zipName;

            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }

            //获取用户信息并放入map中
            Map<Long, AccountVO> accountVOMap = new HashMap<>(16);
            List<Long> accountIds = new ArrayList<>();
            for (StudentCaseVO studentCaseVO : studentCaseVOList) {
                if (!accountIds.contains(studentCaseVO.getAccountId())) {
                    accountIds.add(studentCaseVO.getAccountId());
                }
            }
            List<AccountVO> accountVOList = accountClient.idsGet(accountIds);
            if (CollectionUtils.isNotEmpty(accountVOList)) {
                for (AccountVO accountVO : accountVOList) {
                    accountVOMap.put(accountVO.getId(), accountVO);
                }
            }
            //得处一共有几个子文件
            int maxFileZipNumber= studentCaseVOList.size();
            for (StudentCaseVO studentCaseVO : studentCaseVOList) {
                //标识是第几个子文件
                int index=maxFileZipNumber;
                //如果有附件就下载到压缩文件夹中并压缩
                if (StringUtils.isNotEmpty(studentCaseVO.getAccessoryUrl())) {
                    List<AccessoryUrlVO> accessoryUrlVOList = util.getAccessoryUrlList(studentCaseVO.getId());
                    if (CollectionUtils.isNotEmpty(accessoryUrlVOList)) {
                        AccountVO accountVO = accountVOMap.get(studentCaseVO.getAccountId());
                        String name = accountVO.getFullName() == null ? accountVO.getName() : accountVO.getFullName();
                        //在压缩文件下新建该学生案例的文件夹
                        String studentCaseDir = requestPath + File.separator + name + studentCaseVO.getTitle() + sdf.format(new Date())+index;
                        File fileDir1 = new File(studentCaseDir);
                        if (!fileDir1.exists()) {
                            fileDir1.mkdir();
                        }
                        for (AccessoryUrlVO accessoryUrlVO : accessoryUrlVOList) {
                            DownloadOssUrl.downLoadFromUrl(accessoryUrlVO.getUrl(), accessoryUrlVO.getName(), studentCaseDir);
                        }
                        //子文件不压缩
                        String childPath = studentCaseDir + ".zip";
                        ZipUtil.zip(studentCaseDir, childPath);
                        Util.deleteDir(new File(studentCaseDir));
                    }
                }
                maxFileZipNumber--;
            }
            String zipPath = requestPath + ".zip";

            //压缩文件夹
            ZipUtil.zip(requestPath, zipPath);


            File file = null;
            File pathFile = null;
            try {
                //阿里云返回url
                String uploadUrl = OssUpload.upload(zipPath, zipName);
                success(taskContext, "导出成功", uploadUrl);
                file = new File(zipPath);
                pathFile = new File(requestPath);
            } catch (Exception e2) {
                e2.printStackTrace();
                fail(taskContext, "学员案例审核/评分导出案例写入过程中发生错误");
                LOGGER.error("学员案例审核/评分导出案例写入过程中发生错误");
            } finally {
                if (file != null) {
                    file.delete();
                }
                if (pathFile != null) {
                    Util.deleteDir(pathFile);
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "学员案例审核/评分导出案例下载发生错误");
            LOGGER.error("学员案例审核/评分导出案例下载发生错误，请查看日志");
        }


        return null;
    }
}
