package com.yizhi.library.application.asynchronousTask;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.library.application.domain.StatisticStudentCaseAccount;
import com.yizhi.library.application.service.IStatisticStudentCaseAccount;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class StudentCaseByAccountExport  extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger logger = LoggerFactory.getLogger(StudentCaseByAccountExport.class);

    @Autowired
    private IStatisticStudentCaseAccount statisticStudentCaseAccount;

	@Override
	protected String execute(Map<String, Object> arg0) {
		Long accountId=(Long) arg0.get("accountId");
		Long siteId=(Long) arg0.get("siteId");
		Long companyId=(Long) arg0.get("companyId");
		Long taskId=(Long) arg0.get("taskId");
		Date submitTime=(Date) arg0.get("submitTime");
		String serialNo=(String) arg0.get("serialNo");
		String taskName=(String) arg0.get("taskName");
		String kwd=(String) arg0.get("kwd");
		/**
		 * 走异步任务
		 */
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, submitTime, siteId, companyId);
		working(taskContext);
		String upLoadUrl=null;
		String requestPath= FileConstant.SAVE_PATH;
//		String requestPath = "/Users/dingxiaowei/aaa";
		File fileDir=new File(requestPath);
		if(!fileDir.exists()) {
			fileDir.mkdir();
		}
        if (StringUtils.isNotBlank(kwd)) {
        	kwd = kwd.trim();
        }
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
		SimpleDateFormat sf = new SimpleDateFormat("yyyyMMddHHmmss");
		StatisticStudentCaseAccount vo = new StatisticStudentCaseAccount();
		vo.setSiteId(siteId);
		EntityWrapper<StatisticStudentCaseAccount> wrapper = new EntityWrapper<StatisticStudentCaseAccount>(vo);
		wrapper.like("account_name", kwd).or().like("full_name", kwd);
		List<StatisticStudentCaseAccount> listVO = statisticStudentCaseAccount.selectList(wrapper);
		try {
			//excel生成过程: excel-->sheet-->row-->cell
			// 第一步，创建一个Excel文件
			XSSFWorkbook wb = new XSSFWorkbook();
			// 第二步，在webbook中添加一个sheet,对应Excel文件中的sheet
			XSSFSheet sheet = wb.createSheet();
			XSSFFont font = wb.createFont();
			font.setFontName("宋体");
			font.setFontHeightInPoints((short) 14);
			font.setBold(true);
			XSSFCellStyle style = wb.createCellStyle();
			style.setFont(font);
			style.setAlignment(HorizontalAlignment.CENTER); // 创建一个居中格式
			style.setVerticalAlignment(VerticalAlignment.CENTER);
			// 第三步，在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制short
			XSSFRow row = sheet.createRow((int) 0);
			row.setHeight((short) (20*20));
			// 合并单元格
			CellRangeAddress cra=new CellRangeAddress(0,0,0,11); // 起始行, 终止行, 起始列, 终止列
			sheet.addMergedRegion(cra);
			XSSFCell cell = row.createCell((short) 0);
			cell.setCellValue("案例统计报表——按用户统计");
			cell.setCellStyle(style);
			XSSFRow row1 = sheet.createRow((int) 1);
			row.setHeight((short) (20*20));
			// 合并单元格
			CellRangeAddress cra1=new CellRangeAddress(1,1,0,13); // 起始行, 终止行, 起始列, 终止列
			sheet.addMergedRegion(cra1);
			XSSFCell cell1 = row1.createCell((short) 0);
			cell1.setCellValue("统计周期：不限时间");
			// 第三步，在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制short
			XSSFRow row2 = sheet.createRow((int) 2);
			row2.setHeight((short) (14*20));
			// 第四步，创建单元格
			row2.createCell((short) 0).setCellValue("用户名");
			row2.createCell((short) 1).setCellValue("姓名");
			row2.createCell((short) 2).setCellValue("所在部门");
			row2.createCell((short) 3).setCellValue("所在组织架构");
			row2.createCell((short) 4).setCellValue("职务");
			row2.createCell((short) 5).setCellValue("提交数");
			row2.createCell((short) 6).setCellValue("必须提交数");
			row2.createCell((short) 7).setCellValue("精选案例数");
			row2.createCell((short) 8).setCellValue("点赞数");
			row2.createCell((short) 9).setCellValue("被点赞数");
			row2.createCell((short) 10).setCellValue("举报");
			row2.createCell((short) 11).setCellValue("被举报");
			row2.createCell((short) 12).setCellValue("学习次数");
			row2.createCell((short) 13).setCellValue("用户状态");
				for(int i=0;i<listVO.size();i++) {
					font.setBold(false);
					row = sheet.createRow(i + 3);
					row.setHeight((short) (13*20));
					// 第四步，创建单元格，并设置值
					vo=listVO.get(i);
					if(vo!=null&&vo.getAccountName()!=null) {
						row.createCell((short) 0).setCellValue(vo.getAccountName());
					}
					if(vo!=null&&vo.getFullName()!=null) {
						row.createCell((short) 1).setCellValue(vo.getFullName());
					}
					if(vo!=null&&StringUtils.isNotBlank(vo.getOrgName())) {
						row.createCell((short) 2).setCellValue(vo.getOrgName());
					}else {
						row.createCell((short) 2).setCellValue("--");
					}
					if(vo!=null&&StringUtils.isNotBlank(vo.getOrgNames())) {
						row.createCell((short) 3).setCellValue(vo.getOrgNames());
					}else {
						row.createCell((short) 3).setCellValue("--");
					}
					if(vo!=null&&vo.getPosition()!=null) {
						row.createCell((short) 4).setCellValue(vo.getPosition());
					}
					if(vo!=null&&vo.getSubmitNum()!=null) {
						row.createCell((short) 5).setCellValue(vo.getSubmitNum());
					}
					if(vo!=null&&vo.getMustSubmitNum()!=null) {
					    row.createCell((short) 6).setCellValue(vo.getMustSubmitNum());
					}
					if (vo!=null&&vo.getCaseNum()!=null) {
						row.createCell((short) 7).setCellValue(vo.getCaseNum());
					}
					if (vo!=null&&vo.getUps()!=null) {
					    row.createCell((short) 8).setCellValue(vo.getUps());
					}
					if (vo!=null&&vo.getUpNum()!=null) {
					    row.createCell((short) 9).setCellValue(vo.getUpNum());
					}
					if (vo!=null&&vo.getInforms()!=null) {
					    row.createCell((short) 10).setCellValue(vo.getInforms());
					}
					if (vo!=null&&vo.getInformNum()!=null) {
					    row.createCell((short) 11).setCellValue(vo.getInformNum());
					}
					if (vo!=null&&vo.getStudyNum()!=null) {
					    row.createCell((short) 12).setCellValue(vo.getStudyNum());
					}
					if (vo!=null&&vo.getStatus()!=null) {
						row.createCell((short) 13).setCellValue(vo.getStatus());
					}
				}

			// 第五步，写入实体数据 实际应用中这些数据从数据库得到，
			StringBuffer fileNameSb = new StringBuffer().append("案例统计报表——按用户统计").append(sf.format(submitTime)).append(".xlsx");
			String fileName = fileNameSb.toString();
			String path = new StringBuffer().append(requestPath).append(fileNameSb).toString();
			FileOutputStream os=null;
			File file=null;
			try {
				os= new FileOutputStream(path);
				wb.write(os);
				//阿里云返回url
				upLoadUrl = OssUpload.upload(path, fileName);
				file=new File(path);
				success(taskContext,"成功", upLoadUrl);
			} catch (Exception e1) {
				e1.printStackTrace();
				fail(taskContext, "写入数据到Excel的过程中或者上传到阿里云中发生错误"+e1.getMessage());
				logger.error("写入数据到Excel的过程中或者上传到阿里云中发生错误"+e1.getMessage());
			}
			finally {
				if(os!=null) {
				   os.close();
				}
				if(wb!=null) {
				   wb.close();
				}
				if(file!=null) {
				   file.delete();
				}
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			fail(taskContext, "评论信息导出过程中发生错误，请查看日志"+e.getMessage());
			logger.error("评论信息导出过程中发生错误，请查看日志"+e.getMessage());
		}
		return upLoadUrl;
	}
}
