package com.yizhi.library.application.controller;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.library.application.asynchronousTask.ImportAuditor;
import com.yizhi.library.application.asynchronousTask.ImportScorer;
import com.yizhi.library.application.asynchronousTask.ImportSubmitter;
import com.yizhi.library.application.domain.CaseLibrary;
import com.yizhi.library.application.feign.CaseLibraryClient;
import com.yizhi.library.application.param.RelateProjectParam;
import com.yizhi.library.application.service.ICaseLibraryRelatedClassifyService;
import com.yizhi.library.application.service.ICaseLibraryService;
import com.yizhi.library.application.vo.CaseLibraryVO;
import com.yizhi.library.application.vo.CaselibraryAuthorizeVO;
import com.yizhi.library.application.vo.ClassifyVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-07-03
 */
@RestController
@RequestMapping("/caseLibrary")
public class CaseLibraryController implements CaseLibraryClient {

    @Autowired
    private ICaseLibraryService caseLibraryService;

    @Autowired
    private ICaseLibraryRelatedClassifyService caseLibraryRelatedClassifyService;

    @Autowired
    private ImportSubmitter importSubmitter;

    @Autowired
    private ImportAuditor importAuditor;

    @Autowired
    private ImportScorer importScorer;

    @Autowired
    private IdGenerator idGenerator;

    @Override
    @GetMapping("/list")
    public Page<CaseLibraryVO> getCaseLibraryList(@RequestParam(name = "name", required = false) String name,
                                                  @RequestParam(name = "state", required = false) Integer state,
                                                  @RequestParam(name = "pageNo") Integer pageNo,
                                                  @RequestParam(name = "pageSize") Integer pageSize) {
        return caseLibraryService.getCaseLibraryList(name, state, pageNo, pageSize);
    }


    @Override
    @PostMapping("/insert")
    public Long insertCaseLibrary(@RequestBody CaseLibraryVO caseLibraryVO) {

        if (null == caseLibraryVO.getId()) {
            RequestContext res = ContextHolder.get();
            String name = caseLibraryVO.getName();
            CaseLibrary caseLibrary = new CaseLibrary();
            caseLibrary.setName(name);
            caseLibrary.setCompanyId(res.getCompanyId());
            caseLibrary.setSiteId(res.getSiteId());
            List<CaseLibrary> list = caseLibraryService.selectList(new EntityWrapper<>(caseLibrary));
            if (CollectionUtils.isNotEmpty(list)) {
                return 0L;
            }
        }

        //判断是否有导入
        boolean isImport = false;
        if (null != caseLibraryVO.getId()) {
            if (StringUtils.isNotBlank(caseLibraryVO.getAuditorUrl())) {
                isImport = true;
            } else if (StringUtils.isNotBlank(caseLibraryVO.getSubmitterUrl())) {
                isImport = true;
            } else if (StringUtils.isNotBlank(caseLibraryVO.getScorerUrl())) {
                isImport = true;
            }

        }
        if (isImport) {

            CaseLibrary caseLibrary = null;
            if (null != caseLibraryVO.getId()) {
                caseLibrary = caseLibraryService.selectById(caseLibraryVO.getId());
            }

            if (StringUtils.isNotBlank(caseLibraryVO.getAuditorUrl())) {
                // 如果原本没有附件,则导入
                if (null == caseLibraryVO.getId() || StringUtils.isBlank(caseLibrary.getAuditorUrl())) {
                    importAuditor(caseLibraryVO.getId(), caseLibraryVO.getAuditorUrl());
                    //原本有附件,且附件跟新建附件地址不一样,则导入
                } else if (!caseLibrary.getAuditorUrl().equals(caseLibraryVO.getAuditorUrl())) {
                    importAuditor(caseLibraryVO.getId(), caseLibraryVO.getAuditorUrl());
                }
            }

            if (StringUtils.isNotBlank(caseLibraryVO.getSubmitterUrl())) {
                // 如果原本没有附件,则导入
                if (null == caseLibraryVO.getId() || StringUtils.isBlank(caseLibrary.getSubmitterUrl())) {
                    importSubmitter(caseLibraryVO.getId(), caseLibraryVO.getSubmitterUrl());
                    //原本有附件,且附件跟新建附件地址不一样,则导入
                } else if (!caseLibrary.getSubmitterUrl().equals(caseLibraryVO.getSubmitterUrl())) {
                    importSubmitter(caseLibraryVO.getId(), caseLibraryVO.getSubmitterUrl());
                }
            }

            if (StringUtils.isNotBlank(caseLibraryVO.getScorerUrl())) {
                // 如果原本没有附件,则导入
                if (null == caseLibraryVO.getId() || StringUtils.isBlank(caseLibrary.getScorerUrl())) {
                    importScorer(caseLibraryVO.getId(), caseLibraryVO.getScorerUrl());
                    //原本有附件,且附件跟新建附件地址不一样,则导入
                } else if (!caseLibrary.getScorerUrl().equals(caseLibraryVO.getScorerUrl())) {
                    importScorer(caseLibraryVO.getId(), caseLibraryVO.getScorerUrl());
                }
            }
        }

        return caseLibraryService.insertCaseLibrary(caseLibraryVO);
    }

    @Override
    @GetMapping("/getClassify")
    public List<ClassifyVO> getClassify(@RequestParam(name = "caseLibraryId", required = false) Long caseLibraryId) {
        return caseLibraryService.getClassify(caseLibraryId);
    }

    @Override
    @GetMapping("/importAuditor")
    public String importAuditor(Long caseLibraryId, String url) {
        Map<String, Object> map = new HashMap<>(16);
        RequestContext res = ContextHolder.get();
        Long taskId = idGenerator.generate();
        String serialNo = "AUDITOR-IMPORT-" + taskId.toString();
        String taskName = "审核对象导入-" + taskId.toString();
        map.put("res", res);
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("caseLibraryId", caseLibraryId);
        map.put("url", url);
        importAuditor.execute(map, true);
        return serialNo;
    }

    @Override
    @GetMapping("/importScorer")
    public String importScorer(Long caseLibraryId, String url) {
        Map<String, Object> map = new HashMap<>(16);
        RequestContext res = ContextHolder.get();
        Long taskId = idGenerator.generate();
        String serialNo = "SCORER-IMPORT-" + taskId.toString();
        String taskName = "评分对象导入-" + taskId.toString();
        map.put("res", res);
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("caseLibraryId", caseLibraryId);
        map.put("url", url);
        importScorer.execute(map, true);
        return serialNo;
    }

    @Override
    @GetMapping("/importSubmitter")
    public String importSubmitter(Long caseLibraryId, String url) {

        Map<String, Object> map = new HashMap<>(16);
        RequestContext res = ContextHolder.get();
        Long taskId = idGenerator.generate();
        String serialNo = "SUBMITTER-IMPORT-" + taskId.toString();
        String taskName = "必须提交人导入-" + taskId.toString();
        map.put("res", res);
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("caseLibraryId", caseLibraryId);
        map.put("url", url);
        importSubmitter.execute(map, true);
        return serialNo;
    }

    @Override
    @GetMapping("/view")
    public CaseLibraryVO viewCaseLibrary(Long caseLibraryId) {
        return caseLibraryService.viewCaseLibrary(caseLibraryId);
    }

    @Override
    @PostMapping("/update")
    public boolean updateCaseLibrary(@RequestBody CaseLibraryVO caseLibraryVO) {
        return caseLibraryService.updateCaseLibrary(caseLibraryVO);
    }

    @Override
    @GetMapping("/relatedClassify/delete")
    public boolean deleteRelatedClassify(Long id) {
        return caseLibraryRelatedClassifyService.deleteById(id);
    }

    @Override
    @GetMapping("/release")
    public boolean releaseCaseLibrary(Long caseLibraryId) {
        return caseLibraryService.releaseCaseLibrary(caseLibraryId);
    }

    @Override
    @GetMapping("/putDown")
    public boolean putDownCaseLibrary(Long caseLibraryId) {
        return caseLibraryService.putDownCaseLibrary(caseLibraryId);
    }

    @Override
    @GetMapping("/delete")
    public boolean deleteCaseLibrary(Long caseLibraryId) {
        return caseLibraryService.deleteCaseLibrary(caseLibraryId);
    }

    @Override
    @PostMapping("/relateProject")
    public boolean relateProject(@RequestBody RelateProjectParam relateProjectParam) {
        return caseLibraryService.relateProject(relateProjectParam);
    }

    @Override
    @GetMapping("/cancelRelateProject")
    public boolean cancelRelateProject(Long caseLibraryId) {
        return caseLibraryService.cancelRelateProject(caseLibraryId);
    }

    @Override
    @GetMapping("/getProjectList")
    public Page<CaseLibraryVO> getProjectList(@RequestParam(value = "name", required = false) String name,
                                            @RequestParam("pageNo") Integer pageNo,
                                            @RequestParam("pageSize") Integer pageSize) {
        RequestContext res = ContextHolder.get();
        return caseLibraryService.getProjectList(name, res.getCompanyId(), res.getSiteId(), pageNo, pageSize);
    }

    @Override
    @PostMapping("/insertAuthorizes")
    public boolean insertAuthorizes(@RequestBody List<CaselibraryAuthorizeVO> caseLibraryAuthorizeList) {
        return caseLibraryService.insertAuthorizes(caseLibraryAuthorizeList);
    }

    @Override
    @GetMapping("/getAuthorize")
    public List<CaselibraryAuthorizeVO> getAuthorize(Long caselibraryId) {
        return caseLibraryService.getAuthorize(caselibraryId);
    }

    @Override
    @GetMapping("/getCaseLibrary")
    public CaseLibraryVO getCaseLibrary(Long caselibraryId) {
        CaseLibrary caselibrary = caseLibraryService.selectById(caselibraryId);
        CaseLibraryVO vo = new CaseLibraryVO();
        BeanUtils.copyProperties(caselibrary,vo);
        return vo;
    }

    @Override
    @GetMapping("/getHomeList")
    public Page<CaseLibraryVO> getHomeList(Integer pageNo, Integer pageSize) {
       return  caseLibraryService.getHomeList(pageNo, pageSize);
    }

    @Override
    @GetMapping("/getPageByDrools")
    public Page<DroolsVo> getPageByDrools(String field, String value, Integer pageNo, Integer pageSize) {
        Page<DroolsVo> page = new Page<>(pageNo, pageSize);
        return caseLibraryService.getPageByDrools(field, value, page);
    }
}

