package com.yizhi.library.application.controller;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.library.application.domain.Classify;
import com.yizhi.library.application.feign.CaseLibraryClassifyClient;
import com.yizhi.library.application.service.IClassifyService;
import com.yizhi.library.application.vo.ClassifyListVO;
import com.yizhi.library.application.vo.ClassifyVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-07-03
 */
@RestController
@RequestMapping("/classify")
public class ClassifyController implements CaseLibraryClassifyClient {

    @Autowired
    private IClassifyService classifyService;

    @Override
    @PostMapping("/insert")
    public Integer insertClassify(@RequestBody ClassifyVO classify) {
        return classifyService.insertClassify(classify);
    }

    @Override
    @GetMapping("/delete")
    public boolean deleteClassify(Long classifyId) {

        Classify classify = new Classify();
        classify.setId(classifyId);
        classify.setState(0);
        return classifyService.updateById(classify);
    }

    @Override
    @GetMapping("/putDown")
    public boolean putDownClassify(Long classifyId) {

        Classify classify = new Classify();
        classify.setId(classifyId);
        classify.setState(2);
        return classifyService.updateById(classify);
    }

    @Override
    @GetMapping("/release")
    public boolean releaseClassify(Long classifyId) {

        Classify classify = new Classify();
        classify.setId(classifyId);
        classify.setState(1);
        return classifyService.updateById(classify);
    }

    @Override
    @GetMapping("/list")
    public List<ClassifyListVO> getClassifyList() {
        return classifyService.getClassifyList();
    }

    @Override
    @GetMapping("/update")
    public boolean updateClassify(@RequestParam(name = "classifyId") Long classifyId,
                                  @RequestParam(name = "name", required = false) String name,
                                  @RequestParam(name = "description", required = false) String description) {
        RequestContext res = ContextHolder.get();
        Classify classify = classifyService.selectById(classifyId);
        if (StringUtils.isNotBlank(name)) {
            classify.setName(name.trim());
            //验证用户名是否重复
            Classify classify1 = new Classify();
            classify1.setParentId(classify.getParentId());
            classify1.setName(name);
            classify1.setSiteId(res.getSiteId());
            EntityWrapper<Classify> entityWrapper = new EntityWrapper<>(classify1);
            entityWrapper.notIn("id", classifyId);
            entityWrapper.notIn("state", 0);
            List<Classify> classifyList = classifyService.selectList(entityWrapper);
            if (CollectionUtils.isNotEmpty(classifyList)) {
                return false;
            }
        }
        if (StringUtils.isNotBlank(description)) {
            classify.setDescription(description.trim());
        }

        classify.setUpdateById(res.getAccountId());
        classify.setUpdateByName(res.getAccountName());
        classify.setUpdateTime(new Date());
        classifyService.updateById(classify);
        return true;
    }

    @Override
    @GetMapping("/move")
    public Integer moveClassify(@RequestParam(name = "classifyId") Long classifyId,
                                @RequestParam(name = "parentId") Long parentId,
                                @RequestParam(name = "sort") Integer sort,
                                @RequestParam(name = "type") Integer type) {
        RequestContext res = ContextHolder.get();
        //如果是第一个上移,直接不操作
        if (type == 1) {
            Classify preClassify = classifyService.getPreClassify(classifyId, parentId, sort, res.getCompanyId(), res.getSiteId());
            if (null == preClassify) {
                //已经是第一条,不能上移
                return 3;
            } else {
                Integer preSort = preClassify.getSort();
                preClassify.setSort(sort);
                preClassify.setUpdateById(res.getAccountId());
                preClassify.setUpdateByName(res.getAccountName());
                preClassify.setUpdateTime(new Date());
                if (!classifyService.updateById(preClassify)) {
                    //插入失败
                    return 2;
                }
                Classify classify = new Classify();
                classify.setId(classifyId);
                classify.setSort(preSort);
                classify.setUpdateById(res.getAccountId());
                classify.setUpdateByName(res.getAccountName());
                classify.setUpdateTime(new Date());
                if (classifyService.updateById(classify)) {
                    //插入成功
                    return 1;
                } else {
                    return 2;
                }
            }
        }
        //如果是最后一个下移也不操作
        if (type == 2) {

            Classify suffixClassify = classifyService.getSuffixClassify(classifyId, parentId, sort, res.getCompanyId(), res.getSiteId());

            if (null == suffixClassify) {
                //已经是最后一条,不能下移
                return 4;
            } else {
                //下一条的排序
                Integer suffixSort = suffixClassify.getSort();

                suffixClassify.setSort(sort);
                suffixClassify.setUpdateById(res.getAccountId());
                suffixClassify.setUpdateByName(res.getAccountName());
                suffixClassify.setUpdateTime(new Date());
                if (!classifyService.updateById(suffixClassify)) {
                    return 1;
                }
                Classify classify = new Classify();
                classify.setId(classifyId);
                classify.setSort(suffixSort);
                classify.setUpdateById(res.getAccountId());
                classify.setUpdateByName(res.getAccountName());
                classify.setUpdateTime(new Date());
                if (classifyService.updateById(classify)) {
                    return 1;
                } else {
                    return 2;
                }
            }
        }
        return 2;
    }

}

