package com.yizhi.library.application.controller;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.library.application.asynchronousTask.ExportCommentList;
import com.yizhi.library.application.feign.CommentManageClient;
import com.yizhi.library.application.vo.CommentReplyVO;
import com.yizhi.library.application.vo.CommentVO;
import com.yizhi.library.application.service.ICommentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Ding
 * @className CommentManageController
 * @description TODO
 * @date 2019/7/15
 **/
@RestController
@RequestMapping("/comment/manage")
public class CommentManageController implements CommentManageClient {

    @Autowired
    private ICommentService commentService;

    @Autowired
    private ExportCommentList exportCommentList;

    @Override
    @GetMapping("/list")
    public Page<CommentVO> getCommentManageList(@RequestParam(name = "studentCaseId", required = true) Long studentCaseId,
                                                @RequestParam(name = "name", required = false) String name,
                                                @RequestParam(name = "state", required = false) Integer state,
                                                @RequestParam("pageNo") Integer pageNo,
                                                @RequestParam("pageSize") Integer pageSize) {
        return commentService.getCommentManageList(studentCaseId, name, state, pageNo, pageSize);
    }

    @Override
    @GetMapping("/downloadList")
    public String downloadList(@RequestParam(name = "studentCaseId", required = true) Long studentCaseId,
                               @RequestParam(name = "name", required = false) String name,
                               @RequestParam(name = "state", required = false) Integer state) {

        Map<String, Object> map = new HashMap<>(16);
        RequestContext res = ContextHolder.get();
        Long taskId = System.currentTimeMillis();
        String serialNo = "EXPORT-COMMENT-LIST-" + taskId.toString();
        String taskName = "评论信息导出-" + taskId.toString();
        map.put("res", res);
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("studentCaseId", studentCaseId);
        map.put("name", name);
        map.put("state", state);
        exportCommentList.execute(map, true);
        return serialNo;
    }

    @Override
    @GetMapping("/replyList")
    public List<CommentReplyVO> getCommentReplyList(@RequestParam(name = "name", required = false) String name,
                                                    @RequestParam(name = "state", required = false) Integer state,
                                                    @RequestParam("commentId") Long commentId) {
        return commentService.getCommentReplyList(name, state, commentId);
    }

    @Override
    @GetMapping("/putDown")
    public boolean putDown(@RequestParam(name = "id", required = true) Long id,
                           @RequestParam(name = "type", required = true) Integer type) {
        return commentService.putDown(id, type);
    }

    @Override
    @GetMapping("/release")
    public boolean release(Long id, Integer type) {
        return commentService.release(id, type);
    }

    @Override
    @GetMapping("/delete")
    public boolean delete(@RequestParam(name = "id", required = true) Long id,
                          @RequestParam(name = "type", required = true) Integer type) {
        return commentService.deleteCommentOrReply(id, type);
    }
}
