package com.yizhi.library.application.controller;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.library.application.asynchronousTask.StudentCaseBatchExport;
import com.yizhi.library.application.constant.Constant;
import com.yizhi.library.application.domain.StudentCase;
import com.yizhi.library.application.domain.StudentCaseAuthorize;
import com.yizhi.library.application.domain.StudentCaseRelatedClassify;
import com.yizhi.library.application.feign.StudentCaseClient;
import com.yizhi.library.application.param.*;
import com.yizhi.library.application.service.*;
import com.yizhi.library.application.service.impl.StudentCaseServiceImpl;
import com.yizhi.library.application.vo.*;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.util.application.event.TrainingProjectEvent;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * <p>
 * 学生案例表 前端控制器
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-07-09
 */
@RestController
public class StudentCaseController implements StudentCaseClient {

    @Autowired
    private IStudentCaseService studentCaseService;

    @Autowired
    private StudentCaseBatchExport studentCaseBatchExport;

    @Autowired
    private AccountClient accountClient;

    @Autowired
    private IStudentCaseAuthorizeService studentCaseAuthorizeService;

    @Autowired
    private TaskExecutor taskExecutor;

    @Autowired
    private AmqpTemplate amqpTemplate;

    @Autowired
    private StudentCaseServiceImpl studentCaseServiceImpl;

    @Autowired
    private IStudentCaseRelatedClassifyService studentCaseRelatedClassifyService;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private IStudentCaseScoreRecordeService studentCaseScoreRecordeService;

    @Autowired
    private IStudentCaseAccessaryService studentCaseAccessaryService;

    @Override
    @GetMapping("/studentCase/myCaseLibrary/list")
    public Page<CaseLibraryVO> myCaseLibraryList(Integer pageNo, Integer pageSize) {
        return studentCaseService.myCaseLibraryList(pageNo, pageSize);
    }

    @Override
    public Integer myCaseLibraryTotal() {
        return studentCaseService.myCaseLibraryTotal();
    }

    @Override
    @GetMapping("/studentCase/submit/prepare")
    public StudentCaseVO prepareStudentCase(Long caseLibraryId) {
        return studentCaseService.prepareStudentCase(caseLibraryId);
    }

    @Override
    @PostMapping("/studentCase/submit")
    public boolean submitStudentCase(@RequestBody StudentCaseParam studentCaseParam) {
        return studentCaseService.submitStudentCase(studentCaseParam);
    }

    @Override
    @GetMapping("/studentCase/getSubmitRecords")
    public StudentCaseRecordListVO getSubmitRecords(Long caseLibraryId, String endTime) {
        return studentCaseService.getSubmitRecords(caseLibraryId, endTime);
    }

    @Override
    @GetMapping("/studentCase/record/studentCaseRecordSubmit")
    public boolean studentCaseRecordSubmit(Long studentCaseId) {

        RequestContext res = ContextHolder.get();

        Date date = new Date();

        StudentCase studentCase = new StudentCase();
        studentCase.setId(studentCaseId);
        studentCase.setState(1);
        studentCase.setAuditState(0);
        studentCase.setUpdateTime(date);
        studentCase.setUpdateById(res.getAccountId());
        studentCase.setUpdateByName(res.getAccountName());
        studentCase.setSubmitTime(date);

        studentCaseService.updateById(studentCase);
        studentCase = studentCaseService.selectById(studentCaseId);

//        //清除之前评分空记录
//        StudentCaseScoreRecorde studentCaseScoreRecorde = new StudentCaseScoreRecorde();
//        studentCaseScoreRecorde.setStudentCaseId(studentCase.getId());
//        studentCaseScoreRecordeService.delete(new EntityWrapper<>(studentCaseScoreRecorde));
//
//        //插入评分空记录
//        studentCaseServiceImpl.insertScorer(studentCase.getCaseLibraryId(), studentCaseId, studentCase.getAccountId());

        Long caseLibraryId = studentCase.getCaseLibraryId();
        Long accountId = res.getAccountId();
        Date now = new Date();
        Long siteId = res.getSiteId();
        taskExecutor.asynExecute(new AbstractTaskHandler() {

            @Override
            public void handle() {

                try {
                    amqpTemplate.convertAndSend(Constant.TRAINING_PROJECT_EVENT_QUEUE,
                            new EventWrapper<TrainingProjectEvent>(caseLibraryId,
                                    TrainingProjectEvent.getInstance(caseLibraryId, Constant.CASELIBRARY_TRANINNING_TYPE, accountId, now, siteId, false
                                    ))
                    );
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });

        return true;
    }

    @Override
    @GetMapping("/studentCase/record/studentCaseRecall")
    public boolean studentCaseRecall(Long studentCaseId) {
        return studentCaseService.studentCaseRecall(studentCaseId);
    }

    @Override
    @GetMapping("/studentCase/record/studentCaseView")
    public StudentCaseViewVO studentCaseView(@RequestParam("studentCaseId") Long studentCaseId,
                                             @RequestParam(name = "type", required = false) Integer type) {
        return studentCaseService.studentCaseView(studentCaseId, type);
    }

    @Override
    @GetMapping("/studentCase/getAuditList")
    public Page<StudentCaseStudyVO> getAuditList(@RequestParam("type") Integer type,
                                                 @RequestParam(value = "name", required = false) String name,
                                                 @RequestParam("pageNo") Integer pageNo,
                                                 @RequestParam("pageSize") Integer pageSize) {
        return studentCaseService.getAuditList(type, name, pageNo, pageSize);
    }

    @Override
    @GetMapping("/studentCase/studyStudentCase")
    public StudentCaseStudyVO studyStudentCase(@RequestParam("studentCaseId") Long studentCaseId,
                                               @RequestParam("type") Integer type,
                                               @RequestParam(name = "number", required = false) Integer number,
                                               @RequestParam(name = "name", required = false) String name) {
        return studentCaseService.studyStudentCase(studentCaseId, type, number, name);
    }

    @Override
    @PostMapping("/studentCase/audit")
    public boolean studentCaseAudit(@RequestBody StudentCaseAuditParam studentCaseAuditParam) {
        return studentCaseService.studentCaseAudit(studentCaseAuditParam);
    }

    @Override
    @PostMapping("/studentCase/batchAudit")
    public boolean studentCaseBatchAudit(@RequestBody StudentCaseAuditParam studentCaseAuditParam) {
        return studentCaseService.batchAudit(studentCaseAuditParam);
    }

    @Override
    @PostMapping("/studentCase/score")
    public boolean studentCaseScore(@RequestBody StudentCaseAuditParam studentCaseAuditParam) {
        return studentCaseService.studentCaseScore(studentCaseAuditParam);
    }

    @Override
    @PostMapping("/studentCase/batchScore")
    public boolean studentCaseBatchScore(@RequestBody StudentCaseAuditParam studentCaseAuditParam) {
        return studentCaseService.batchScore(studentCaseAuditParam);
    }

    @Override
    @PostMapping("/studentCase/batchExport")
    public String studentCaseBatchExport(@RequestBody StudentCaseAuditParam studentCaseAuditParam) {

        Map<String, Object> map = new HashMap<>(16);
        RequestContext res = ContextHolder.get();
        Long taskId = System.currentTimeMillis();
        String serialNo = "AUDIT-OR-SCORE-BATCH-EXPORT-" + taskId.toString();
        String taskName = "学员案例审核/评分导出案例" + taskId.toString();
        map.put("res", res);
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("idList", studentCaseAuditParam.getIdList());
        studentCaseBatchExport.execute(map, true);
        return serialNo;
    }

    @Override
    @PostMapping("/studentCase/homeList")
    public Page<StudentCaseVO> getStudentCaseHomeList(@RequestBody HomeListParam homeListParam) {
        return studentCaseService.getStudentCaseHomeList(homeListParam);
    }

    @Override
    @GetMapping("/studentCase/getStudentCaseDetails")
    public StudentCaseStudyVO getStudentCaseDetails(@RequestParam("studentCaseId") Long studentCaseId) {

        StudentCase studentCase = studentCaseService.selectById(studentCaseId);
        StudentCaseStudyVO studyVO = new StudentCaseStudyVO();
        AccountVO accountVO = accountClient.findById(studentCase.getAccountId());

        if (null != accountVO) {
            studyVO.setAuthorName(accountVO.getFullName());
        }
        studyVO.setStudentCaseId(studentCase.getId());
        studyVO.setTitle(studentCase.getTitle());
        studyVO.setLogoUrl(studentCase.getLogoUrl());
        studyVO.setCreateTime(studentCase.getCreateTime());

        return studyVO;

    }

    @Override
    public boolean insertAuthorizes(@RequestBody List<StudentCaseAuthorizeVO> studentCaseAuthorizeList) {
        return studentCaseAuthorizeService.insertAuthorize(studentCaseAuthorizeList);
    }

    @Override
    @GetMapping("/studentCase/getAuthorize")
    public List<StudentCaseAuthorizeVO> getAuthorize(Long studentCaseId) {
        return studentCaseAuthorizeService.getAuthorize(studentCaseId);
    }

    @Override
    @GetMapping("/studentCase/getFavorites")
    public Page<FavoriteVO> getFavorites(@RequestParam("pageNo") Integer pageNo,
                                         @RequestParam("pageSize") Integer pageSize,
                                         @RequestParam(value = "name", required = false) String name) {
        return studentCaseService.getFavorites(pageNo, pageSize, name);
    }

    @Override
    public Integer getFavoritesCount(Long companyId, Long siteId, Long accountId) {
        return studentCaseService.getFavoritesCount(companyId, siteId, accountId);
    }

    @Override
    @GetMapping("/studentCase/getProjectList")
    public Page<StudentCaseStudyVO> getProjectList(@RequestParam(value = "name", required = false) String name, Integer pageNo, Integer pageSize) {
        return studentCaseService.getProjectList(name, pageNo, pageSize);
    }

    @Override
    public Page<FavoriteVO> getSelectionStudentCaseList(@RequestBody SelectionStudentCaseParam selectionStudentCaseParam) {
        return studentCaseService.getSelectionStudentCaseList(selectionStudentCaseParam);
    }

    @Override
    public FavoriteVO getStudentCaseDetail(Long studentCaseId) {
        return studentCaseService.getStudentCaseDetail(studentCaseId);
    }

    @Override
    public List<StudentCaseAuthorizeVO> VisibleRange(Long studentCaseId) {
        StudentCaseAuthorize studentCaseAuthorize = new StudentCaseAuthorize();
        studentCaseAuthorize.setStudentCaseId(studentCaseId);
        studentCaseAuthorize.setState(1);
        EntityWrapper<StudentCaseAuthorize> wrapper = new EntityWrapper<StudentCaseAuthorize>(studentCaseAuthorize);
        List<StudentCaseAuthorize> list = studentCaseAuthorizeService.selectList(wrapper);

        List<StudentCaseAuthorizeVO> vos = new ArrayList<>();
        for (StudentCaseAuthorize studentcaseauthorize : list) {
            StudentCaseAuthorizeVO vo = new StudentCaseAuthorizeVO();
            BeanUtils.copyProperties(studentcaseauthorize,vo);
            vos.add(vo);
        }
        return vos;
    }

    @Override
    public boolean modifyRlatedClassifys(@RequestBody UpdateCaseRelatedClassifyParam param) {
        RequestContext res = ContextHolder.get();
        List<StudentCaseRelatedClassifyVO> list = param.getRelatedClassifyList();
        if (CollectionUtils.isNotEmpty(list)) {
            for (StudentCaseRelatedClassifyVO relatedClassify : list) {
                relatedClassify.setId(idGenerator.generate());
                relatedClassify.setStudentCaseId(param.getStudentCaseId());
                relatedClassify.setCreateById(res.getAccountId());
                relatedClassify.setCreateByName(res.getAccountName());
                relatedClassify.setCreateTime(new Date());
                relatedClassify.setUpdateById(res.getAccountId());
                relatedClassify.setUpdateByName(res.getAccountName());
                relatedClassify.setUpdateTime(new Date());
                relatedClassify.setCompanyId(res.getCompanyId());
                relatedClassify.setSiteId(res.getSiteId());
                relatedClassify.setOrgId(res.getOrgId());
            }
            //先清除原来记录,再插入记录
            StudentCaseRelatedClassify studentCaseRelatedClassify1 = new StudentCaseRelatedClassify();
            studentCaseRelatedClassify1.setStudentCaseId(param.getStudentCaseId());
            studentCaseRelatedClassifyService.delete(new EntityWrapper<>(studentCaseRelatedClassify1));

            List<StudentCaseRelatedClassify> vos = new ArrayList<>();
            for (StudentCaseRelatedClassifyVO studentCaseRelatedClassifyVO : list) {
                StudentCaseRelatedClassify vo = new StudentCaseRelatedClassify();
                BeanUtils.copyProperties(studentCaseRelatedClassifyVO,vo);
                vos.add(vo);
            }
            studentCaseRelatedClassifyService.insertBatch(vos);
            return true;
        }
        return false;
    }

    @Override
    public boolean verifyVedio(@RequestBody List<Long> ids) {
        return studentCaseAccessaryService.verifyVedio(ids);
    }

    @Override
    public boolean updateTranscodeUrl(@RequestBody Map<String, String> map) {
        return studentCaseAccessaryService.updateTranscodeUrl(map);
    }

    /**
     * 搜素下拉框
     * @param field
     * @param value
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    public Page<DroolsVo> getPageByDrools(String field, String value, Integer pageNo, Integer pageSize) {
        Page<DroolsVo> page = new Page<>(pageNo, pageSize);
        return studentCaseService.getPageByDrools(field, value, page);
    }
}

