package com.yizhi.library.application.controller;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.library.application.asynchronousTask.*;
import com.yizhi.library.application.domain.StudentCase;
import com.yizhi.library.application.feign.StudentCaseManageClient;
import com.yizhi.library.application.param.ExportAuditListParam;
import com.yizhi.library.application.service.IInformRecordeService;
import com.yizhi.library.application.service.IStudentCaseService;
import com.yizhi.library.application.vo.SelectionCaseListVO;
import com.yizhi.library.application.vo.StudentCaseManageListVO;
import com.yizhi.library.application.vo.StudentCaseStudyVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;

/**
 * @author Ding
 * @className StudentCaseManageController
 * @description TODO
 * @date 2019/7/25
 **/

@RestController
@RequestMapping("/manage/studentCase")
public class StudentCaseManageController implements StudentCaseManageClient {

    @Autowired
    private IStudentCaseService studentCaseService;

    @Autowired
    private ExportStudentCaseManageList exportStudentCaseManageList;

    @Autowired
    private ExportAuditList exportAuditList;

    @Autowired
    private ImportAuditList importAuditList;

    @Autowired
    private ExportScoreList exportScoreList;

    @Autowired
    private ImportScoreList importScoreList;

    @Autowired
    private SelectionListExport selectionListExport;

    @Autowired
    private InformListExport informListExport;

    @Autowired
    private IInformRecordeService informRecordeService;

    @Override
    @GetMapping("/list")
    public Page<StudentCaseManageListVO> studentCaseManageList(@RequestParam(name = "caseLibraryId", required = false) Long caseLibraryId,
                                                               @RequestParam(name = "studentCaseName", required = false) String studentCaseName,
                                                               @RequestParam(name = "accountName", required = false) String accountName,
                                                               @RequestParam(name = "state", required = false) Integer state,
                                                               @RequestParam(name = "orgName", required = false) String orgName,
                                                               @RequestParam(name = "pageNo", required = true) Integer pageNo,
                                                               @RequestParam(name = "pageSize", required = true) Integer pageSize) {

        RequestContext res = ContextHolder.get();

        return studentCaseService.studentCaseManageList(caseLibraryId, studentCaseName, accountName, state, orgName, pageNo, pageSize, res, 1);
    }

    @Override
    @GetMapping("/auditOrScore")
    public StudentCaseStudyVO studentCaseManageAuditOrScoreView(@RequestParam(name = "studentCaseId", required = true) Long studentCaseId,
                                                                @RequestParam(name = "caseLibraryId", required = false) Long caseLibraryId,
                                                                @RequestParam(name = "studentCaseName", required = false) String studentCaseName,
                                                                @RequestParam(name = "accountName", required = false) String accountName,
                                                                @RequestParam(name = "state", required = false) Integer state,
                                                                @RequestParam(name = "orgName", required = false) String orgName,
                                                                @RequestParam(name = "number", required = false) Integer number) {

        return studentCaseService.studentCaseManageAuditOrScoreView(studentCaseId, caseLibraryId, studentCaseName, accountName, state, orgName, number);

    }

    @Override
    @GetMapping("/list/export")
    public String exportStudentCaseManageList(@RequestParam(name = "caseLibraryName", required = false) String caseLibraryName,
                                              @RequestParam(name = "trainingProjectName", required = false) String trainingProjectName,
                                              @RequestParam(name = "caseLibraryId", required = true) Long caseLibraryId,
                                              @RequestParam(name = "studentCaseName", required = false) String studentCaseName,
                                              @RequestParam(name = "accountName", required = false) String accountName,
                                              @RequestParam(name = "state", required = false) Integer state) {

        Map<String, Object> map = new HashMap<>(16);
        RequestContext res = ContextHolder.get();
        Long taskId = System.currentTimeMillis();
        String serialNo = "STUDENT-CASE-MANAGE-EXPORT-" + taskId.toString();
        String taskName = "审核评分管理列表下载报表-" + taskId.toString();
        map.put("res", res);
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("caseLibraryId", caseLibraryId);
        map.put("caseLibraryName", caseLibraryName);
        map.put("trainingProjectName", trainingProjectName);
        map.put("studentCaseName", studentCaseName);
        map.put("accountName", accountName);
        map.put("state", state);

        exportStudentCaseManageList.execute(map, true);
        return serialNo;
    }

    @Override
    @PostMapping("/auditList/export")
    public String exportAuditList(@RequestBody ExportAuditListParam param) {
        Map<String, Object> map = new HashMap<>(16);
        RequestContext res = ContextHolder.get();
        Long taskId = System.currentTimeMillis();
        String serialNo = "STUDENT-CASE-AUDIT-EXPORT-" + taskId.toString();
        String taskName = "导出待审核案例-" + taskId.toString();
        map.put("res", res);
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("param", param);

        exportAuditList.execute(map, true);
        return serialNo;
    }

    @Override
    @GetMapping("/auditList/import")
    public String importAuditList(@RequestParam("ossUrl") String ossUrl) {
        Map<String, Object> map = new HashMap<>(16);
        RequestContext res = ContextHolder.get();
        Long taskId = System.currentTimeMillis();
        String serialNo = "STUDENT-CASE-AUDIT-IMPORT" + taskId.toString();
        String taskName = "导入审核结果-" + taskId.toString();
        map.put("res", res);
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("url", ossUrl);

        importAuditList.execute(map, true);
        return serialNo;
    }

    @Override
    @PostMapping("/scoreList/export")
    public String exportScoreList(@RequestBody ExportAuditListParam param) {
        Map<String, Object> map = new HashMap<>(16);
        RequestContext res = ContextHolder.get();
        Long taskId = System.currentTimeMillis();
        String serialNo = "STUDENT-CASE-SCORE-EXPORT-" + taskId.toString();
        String taskName = "导出待评分案例-" + taskId.toString();
        map.put("res", res);
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("param", param);

        exportScoreList.execute(map, true);
        return serialNo;
    }

    @Override
    @GetMapping("/scoreList/import")
    public String importScoreList(@RequestParam("ossUrl") String ossUrl) {
        Map<String, Object> map = new HashMap<>(16);
        RequestContext res = ContextHolder.get();
        Long taskId = System.currentTimeMillis();
        String serialNo = "STUDENT-CASE-SCORE-IMPORT-" + taskId.toString();
        String taskName = "导入评分结果-" + taskId.toString();
        map.put("res", res);
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("url", ossUrl);

        importScoreList.execute(map, true);
        return serialNo;
    }

    @Override
    @GetMapping("/selectionList")
    public Page<SelectionCaseListVO> getSelectionList(@RequestParam(name = "studentCaseName", required = false) String studentCaseName,
                                                      @RequestParam(name = "accountName", required = false) String accountName,
                                                      @RequestParam(name = "state", required = false) Integer state,
                                                      @RequestParam("pageNo") Integer pageNo,
                                                      @RequestParam("pageSize") Integer pageSize) {
        RequestContext res = ContextHolder.get();
        return studentCaseService.getSelectionList(studentCaseName, accountName, state, pageNo, pageSize, res);
    }

    @Override
    @GetMapping("/selectionList/export")
    public String selectionListExport(@RequestParam(name = "studentCaseName", required = false) String studentCaseName,
                                      @RequestParam(name = "accountName", required = false) String accountName,
                                      @RequestParam(name = "state", required = false) Integer state) {
        Map<String, Object> map = new HashMap<>(16);
        RequestContext res = ContextHolder.get();
        Long taskId = System.currentTimeMillis();
        String serialNo = "SELECTION-LIST-EXPORT-" + taskId.toString();
        String taskName = "精选案例管理列表下载报表-" + taskId.toString();
        map.put("res", res);
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("studentCaseName", studentCaseName);
        map.put("accountName", accountName);
        map.put("state", state);

        selectionListExport.execute(map, true);
        return serialNo;
    }

    @Override
    @GetMapping("/informList")
    public Page<StudentCaseManageListVO> getInformList(@RequestParam(name = "studentCaseName", required = false) String studentCaseName,
                                                       @RequestParam(name = "accountName", required = false) String accountName,
                                                       @RequestParam("pageNo") Integer pageNo,
                                                       @RequestParam("pageSize") Integer pageSize) {
        RequestContext res = ContextHolder.get();

        return studentCaseService.getInformList(studentCaseName, accountName, pageNo, pageSize, res);
    }

    @Override
    @GetMapping("/informList/export")
    public String informListExport(@RequestParam(name = "studentCaseName", required = false) String studentCaseName,
                                   @RequestParam(name = "accountName", required = false) String accountName) {
        Map<String, Object> map = new HashMap<>(16);
        RequestContext res = ContextHolder.get();
        Long taskId = System.currentTimeMillis();
        String serialNo = "INFORM-LIST-EXPORT-" + taskId.toString();
        String taskName = "举报记录列表导出-" + taskId.toString();
        map.put("res", res);
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("studentCaseName", studentCaseName);
        map.put("accountName", accountName);

        informListExport.execute(map, true);
        return serialNo;
    }

    @Override
    @GetMapping("/informList/view")
    public StudentCaseStudyVO informListView(@RequestParam("informRecordId") Long informRecordId) {
        RequestContext res = ContextHolder.get();
        return studentCaseService.informListView(informRecordId, res);
    }

    @Override
    @GetMapping("/list/getCount")
    public Map<String, Object> getManageListCount(Long caseLibraryId) {
        return studentCaseService.getManageListCount(caseLibraryId);
    }

    @Override
    @GetMapping("/putDown")
    public boolean putDown(Long studentCaseId) {

        StudentCase studentCase = new StudentCase();
        studentCase.setId(studentCaseId);
        studentCase.setState(3);

        return studentCaseService.updateById(studentCase);
    }

    @Override
    @GetMapping("/release")
    public boolean release(Long studentCaseId) {

        StudentCase studentCase = new StudentCase();
        studentCase.setId(studentCaseId);
        studentCase.setState(2);

        return studentCaseService.updateById(studentCase);
    }

    @Override
    @GetMapping("/delete")
    public boolean delete(Long studentCaseId) {
        return studentCaseService.deleteStudentCase(studentCaseId);
    }

    @Override
    @GetMapping("/informRecordRemove")
    public boolean informRecordRemove(Long informRecordId) {

        return informRecordeService.deleteById(informRecordId);
    }
}
