package com.yizhi.library.application.controller;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.library.application.asynchronousTask.StudentCaseByAccountExport;
import com.yizhi.library.application.asynchronousTask.StudentCaseByOrgExport;
import com.yizhi.library.application.asynchronousTask.StudentCaseExport;
import com.yizhi.library.application.domain.StatisticStudentCaseAccount;
import com.yizhi.library.application.domain.StatisticStudentCaseOrg;
import com.yizhi.library.application.service.IStudentCaseReportService;
import com.yizhi.library.application.vo.StatisticStudentCaseVO;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping("/studentCaseReport")
public class StudentCaseReportController {

    @Autowired
    private IStudentCaseReportService studentCaseReportService;

    @Autowired
    private StudentCaseByAccountExport studentCaseByAccountExport;

    @Autowired
    private StudentCaseByOrgExport studentCaseByOrgExport;

    @Autowired
    private StudentCaseExport studentCaseExport;

	@GetMapping("/list/org")
	public Page<StatisticStudentCaseOrg> studentCaseByOrg(
			@ApiParam(value = "查询关键字", required = false) @RequestParam(name = "kwd", required = false) String kwd,
			@ApiParam(value = "每页条数，默认10", required = false) @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize,
			@ApiParam(value = "当前页数", required = false) @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo) {
		RequestContext requestContext = ContextHolder.get();
		Page<StatisticStudentCaseOrg> page = studentCaseReportService.studentCaseByOrg(kwd, requestContext.getSiteId(), pageSize, pageNo);
		return page;
	}

	@GetMapping("/list")
	public Page<StatisticStudentCaseVO> studentCaseList(
			@ApiParam(value = "开始时间（yyyy-mm-dd）", required = false) @RequestParam(name = "startDate", required = false) String startDate,
			@ApiParam(value = "结束时间（yyyy-mm-dd）", required = false) @RequestParam(name = "endDate", required = false) String endDate,
			@ApiParam(value = "查询关键字", required = false) @RequestParam(name = "kwd", required = false) String kwd,
			@ApiParam(value = "每页条数，默认10", required = false) @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize,
			@ApiParam(value = "当前页数", required = false) @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo) {
		RequestContext requestContext = ContextHolder.get();
		Page<StatisticStudentCaseVO> page = studentCaseReportService.studentCaseList(startDate, endDate, kwd,requestContext.getSiteId(), pageSize, pageNo);
		return page;
	}

	@GetMapping("/list/account")
	public Page<StatisticStudentCaseAccount> studentCaseByAccount(
			@ApiParam(value = "查询关键字", required = false) @RequestParam(name = "kwd", required = false) String kwd,
			@ApiParam(value = "每页条数，默认10", required = false) @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize,
			@ApiParam(value = "当前页数", required = false) @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo) {
		RequestContext requestContext = ContextHolder.get();
		Page<StatisticStudentCaseAccount> page = studentCaseReportService.studentCaseByAccount(kwd, requestContext.getSiteId(), pageSize, pageNo);
		return page;
	}

	@GetMapping("/list/account/export")
	public Map<String, Object> studentCaseExportByAccount(@ApiParam(value = "查询关键字", required = false) @RequestParam(name = "kwd", required = false) String kwd) {
		RequestContext requestContext = ContextHolder.get();
		Date submitTime = new Date();
		Long taskId = submitTime.getTime();
		String serialNo = "USER-EXPORT-" + taskId;
		String taskName = "导出学员案例按用户统计报表-" + taskId;
		String result = "任务编号：" + serialNo + "。任务名称：" + taskName;
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("accountId", requestContext.getAccountId());
		map.put("siteId", requestContext.getSiteId());
		map.put("companyId", requestContext.getCompanyId());
		map.put("serialNo", serialNo);
		map.put("taskId", taskId);
		map.put("submitTime", submitTime);
		map.put("taskName", taskName);
		map.put("result", result);
		map.put("kwd", kwd);
		studentCaseByAccountExport.execute(map, true);
		return map;
	}

	@GetMapping("/list/org/export")
	public Map<String, Object> studentCaseExportByOrg(@ApiParam(value = "查询关键字", required = false) @RequestParam(name = "kwd", required = false) String kwd) {
		RequestContext requestContext = ContextHolder.get();
		Date submitTime = new Date();
		Long taskId = submitTime.getTime();
		String serialNo = "USER-EXPORT-" + taskId;
		String taskName = "导出学员案例按部门统计报表-" + taskId;
		String result = "任务编号：" + serialNo + "。任务名称：" + taskName;
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("accountId", requestContext.getAccountId());
		map.put("siteId", requestContext.getSiteId());
		map.put("companyId", requestContext.getCompanyId());
		map.put("serialNo", serialNo);
		map.put("taskId", taskId);
		map.put("submitTime", submitTime);
		map.put("taskName", taskName);
		map.put("result", result);
		map.put("kwd", kwd);
		studentCaseByOrgExport.execute(map, true);
		return map;
	}

	@GetMapping("/list/export")
	public Map<String, Object> studentCaseExport(@ApiParam(value = "开始时间（yyyy-mm-dd）", required = false) @RequestParam(name = "startDate", required = false) String startDate,
			@ApiParam(value = "结束时间（yyyy-mm-dd）", required = false) @RequestParam(name = "endDate", required = false) String endDate,
			@ApiParam(value = "查询关键字", required = false) @RequestParam(name = "kwd", required = false) String kwd) {
		RequestContext requestContext = ContextHolder.get();
		Date submitTime = new Date();
		Long taskId = submitTime.getTime();
		String serialNo = "USER-EXPORT-" + taskId;
		String taskName = "导出学员案例按案例统计报表-" + taskId;
		String result = "任务编号：" + serialNo + "。任务名称：" + taskName;
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("accountId", requestContext.getAccountId());
		map.put("siteId", requestContext.getSiteId());
		map.put("companyId", requestContext.getCompanyId());
		map.put("serialNo", serialNo);
		map.put("taskId", taskId);
		map.put("submitTime", submitTime);
		map.put("taskName", taskName);
		map.put("result", result);
		map.put("startDate", startDate);
		map.put("endDate", endDate);
		map.put("kwd", kwd);
		studentCaseExport.execute(map, true);
		return map;
	}
}
