package com.yizhi.library.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.library.application.domain.CaseLibrary;
import com.yizhi.library.application.vo.CaseLibraryVO;
import com.yizhi.library.application.vo.ClassifyVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * <p>
 * Mapper 接口
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-07-03
 */
public interface CaseLibraryMapper extends BaseMapper<CaseLibrary> {

    /**
     * 案例活动上架
     *
     * @param caseLibraryId
     * @return
     */
    boolean releaseCaseLibrary(@Param("caseLibraryId") Long caseLibraryId);

    /**
     * 案例活动下架
     *
     * @param caseLibraryId
     * @return
     */
    boolean putDownCaseLibrary(@Param("caseLibraryId") Long caseLibraryId);

    /**
     * 案例活动添加维度 获取列表
     *
     *
     * @param classifyIds
     * @param caseLibraryId
     * @param companyId
     * @param siteId
     * @return
     */
    List<ClassifyVO> getClassify(@Param("classifyIds") List<Long> classifyIds, @Param("caseLibraryId") Long caseLibraryId, @Param("companyId") Long companyId, @Param("siteId") Long siteId);

    /**
     * 案例活动修改获取信息
     *
     * @param caseLibraryId
     * @return
     */
    CaseLibrary viewCaseLibrary(@Param("caseLibraryId") Long caseLibraryId);

    /**
     * 案例活动修改获取关联维度列表
     *
     * @param caseLibraryId
     * @return
     */
    List<Long> relatedClassify(@Param("caseLibraryId") Long caseLibraryId);

    /**
     * 获取案例库管理列表
     *
     * @param name
     * @param state
     * @param page
     * @param companyId
     * @param siteId
     * @return
     */
    List<CaseLibraryVO> getCaseLibraryList(@Param("name") String name, @Param("state") Integer state,
                                           Page<CaseLibraryVO> page,
                                           @Param("companyId") Long companyId,
                                           @Param("siteId") Long siteId);

    /**
     * 案例活动逻辑删除
     *
     * @param caseLibraryId
     * @return
     */
    boolean deleteCaseLibrary(@Param("caseLibraryId") Long caseLibraryId);

    /**
     * 案例活动关联培训项目
     *
     * @param caseLibraryIds
     * @param projectId
     * @return
     */
    boolean relateProject(@Param("caseLibraryIds") List<Long> caseLibraryIds, @Param("projectId") Long projectId);

    /**
     * 案例活动解除关联项目
     *
     * @param projectId
     * @param caseLibraryIds
     * @return
     */
    boolean relieveRelateProject(@Param("projectId") Long projectId, @Param("caseLibraryIds") List<Long> caseLibraryIds);

    /**
     * 培训项目关联案例活动获取列表
     *
     * @param name
     * @param companyId
     * @param siteId
     * @param page
     * @return
     */
    List<CaseLibraryVO> getProjectList(@Param("name") String name,
                                     @Param("companyId") Long companyId,
                                     @Param("siteId") Long siteId,
                                     Page<CaseLibraryVO> page);

    /**
     * 案例活动取消关联培训项目
     *
     * @param caseLibraryId
     * @return
     */
    boolean cancelRelateProject(@Param("caseLibraryId") Long caseLibraryId);

    /**
     * 管理端首页配置上传案例获取案例活动列表
     *
     * @param companyId
     * @param siteId
     * @param page
     * @return
     */
    List<CaseLibraryVO> getHomeList(@Param("companyId") Long companyId, @Param("siteId") Long siteId, Page<CaseLibraryVO> page);

    /**
     * 查找有子分类的维度
     * @param companyId
     * @param siteId
     * @return
     */
    List<Long> getHasChildrenIds(@Param("companyId") Long companyId, @Param("siteId") Long siteId);
}
