package com.yizhi.library.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.library.application.domain.CaseLibrary;
import com.yizhi.library.application.domain.StudentCase;
import com.yizhi.library.application.vo.StudentCaseViewVO;
import com.yizhi.library.application.vo.*;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 学生案例表 Mapper 接口
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-07-09
 */
public interface StudentCaseMapper extends BaseMapper<StudentCase> {

    /**
     * pc端,移动端我的案例列表
     *
     * @param accountId
     * @param caseLibraryIds
     * @param companyId
     * @param siteId
     * @param page
     * @return
     */
    List<CaseLibraryVO> myCaseLibraryList(@Param("accountId") Long accountId, @Param("caseLibraryIds") List<Long> caseLibraryIds, @Param("companyId") Long companyId, @Param("siteId") Long siteId, Page<CaseLibraryVO> page);

    /**
     * pc端,移动端我的案例记录总数
     * @param accountId
     * @param caseLibraryIds
     * @param companyId
     * @param siteId
     * @return
     */
    Integer myCaseLibraryTotal(@Param("accountId") Long accountId,@Param("caseLibraryIds") List<Long> caseLibraryIds,@Param("companyId") Long companyId,@Param("siteId") Long siteId);

    /**
     * 学员案例提交记录中撤回
     *
     * @param studentCaseId
     * @return
     */
    boolean studentCaseRecall(@Param("studentCaseId") Long studentCaseId);

    /**
     * 学员案例记录查看或者编辑获得信息
     *
     * @param studentCaseId
     * @return
     */
    StudentCaseViewVO studentCaseView(@Param("studentCaseId") Long studentCaseId);

    /**
     * 学员案例学习获取案例信息/审核,评分获取案例信息
     *
     * @param studentCaseId
     * @param type
     * @param accountId
     * @return
     */
    StudentCaseStudyVO studyStudentCase(@Param("studentCaseId") Long studentCaseId,
                                        @Param("type") Integer type,
                                        @Param("accountId") Long accountId);

    /**
     * 学员案例评分提交
     *
     * @param studentCaseId
     * @param score
     * @return
     */
    boolean studentCaseScore(@Param("studentCaseId") Long studentCaseId,
                             @Param("score") Integer score);

    /**
     * 学员端审核获得案例信息
     *
     * @param accountId
     * @param companyId
     * @param siteId
     * @param name
     * @param pageSize
     * @param page
     * @return
     */
    List<StudentCaseStudyVO> getAuditCaseVO(@Param("accountId") Long accountId,
                                            @Param("companyId") Long companyId,
                                            @Param("siteId") Long siteId,
                                            @Param("name") String name,
                                            @Param("pageSize") Integer pageSize,
                                            Page<StudentCaseStudyVO> page);

    /**
     * 学员端评分获得案例信息
     *
     * @param accountId
     * @param companyId
     * @param siteId
     * @param name
     * @param pageSize
     * @param page
     * @return
     */
    List<StudentCaseStudyVO> getScoreCaseVO(@Param("accountId") Long accountId,
                                            @Param("companyId") Long companyId,
                                            @Param("siteId") Long siteId,
                                            @Param("name") String name,
                                            @Param("pageSize") Integer pageSize,
                                            Page<StudentCaseStudyVO> page);

    /**
     * 获取审核通过后前端显示类型(自动上架/手动上架)
     *
     * @param studentCaseId
     * @return
     */
    Map<String, Object> getExamine(@Param("studentCaseId") Long studentCaseId);

    /**
     * 批量审核(通过)
     *
     * @param auditState
     * @param audiEvaluate
     * @param idList
     * @return
     */
//    boolean batchAuditPass(@Param("auditState") Integer auditState, @Param("audiEvaluate") String audiEvaluate, @Param("idList") List<Long> idList);
//
//    /**
//     * 批量审核(不通过)
//     *
//     * @param auditState
//     * @param audiEvaluate
//     * @param idList
//     * @return
//     */
    boolean batchAuditNoPass(@Param("auditState") Integer auditState, @Param("audiEvaluate") String audiEvaluate, @Param("idList") List<Long> idList);

    /**
     * 批量评分
     *
     * @param score
     * @param idList
     * @return
     */
    boolean batchScore(@Param("score") Integer score, @Param("idList") List<Long> idList);

    /**
     * 管理员批量评分
     * @param score
     * @param idList
     * @return
     */
    boolean batchAdminScore(@Param("score") Integer score,@Param("idList") List<Long> idList);

    /**
     * 学员案例审核/评分导出案例
     *
     * @param idList
     * @return
     */
    List<StudentCaseVO> studentCaseBatchExport(@Param("idList") List<Long> idList);

    /**
     * 首页配置获取列表
     *
     *
     * @param accountIds
     * @param key
     * @param companyId
     * @param siteId
     * @param studentCaseIds
     * @param page
     * @return
     */
    List<StudentCaseVO> getStudentCaseHomeList(@Param("accountIds") List<Long> accountIds,
                                             @Param("key") String key,
                                             @Param("companyId") Long companyId,
                                             @Param("siteId") Long siteId,
                                             @Param("studentCaseIds") List<Long> studentCaseIds,
                                             Page<StudentCaseVO> page);

    /**
     * 所有学员案例管理列表/审核评分管理列表
     *
     * @param caseLibraryId
     * @param accountIds
     * @param studentCaseName
     * @param auditState
     * @param scoreState
     * @param companyId
     * @param siteId
     * @param page
     * @return
     */
    List<StudentCaseManageListVO> studentCaseManageList(@Param("caseLibraryId") Long caseLibraryId,
                                                        @Param("accountIds") List<Long> accountIds, @Param("studentCaseName") String studentCaseName,
                                                        @Param("auditState") Integer auditState,
                                                        @Param("scoreState") Integer scoreState,
                                                        @Param("companyId") Long companyId, @Param("siteId") Long siteId,
                                                        Page<StudentCaseManageListVO> page);

    /**
     * 所有学员案例管理列表/审核评分管理列表导出
     * @param caseLibraryId
     * @param accountIds
     * @param studentCaseName
     * @param auditState
     * @param scoreState
     * @param companyId
     * @param siteId
     * @param page
     * @return
     */
    List<StudentCaseManageListVO> studentCaseManageExport(@Param("caseLibraryId") Long caseLibraryId,
                                                          @Param("accountIds") List<Long> accountIds, @Param("studentCaseName") String studentCaseName,
                                                          @Param("auditState") Integer auditState,
                                                          @Param("scoreState") Integer scoreState,
                                                          @Param("companyId") Long companyId, @Param("siteId") Long siteId,
                                                          Page<StudentCaseManageListVO> page);

    /**
     * 学员案例审核/评分获取学员案例信息
     *
     * @param studentCaseId
     * @return
     */
    StudentCaseStudyVO studentCaseManageAuditOrScoreView(@Param("studentCaseId") Long studentCaseId);

    /**
     * 导出待审核案例
     *
     * @param studentCaseIds
     * @param companyId
     * @param siteId
     * @return
     */
    List<StudentCaseManageListVO> exportAuditList(@Param("studentCaseIds") List<Long> studentCaseIds,
                                                  @Param("companyId") Long companyId,
                                                  @Param("siteId") Long siteId);

    /**
     * 导出待评分案例
     *
     * @param studentCaseIds
     * @param companyId
     * @param siteId
     * @return
     */
    List<StudentCaseManageListVO> exportScoreList(@Param("studentCaseIds") List<Long> studentCaseIds,
                                                  @Param("companyId") Long companyId,
                                                  @Param("siteId") Long siteId);

    /**
     * 精选案例管理列表
     *
     * @param studentCaseName
     * @param state
     * @param accountIds
     * @param page
     * @param companyId
     * @param siteId
     * @return
     */
    List<SelectionCaseListVO> getSelectionList(@Param("studentCaseName") String studentCaseName,
                                               @Param("state") Integer state,
                                               @Param("accountIds") List<Long> accountIds,
                                               @Param("companyId") Long companyId,
                                               @Param("siteId") Long siteId,
                                               Page<SelectionCaseListVO> page);

    /**
     * 举报记录列表
     *
     * @param studentCaseName
     * @param accountIds
     * @param companyId
     * @param siteId
     * @param page
     * @return
     */
    List<StudentCaseManageListVO> getInformList(@Param("studentCaseName") String studentCaseName,
                                                @Param("accountIds") List<Long> accountIds,
                                                @Param("companyId") Long companyId,
                                                @Param("siteId") Long siteId, Page<StudentCaseManageListVO> page);

    /**
     * 举报记录列表导出
     *
     * @param studentCaseName
     * @param accountIds
     * @param companyId
     * @param siteId
     * @return
     */
    List<StudentCaseManageListVO> getInformExportList(@Param("studentCaseName") String studentCaseName,
                                                      @Param("accountIds") List<Long> accountIds,
                                                      @Param("companyId") Long companyId,
                                                      @Param("siteId") Long siteId);

    /**
     * 举报记录查看
     *
     * @param informRecordId
     * @return
     */
    StudentCaseStudyVO informListView(@Param("informRecordId") Long informRecordId);

    /**
     * 获取待审核/评分记录数
     *
     * @param caseLibraryId
     * @param accountIds
     * @param studentCaseName
     * @param auditState
     * @param scoreState
     * @param companyId
     * @param siteId
     * @return
     */
    Integer getStudentCaseManageCount(@Param("caseLibraryId") Long caseLibraryId,
                                      @Param("accountIds") List<Long> accountIds,
                                      @Param("studentCaseName") String studentCaseName,
                                      @Param("auditState") Integer auditState,
                                      @Param("scoreState") Integer scoreState,
                                      @Param("companyId") Long companyId,
                                      @Param("siteId") Long siteId);

    /**
     * 通过案例活动i获得项目id
     *
     * @param caseLibraryId
     * @return
     */
    Map<String, Object> getProjectByCaseLibrary(@Param("caseLibraryId") Long caseLibraryId);

    /**
     * 通过学员案例获得项目id
     *
     * @param studentLibraryId
     * @return
     */
    Map<String, Object> getProjectByStudentCase(@Param("studentLibraryId") Long studentLibraryId);

    /**
     * 获取案例活动信息
     *
     * @param caseLibraryId
     * @return
     */
    CaseLibrary getCaseLibrary(@Param("caseLibraryId") Long caseLibraryId);

    /**
     * 管理端获取审核,评分上一篇id
     *
     * @param caseLibraryId
     * @param studentCaseId
     * @param submitTime
     * @param auditState
     * @param scoreState
     * @param accountIds
     * @param companyId
     * @param siteId
     * @return
     */
    List<Long> getManageAuditOrScorePrefix(@Param("caseLibraryId") Long caseLibraryId,
                                           @Param("studentCaseId") Long studentCaseId,
                                           @Param("submitTime") Date submitTime,
                                           @Param("auditState") Integer auditState,
                                           @Param("scoreState") Integer scoreState,
                                           @Param("accountIds") List<Long> accountIds,
                                           @Param("companyId") Long companyId,
                                           @Param("siteId") Long siteId);

    /**
     * 管理端获取审核,评分下一篇id
     *
     * @param caseLibraryId
     * @param studentCaseId
     * @param submitTime
     * @param auditState
     * @param scoreState
     * @param accountIds
     * @param companyId
     * @param siteId
     * @return
     */
    List<Long> getManageAuditOrScoreSuffix(@Param("caseLibraryId") Long caseLibraryId,
                                           @Param("studentCaseId") Long studentCaseId,
                                           @Param("submitTime") Date submitTime,
                                           @Param("auditState") Integer auditState,
                                           @Param("scoreState") Integer scoreState,
                                           @Param("accountIds") List<Long> accountIds,
                                           @Param("companyId") Long companyId,
                                           @Param("siteId") Long siteId);

    /**
     * 举报记录获取上一条id
     *
     * @param informRecordId
     * @param informTime
     * @param accountIds
     * @param companyId
     * @param siteId
     * @return
     */
    List<Long> getManageInformPrefix(@Param("informRecordId") Long informRecordId,
                                     @Param("informTime") Date informTime,
                                     @Param("accountIds") List<Long> accountIds,
                                     @Param("companyId") Long companyId,
                                     @Param("siteId") Long siteId);

    /**
     * 举报记录获取下一条id
     *
     * @param informRecordId
     * @param informTime
     * @param accountIds
     * @param companyId
     * @param siteId
     * @return
     */
    List<Long> getManageInformSuffix(@Param("informRecordId") Long informRecordId,
                                     @Param("informTime") Date informTime,
                                     @Param("accountIds") List<Long> accountIds,
                                     @Param("companyId") Long companyId,
                                     @Param("siteId") Long siteId);

    /**
     * 学员端审核获取上一条
     *
     * @param studentCaseId
     * @param submitTime
     * @param accountId
     * @param companyId
     * @param siteId
     * @return
     */
    List<Long> getStudentAuditPrefix(@Param("studentCaseId") Long studentCaseId,
                                     @Param("submitTime") Date submitTime,
                                     @Param("accountId") Long accountId,
                                     @Param("companyId") Long companyId,
                                     @Param("siteId") Long siteId);

    /**
     * 学员端审核获取下一条
     *
     * @param studentCaseId
     * @param submitTime
     * @param accountId
     * @param companyId
     * @param siteId
     * @return
     */
    List<Long> getStudentAuditSuffix(@Param("studentCaseId") Long studentCaseId,
                                     @Param("submitTime") Date submitTime,
                                     @Param("accountId") Long accountId,
                                     @Param("companyId") Long companyId,
                                     @Param("siteId") Long siteId);

    /**
     * 学员端评分获取上一条
     *
     * @param studentCaseId
     * @param submitTime
     * @param accountId
     * @param companyId
     * @param siteId
     * @return
     */
    List<Long> getStudentScorePrefix(@Param("studentCaseId") Long studentCaseId,
                                     @Param("submitTime") Date submitTime,
                                     @Param("accountId") Long accountId,
                                     @Param("companyId") Long companyId,
                                     @Param("siteId") Long siteId);

    /**
     * 学员端评分获取下一条
     *
     * @param studentCaseId
     * @param submitTime
     * @param accountId
     * @param companyId
     * @param siteId
     * @return
     */
    List<Long> getStudentScoreSuffix(@Param("studentCaseId") Long studentCaseId,
                                     @Param("submitTime") Date submitTime,
                                     @Param("accountId") Long accountId,
                                     @Param("companyId") Long companyId,
                                     @Param("siteId") Long siteId);

    /**
     * 学员案例逻辑删除
     *
     * @param studentCaseId
     * @return
     */
    boolean deleteStudentCase(@Param("studentCaseId") Long studentCaseId);

    /**
     * 获取我的收藏案例列表
     *
     * @param companyId
     * @param siteId
     * @param name
     * @param accountId
     * @param page
     * @return
     */
    List<FavoriteVO> getFavorites(@Param("companyId") Long companyId,
                                  @Param("siteId") Long siteId,
                                  @Param("name") String name,
                                  @Param("accountId") Long accountId,
                                  Page<FavoriteVO> page);

    /**
     * 培训项目关联精选案例获取列表
     *
     * @param name
     * @param companyId
     * @param siteId
     * @param page
     * @return
     */
    List<StudentCaseStudyVO> getProjectList(@Param("name") String name,
                                            @Param("companyId") Long companyId,
                                            @Param("siteId") Long siteId, Page<StudentCaseStudyVO> page);

    /**
     * 分类筛选获得学员案例id
     *
     * @param companyId
     * @param siteId
     * @param classifyIds
     * @return
     */
    List<Long> getStudentCaseIdsByClassifyIds(@Param("companyId") Long companyId,
                                              @Param("siteId") Long siteId,
                                              @Param("classifyIds") List<Long> classifyIds);

    /**
     * 精选案例搜索页获取列表
     * @param key
     * @param rangeIds
     * @param accountIds
     * @param studentCaseIds
     * @param sortType
     * @param companyId
     * @param siteId
     * @param page
     * @return
     */
    List<FavoriteVO> getSelectionStudentCaseList(@Param("key") String key,
                                                 @Param("accountIds") List<Long> accountIds,
                                                 @Param("rangeIds") List<Long> rangeIds,
                                                 @Param("studentCaseIds") List<Long> studentCaseIds,
                                                 @Param("sortType") Integer sortType,
                                                 @Param("companyId") Long companyId,
                                                 @Param("siteId") Long siteId,
                                                 Page<FavoriteVO> page);

    /**
     * 获取指定范围的该用户可见的学员案例id
     * @param relationIds
     * @return
     */
    List<Long> getRangeIds(@Param("relationIds") List<Long> relationIds);

    /**
     * 首页获取精选案例信息
     * @param studentCaseId
     * @return
     */
    FavoriteVO getStudentCaseDetail(@Param("studentCaseId") Long studentCaseId);

    /**
     * 首页获取案例点赞总数
     * @param companyId
     * @param siteId
     * @param accountId
     * @return
     */
    Integer getFavoritesCount(@Param("companyId") Long companyId,@Param("siteId") Long siteId,@Param("accountId") Long accountId);

    /**
     * 更新学习数
     * @param studentCaseId
     */
    void updateStudyNum(@Param("studentCaseId") Long studentCaseId);

    /**
     * 获取有提交记录的案例活动id
     * @param companyId
     * @param siteId
     * @param accountId
     * @return
     */
    List<Long> getSubmitCaseLibraryIds(@Param("companyId") Long companyId,@Param("siteId") Long siteId,@Param("accountId") Long accountId);

    /**
     * 每次跑批初始化学员案例表中的点赞数
     */
    void initializeAdmireNum();

    /**
     * 每次跑批校正学员案例表中的点赞数
     */
    void checkAdmireNum();

    /**
     * 校正点赞数
     * @param id
     * @return
     */
    Integer getAdmireNum(@Param("id") Long id);
}
