package com.yizhi.library.application.service;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.IService;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.library.application.domain.CaseLibrary;
import com.yizhi.library.application.param.RelateProjectParam;
import com.yizhi.library.application.vo.CaseLibraryVO;
import com.yizhi.library.application.vo.CaselibraryAuthorizeVO;
import com.yizhi.library.application.vo.ClassifyVO;

import java.util.List;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-07-03
 */
public interface ICaseLibraryService extends IService<CaseLibrary> {

    /**
     * 新建案例活动
     *
     * @param caseLibraryVO
     * @return
     */
    Long insertCaseLibrary(CaseLibraryVO caseLibraryVO);

    /**
     * 案例活动修改
     *
     * @param caseLibraryVO
     * @return
     */
    boolean updateCaseLibrary(CaseLibraryVO caseLibraryVO);

    /**
     * 案例活动发布上架
     *
     * @param caseLibraryId
     * @return
     */
    boolean releaseCaseLibrary(Long caseLibraryId);

    /**
     * 案例活动下架
     *
     * @param caseLibraryId
     * @return
     */
    boolean putDownCaseLibrary(Long caseLibraryId);

    /**
     * 新建案例活动获取维度列表
     *
     * @param caseLibraryId
     * @return
     */
    List<ClassifyVO> getClassify(Long caseLibraryId);

    /**
     * 案例活动修改获取信息
     *
     * @param caseLibraryId
     * @return
     */
    CaseLibraryVO viewCaseLibrary(Long caseLibraryId);

    /**
     * 获取案例库管理列表
     *
     * @param name
     * @param state
     * @param pageNo
     * @param pageSize
     * @return
     */
    Page<CaseLibraryVO> getCaseLibraryList(String name, Integer state, Integer pageNo, Integer pageSize);

    /**
     * 逻辑删除案例活动
     *
     * @param caseLibraryId
     * @return
     */
    boolean deleteCaseLibrary(Long caseLibraryId);

    /**
     * 案例活动关联培训项目
     *
     * @param relateProjectParam
     * @return
     */
    boolean relateProject(RelateProjectParam relateProjectParam);

    /**
     * 培训项目关联案例活动获取列表
     *
     *
     * @param name
     * @param companyId
     * @param siteId
     * @param pageNo
     * @param pageSize
     * @return
     */
    Page<CaseLibraryVO> getProjectList(String name, Long companyId, Long siteId, Integer pageNo, Integer pageSize);

    /**
     * 案例活动取消关联培训项目
     * @param caseLibraryId
     * @return
     */
    boolean cancelRelateProject(Long caseLibraryId);

    /**
     * 案例活动插入指定范围
     * @param caseLibraryAuthorizeList
     * @return
     */
    boolean insertAuthorizes(List<CaselibraryAuthorizeVO> caseLibraryAuthorizeList);

    /**
     * 案例活动获取指定范围
     * @param caselibraryId
     * @return
     */
    List<CaselibraryAuthorizeVO> getAuthorize(Long caselibraryId);

    /**
     * 管理端首页配置上传案例获取案例活动列表
     * @param pageNo
     * @param pageSize
     * @return
     */
    Page<CaseLibraryVO> getHomeList(Integer pageNo, Integer pageSize);

    Page<DroolsVo> getPageByDrools(String field, String value, Page<DroolsVo> page);

    List<Long> getCaseIdsByName(String name);
}
