package com.yizhi.library.application.service;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.IService;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.library.application.domain.Comment;
import com.yizhi.library.application.param.CommentParam;
import com.yizhi.library.application.vo.CommentReplyVO;
import com.yizhi.library.application.vo.CommentVO;

import java.util.List;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-07-12
 */
public interface ICommentService extends IService<Comment> {

    /**
     * 发表评论/回复
     *
     * @param commentParam
     * @return
     */
    boolean comment(CommentParam commentParam);

    /**
     * 评论点赞
     *
     * @param commentId
     * @return
     */
    boolean admireComment(Long commentId);

    /**
     * 评论取消点赞
     *
     * @param commentId
     * @return
     */
    boolean cancelAdmireComment(Long commentId);

    /**
     * 获取评论列表
     *
     * @param studentCaseId
     * @param pageNo
     * @param pageSize
     * @return
     */
    Page<CommentVO> getCommentList(Long studentCaseId, Integer pageNo, Integer pageSize);

    /**
     * 获取评论管理列表
     *
     * @param studentCaseId
     * @param name
     * @param state
     * @param pageNo
     * @param pageSize
     * @return
     */
    Page<CommentVO> getCommentManageList(Long studentCaseId, String name, Integer state, Integer pageNo, Integer pageSize);

    /**
     * 获取评论回复列表
     *
     * @param name
     * @param state
     * @param commentId
     * @return
     */
    List<CommentReplyVO> getCommentReplyList(String name, Integer state, Long commentId);

    /**
     * 评论/回复下架
     *
     * @param id
     * @param type
     * @return
     */
    boolean putDown(Long id, Integer type);

    /**
     * 评论/回复删除
     *
     * @param id
     * @param type
     * @return
     */
    boolean deleteCommentOrReply(Long id, Integer type);

    /**
     * 评论/回复上架
     *
     * @param id
     * @param type
     * @return
     */
    boolean release(Long id, Integer type);

    /**
     * 案例评论记录下载
     * @param res
     * @param studentCaseId
     * @param name
     * @param state
     * @return
     */
    List<CommentVO> downloadList(RequestContext res, Long studentCaseId, String name, Integer state);
}
