package com.yizhi.library.application.service;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.IService;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.library.application.domain.StudentCase;
import com.yizhi.library.application.param.HomeListParam;
import com.yizhi.library.application.param.SelectionStudentCaseParam;
import com.yizhi.library.application.param.StudentCaseAuditParam;
import com.yizhi.library.application.param.StudentCaseParam;
import com.yizhi.library.application.vo.StudentCaseViewVO;
import com.yizhi.library.application.vo.*;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 学生案例表 服务类
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-07-09
 */
public interface IStudentCaseService extends IService<StudentCase> {

    /**
     * pc端,移动端我的案例列表
     *
     * @param pageNo
     * @param pageSize
     * @return
     */
    Page<CaseLibraryVO> myCaseLibraryList(Integer pageNo, Integer pageSize);

    /**
     * 学员提交案例获取案例库信息
     *
     * @param caseLibraryId
     * @return
     */
    StudentCaseVO prepareStudentCase(Long caseLibraryId);

    /**
     * 学员提交案例
     *
     * @param studentCaseParam
     * @return
     */
    boolean submitStudentCase(StudentCaseParam studentCaseParam);

    /**
     * 获取学员案例提交记录列表
     *
     * @param caseLibraryId
     * @param endTime
     * @return
     */
    StudentCaseRecordListVO getSubmitRecords(Long caseLibraryId, String endTime);

    /**
     * 学员案例提交记录中撤回
     *
     * @param studentCaseId
     * @return
     */
    boolean studentCaseRecall(Long studentCaseId);

    /**
     * 学员案例记录查看或者编辑获得信息
     *
     * @param studentCaseId
     * @return
     */
    StudentCaseViewVO studentCaseView(Long studentCaseId, Integer type);

    /**
     * 学员案例学习获取案例信息/审核,评分获取案例信息
     *
     * @param studentCaseId
     * @param type
     * @param name
     * @param number
     * @return
     */
    StudentCaseStudyVO studyStudentCase(Long studentCaseId, Integer type, Integer number, String name);

    /**
     * 学员案例审核提交
     *
     * @param studentCaseAuditParam
     * @return
     */
    boolean studentCaseAudit(StudentCaseAuditParam studentCaseAuditParam);

    /**
     * 学员案例评分提交
     *
     * @param studentCaseAuditParam
     * @return
     */
    boolean studentCaseScore(StudentCaseAuditParam studentCaseAuditParam);

    /**
     * 学员案例审核,评分获取案例列表
     *
     * @param type
     * @param name
     * @param pageNo
     * @param pageSize
     * @return
     */
    Page<StudentCaseStudyVO> getAuditList(Integer type, String name, Integer pageNo, Integer pageSize);

    /**
     * 学员案例批量审核
     *
     * @param studentCaseAuditParam
     * @return
     */
    boolean batchAudit(StudentCaseAuditParam studentCaseAuditParam);

    /**
     * 学员案例批量评分
     *
     * @param studentCaseAuditParam
     * @return
     */
    boolean batchScore(StudentCaseAuditParam studentCaseAuditParam);

    /**
     * 学员案例审核/评分导出案例
     *
     * @param idList
     * @return
     */
    List<StudentCaseVO> studentCaseBatchExport(List<Long> idList);

    /**
     * 首页配置获取列表
     *
     * @param homeListParam
     * @return
     */
    Page<StudentCaseVO> getStudentCaseHomeList(HomeListParam homeListParam);

    /**
     * 审核评分管理列表
     *
     * @param caseLibraryId
     * @param orgName
     * @param studentCaseName
     * @param accountName
     * @param state
     * @param pageNo
     * @param pageSize
     * @return
     */
    Page<StudentCaseManageListVO> studentCaseManageList(Long caseLibraryId, String studentCaseName, String accountName,
                                                        Integer state, String orgName, Integer pageNo, Integer pageSize,
                                                        RequestContext res, Integer type);

    /**
     * 学员案例审核/评分获取学员案例信息
     *
     * @param studentCaseId
     * @param caseLibraryId
     * @param studentCaseName
     * @param accountName
     * @param state
     * @param orgName
     * @param number
     * @return
     */
    StudentCaseStudyVO studentCaseManageAuditOrScoreView(Long studentCaseId, Long caseLibraryId, String studentCaseName, String accountName, Integer state, String orgName, Integer number);

    /**
     * 导出待审核案例
     *
     * @param studentCaseIds
     * @param res
     * @return
     */
    List<StudentCaseManageListVO> exportAuditList(List<Long> studentCaseIds, RequestContext res);

    /**
     * 导出待评分案例
     *
     * @param studentCaseIds
     * @param res
     * @return
     */
    List<StudentCaseManageListVO> exportScoreList(List<Long> studentCaseIds, RequestContext res);

    /**
     * 精选案例管理列表
     *
     * @param studentCaseName
     * @param accountName
     * @param state
     * @param pageNo
     * @param pageSize
     * @param res
     * @return
     */
    Page<SelectionCaseListVO> getSelectionList(String studentCaseName, String accountName, Integer state, Integer pageNo, Integer pageSize, RequestContext res);

    /**
     * 举报记录列表
     *
     * @param studentCaseName
     * @param accountName
     * @param pageNo
     * @param pageSize
     * @param res
     * @return
     */
    Page<StudentCaseManageListVO> getInformList(String studentCaseName, String accountName, Integer pageNo, Integer pageSize, RequestContext res);

    /**
     * 举报记录列表导出
     *
     * @param studentCaseName
     * @param accountName
     * @param res
     * @return
     */
    List<StudentCaseManageListVO> getInformExportList(String studentCaseName, String accountName, RequestContext res);

    /**
     * 举报记录查看
     *
     * @param informRecordId
     * @param res
     * @return
     */
    StudentCaseStudyVO informListView(Long informRecordId, RequestContext res);

    /**
     * 案例管理列表获取记录数
     *
     * @param caseLibraryId
     * @return
     */
    Map<String, Object> getManageListCount(Long caseLibraryId);

    /**
     * 获取案例活动id及项目id
     *
     * @param caseLibraryId
     * @param studentLibraryId
     * @return
     */
    Map<String, Object> getLibraryProjectName(Long caseLibraryId, Long studentLibraryId);

    /**
     * 学员案例逻辑删除
     *
     * @param studentCaseId
     * @return
     */
    boolean deleteStudentCase(Long studentCaseId);

    /**
     * 获取我的收藏案例列表
     *
     * @param pageNo
     * @param pageSize
     * @param name
     * @return
     */
    Page<FavoriteVO> getFavorites(Integer pageNo, Integer pageSize, String name);

    /**
     * 培训项目关联精选案例获取列表
     *
     * @param name
     * @param pageNo
     * @param pageSize
     * @return
     */
    Page<StudentCaseStudyVO> getProjectList(String name, Integer pageNo, Integer pageSize);

    /**
     * 精选案例搜索页获取列表
     *
     * @param selectionStudentCaseParam
     * @return
     */
    Page<FavoriteVO> getSelectionStudentCaseList(SelectionStudentCaseParam selectionStudentCaseParam);

    /**
     * pc端,移动端我的案例记录总数
     *
     * @return
     */
    Integer myCaseLibraryTotal();

    /**
     * 首页获取精选案例信息
     * @param studentCaseId
     * @return
     */
    FavoriteVO getStudentCaseDetail(Long studentCaseId);

    /**
     * 首页获取案例点赞总数
     * @param companyId
     * @param siteId
     * @param accountId
     * @return
     */
    Integer getFavoritesCount(Long companyId, Long siteId, Long accountId);

    Page<DroolsVo> getPageByDrools(String field, String value, Page<DroolsVo> page);

    /**
     * 每次跑批校正学员案例表中的点赞数
     */
    void checkAdmireNum();
}
