package com.yizhi.library.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.enums.TaskParamsEnums;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.library.application.constant.Constant;
import com.yizhi.library.application.domain.CaseLibrary;
import com.yizhi.library.application.domain.CaseLibraryAuthorize;
import com.yizhi.library.application.domain.CaseLibraryDimensionGrade;
import com.yizhi.library.application.domain.CaseLibraryRelatedClassify;
import com.yizhi.library.application.mapper.CaseLibraryDimensionGradeMapper;
import com.yizhi.library.application.mapper.CaseLibraryMapper;
import com.yizhi.library.application.param.RelateProjectParam;
import com.yizhi.library.application.service.*;
import com.yizhi.library.application.vo.*;
import com.yizhi.system.application.constant.AuthzConstant;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.vo.HQAccountInManageParam;
import com.yizhi.system.application.vo.ReportAccountRespInManageVO;
import com.yizhi.system.application.vo.ReportRangeAccountReqVO;
import com.yizhi.training.application.feign.TpProjectClient;
import com.yizhi.training.application.vo.domain.TrainingProjectVo;
import com.yizhi.util.application.clazz.ClassUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-07-03
 */
@Service
public class CaseLibraryServiceImpl extends ServiceImpl<CaseLibraryMapper, CaseLibrary> implements ICaseLibraryService {

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ICaseLibraryRelatedClassifyService caseLibraryRelatedClassifyService;

    @Autowired
    private CaseLibraryMapper caseLibraryMapper;

    @Autowired
    private TpProjectClient tpProjectClient;

    @Autowired
    private ICaseLibraryAuthorizeService caseLibraryAuthorizeService;

    @Autowired
    ReportClient reportClient;

    @Autowired
    IClassifyService classifyService;

    @Autowired
    private CaseLibraryDimensionGradeMapper caseLibraryDimensionGradeMapper;

    @Autowired
    private ICaseLibraryDimensionGradeService caseLibraryDimensionGradeService;

    private Logger logger = LoggerFactory.getLogger(CaseLibraryServiceImpl.class);

    @Override
    public Long insertCaseLibrary(CaseLibraryVO caseLibraryVO) {
        RequestContext res = ContextHolder.get();
        if (null != caseLibraryVO) {
            if (null != caseLibraryVO.getId()) {
                updateCaseLibrary(caseLibraryVO);
                // 增加多维度打分设置功能
                dealCaseLibraryDimensionGradeInsert(caseLibraryVO, res);
                return caseLibraryVO.getId();
            }

            CaseLibrary caseLibrary = new CaseLibrary();

            BeanUtils.copyProperties(caseLibraryVO, caseLibrary);
            caseLibrary.setId(idGenerator.generate());
            caseLibrary.setCreateById(res.getAccountId());
            caseLibrary.setCreateByName(res.getAccountName());
            caseLibrary.setCreateTime(new Date());
            caseLibrary.setUpdateById(res.getAccountId());
            caseLibrary.setUpdateByName(res.getAccountName());
            caseLibrary.setUpdateTime(new Date());
            caseLibrary.setCompanyId(res.getCompanyId());
            caseLibrary.setSiteId(res.getSiteId());
            caseLibrary.setOrgId(res.getOrgId());
            caseLibrary.setState(1);
            caseLibrary.setExamine(1);
            caseLibrary.setTextEditType(1);
            caseLibrary.setShowScore(caseLibraryVO.getShowScore());
            caseLibrary.setCaseLibraryDimensionGradeDescription(caseLibraryVO.getCaseLibraryDimensionGradeDescription());

            if (null == caseLibraryVO.getId()) {
                if (!this.insert(caseLibrary)) {
                    // 增加多维度打分设置功能
                    caseLibraryVO.setId(caseLibrary.getId());
                    dealCaseLibraryDimensionGradeInsert(caseLibraryVO, res);
                    return null;
                }
            } else {
                if (!this.updateById(caseLibrary)) {
                    // 增加多维度打分设置功能
                    dealCaseLibraryDimensionGradeInsert(caseLibraryVO, res);
                    return null;
                }
            }

//            //先删除之前关联的分类
//            caseLibraryRelatedClassifyService.deleteNotClassifyList(caseLibrary.getId(), null);
            if (CollectionUtils.isNotEmpty(caseLibraryVO.getClassifyIdList())) {
                List<CaseLibraryRelatedClassify> classifyList = new ArrayList<>();
                for (Long classifyId : caseLibraryVO.getClassifyIdList()) {
                    CaseLibraryRelatedClassify caseLibraryRelatedClassify = new CaseLibraryRelatedClassify();
                    caseLibraryRelatedClassify.setId(idGenerator.generate());
                    caseLibraryRelatedClassify.setCaseLiraryId(caseLibrary.getId());
                    caseLibraryRelatedClassify.setClassifyId(classifyId);
                    caseLibraryRelatedClassify.setCreateById(res.getAccountId());
                    caseLibraryRelatedClassify.setCreateByName(res.getAccountName());
                    caseLibraryRelatedClassify.setCreateTime(new Date());
                    caseLibraryRelatedClassify.setUpdateById(res.getAccountId());
                    caseLibraryRelatedClassify.setUpdateByName(res.getAccountName());
                    caseLibraryRelatedClassify.setUpdateTime(new Date());
                    classifyList.add(caseLibraryRelatedClassify);
                }
                if (!caseLibraryRelatedClassifyService.insertBatch(classifyList)) {
                    return null;
                }
            }
            return caseLibrary.getId();
        }
        return null;
    }

    @Override
    public boolean updateCaseLibrary(CaseLibraryVO caseLibraryVO) {

        RequestContext res = ContextHolder.get();
        if (null != caseLibraryVO) {
            CaseLibrary caseLibrary = new CaseLibrary();
            BeanUtils.copyProperties(caseLibraryVO, caseLibrary);
            caseLibrary.setUpdateById(res.getAccountId());
            caseLibrary.setUpdateByName(res.getAccountName());
            caseLibrary.setUpdateTime(new Date());

            //一旦上架,得分规则不可以再修改
            if (null != caseLibrary.getState() && caseLibrary.getState() == 2) {
                caseLibrary.setCanModifyScoreRule(0);
            }

            if (!this.updateById(caseLibrary)) {
                return false;
            }

            //将之前的分类关联删除重新关联
            CaseLibraryRelatedClassify caseLibraryRelatedClassify1 = new CaseLibraryRelatedClassify();
            caseLibraryRelatedClassify1.setCaseLiraryId(caseLibraryVO.getId());
            if (caseLibraryVO.getType() == null) {
                caseLibraryRelatedClassifyService.delete(new EntityWrapper<>(caseLibraryRelatedClassify1));
            }
            //关联分类不为空说明是第一步,执行以下
            if (CollectionUtils.isNotEmpty(caseLibraryVO.getClassifyIdList())) {
                List<CaseLibraryRelatedClassify> classifyList = new ArrayList<>();
                for (Long classifyId : caseLibraryVO.getClassifyIdList()) {
                    CaseLibraryRelatedClassify caseLibraryRelatedClassify = new CaseLibraryRelatedClassify();
                    caseLibraryRelatedClassify.setId(idGenerator.generate());
                    caseLibraryRelatedClassify.setCaseLiraryId(caseLibrary.getId());
                    caseLibraryRelatedClassify.setClassifyId(classifyId);
                    caseLibraryRelatedClassify.setCreateById(res.getAccountId());
                    caseLibraryRelatedClassify.setCreateByName(res.getAccountName());
                    caseLibraryRelatedClassify.setCreateTime(new Date());
                    caseLibraryRelatedClassify.setUpdateById(res.getAccountId());
                    caseLibraryRelatedClassify.setUpdateByName(res.getAccountName());
                    caseLibraryRelatedClassify.setUpdateTime(new Date());
                    classifyList.add(caseLibraryRelatedClassify);
                }
                if (!caseLibraryRelatedClassifyService.insertBatch(classifyList)) {
                    return false;
                }
            }

            return true;
        }

        return false;
    }

    @Override
    public boolean releaseCaseLibrary(Long caseLibraryId) {
        return caseLibraryMapper.releaseCaseLibrary(caseLibraryId);
    }

    @Override
    public boolean putDownCaseLibrary(Long caseLibraryId) {

        CaseLibrary caseLibrary = this.selectById(caseLibraryId);
        if (null == caseLibrary.getTrainingProjectId()) {
            caseLibraryMapper.putDownCaseLibrary(caseLibraryId);
            return true;
        } else {
            List<Long> trainingProjectIds = new ArrayList<>();
            trainingProjectIds.add(caseLibrary.getTrainingProjectId());
            Map<Long, TrainingProjectVo> map = tpProjectClient.getCaseLibraryProject(trainingProjectIds);
            if (null != map) {
                TrainingProjectVo trainingProject = map.get(caseLibrary.getTrainingProjectId());
                //关联培训项目是上架状态,不能下架
                if (trainingProject.getStatus() == 1) {
                    return false;
                    //关联培训项目不是上架状态,可以下架
                } else {
                    caseLibraryMapper.putDownCaseLibrary(caseLibraryId);
                    return true;
                }
                //没找到关联培训项目直接下架
            } else {
                caseLibraryMapper.putDownCaseLibrary(caseLibraryId);
                return true;
            }
        }
    }

    @Override
    public List<ClassifyVO> getClassify(Long caseLibraryId) {
        RequestContext res = ContextHolder.get();
        List<Long> classifyIds = caseLibraryMapper.getHasChildrenIds(res.getCompanyId(), res.getSiteId());
        if (CollectionUtils.isEmpty(classifyIds)) {
            return null;
        }
        List<ClassifyVO> list = caseLibraryMapper.getClassify(classifyIds, caseLibraryId, res.getCompanyId(), res.getSiteId());
        if (CollectionUtils.isNotEmpty(list)) {
            for (ClassifyVO classifyVO : list) {
                classifyVO.setKey(classifyVO.getClassifyId().toString());
            }
        }
        return list;
    }

    @Override
    public CaseLibraryVO viewCaseLibrary(Long caseLibraryId) {
        CaseLibraryVO caseLibraryVO = new CaseLibraryVO();
        CaseLibrary caseLibrary = caseLibraryMapper.selectById(caseLibraryId);
        if (null != caseLibrary) {
            BeanUtils.copyProperties(caseLibrary, caseLibraryVO);
            List<Long> classifyIdList = caseLibraryMapper.relatedClassify(caseLibraryId);
            caseLibraryVO.setClassifyIdList(classifyIdList);
        }
        if (caseLibrary.getScoreRule() != null && caseLibrary.getScoreRule() == 4) {
            EntityWrapper<CaseLibraryDimensionGrade> ew = new EntityWrapper<>();
            ew.eq("deleted", 0);
            ew.eq("case_library_id", caseLibraryId);
            ew.orderBy("sort");
            List<CaseLibraryDimensionGrade> caseLibraryDimensionGrades = caseLibraryDimensionGradeService.selectList(ew);
            if (CollectionUtils.isNotEmpty(caseLibraryDimensionGrades)) {
                List<CaseLibraryDimensionGradeVO> voList = new ArrayList<>();
                CaseLibraryDimensionGradeVO gradeVO = null;
                for (CaseLibraryDimensionGrade caseLibraryDimensionGrade : caseLibraryDimensionGrades) {
                    gradeVO = new CaseLibraryDimensionGradeVO();
//                    gradeVO.setDescription(caseLibraryDimensionGrade.getDescription());
                    gradeVO.setId(caseLibraryDimensionGrade.getId());
                    gradeVO.setMaxScore(caseLibraryDimensionGrade.getMaxScore());
                    gradeVO.setMinScore(caseLibraryDimensionGrade.getMinScore());
                    gradeVO.setName(caseLibraryDimensionGrade.getName());
                    gradeVO.setSort(caseLibraryDimensionGrade.getSort());
                    voList.add(gradeVO);
                }
                caseLibraryVO.setCaseLibraryDimensionGradeVOS(voList);
            }
        }
        return caseLibraryVO;
    }

    @Override
    public Page<CaseLibraryVO> getCaseLibraryList(String name, Integer state, Integer pageNo, Integer pageSize) {
        Page<CaseLibraryVO> page = new Page<>(pageNo, pageSize);
        HQueryUtil.startHQ(CaseLibrary.class);
        RequestContext res = ContextHolder.get();

        List<CaseLibraryVO> list = caseLibraryMapper.getCaseLibraryList(name, state, page, res.getCompanyId(), res.getSiteId());
        if (CollectionUtils.isNotEmpty(list)) {
//            List<Long> trainingProjectIdList = list.stream().map(CaseLibraryVO::getTrainingProjectId).collect(Collectors.toList());
            List<Long> trainingProjectIdList = new ArrayList<>();
            for (CaseLibraryVO caseLibraryVO : list) {
                if (null != caseLibraryVO.getTrainingProjectId()) {
                    trainingProjectIdList.add(caseLibraryVO.getTrainingProjectId());
                }
            }
            Map<Long, TrainingProjectVo> map = new HashMap<>(16);
            if (CollectionUtils.isNotEmpty(trainingProjectIdList)) {
                map = tpProjectClient.getCaseLibraryProject(trainingProjectIdList);
            }
            for (CaseLibraryVO caseLibraryVO : list) {
                if (null != caseLibraryVO.getTrainingProjectId() && null != map &&
                        null != map.get(caseLibraryVO.getTrainingProjectId())) {
                    caseLibraryVO.setTrainingProjectName(map.get(caseLibraryVO.getTrainingProjectId()).getName());
                }
            }
            page.setRecords(list);
        }
        return page;
    }

    @Override
    public boolean deleteCaseLibrary(Long caseLibraryId) {
        return caseLibraryMapper.deleteCaseLibrary(caseLibraryId);
    }

    @Override
    public boolean relateProject(RelateProjectParam relateProjectParam) {
        if (CollectionUtils.isEmpty(relateProjectParam.getCaseLibraryIds())) {
            //没有关联活动,将该培训项目关联的案例活动全部解除
            return caseLibraryMapper.relieveRelateProject(relateProjectParam.getProjectId(), null);
        } else {
            //关联活动
            caseLibraryMapper.relateProject(relateProjectParam.getCaseLibraryIds(), relateProjectParam.getProjectId());
            //除了关联的案例活动之外的全部解除关联
            caseLibraryMapper.relieveRelateProject(relateProjectParam.getProjectId(), relateProjectParam.getCaseLibraryIds());

        }
        return true;
    }

    @Override
    public Page<CaseLibraryVO> getProjectList(String name, Long companyId, Long siteId, Integer pageNo, Integer pageSize) {

        Page<CaseLibraryVO> page = new Page<>(pageNo, pageSize);
        List<CaseLibraryVO> list = caseLibraryMapper.getProjectList(name, companyId, siteId, page);

        if (CollectionUtils.isNotEmpty(list)) {
            page.setRecords(list);
        }
        return page;
    }

    @Override
    public boolean cancelRelateProject(Long caseLibraryId) {
        return caseLibraryMapper.cancelRelateProject(caseLibraryId);
    }

    @Override
    public boolean insertAuthorizes(List<CaselibraryAuthorizeVO> caseLibraryAuthorizeList) {
        RequestContext res = ContextHolder.get();

        //插入之前,先删除之前的指定范围
        CaseLibraryAuthorize caseLibraryAuthorize = new CaseLibraryAuthorize();
        caseLibraryAuthorize.setCaseLibraryId(caseLibraryAuthorizeList.get(0).getCaseLibraryId());
        caseLibraryAuthorizeService.delete(new EntityWrapper<>(caseLibraryAuthorize));
        Date date = new Date();
        List<CaseLibraryAuthorize> listCaseLibrary = new ArrayList<>();

        //插入数据
        if (CollectionUtils.isNotEmpty(caseLibraryAuthorizeList)) {
            for (CaselibraryAuthorizeVO caseLibraryVO : caseLibraryAuthorizeList) {
                CaseLibraryAuthorize caseLibrary = new CaseLibraryAuthorize();
                BeanUtils.copyProperties(caseLibraryVO, caseLibrary);
                caseLibrary.setId(idGenerator.generate());
                caseLibrary.setCreateById(res.getAccountId());
                caseLibrary.setCreateByName(res.getAccountName());
                caseLibrary.setCreateTime(date);
                caseLibrary.setUpdateById(res.getAccountId());
                caseLibrary.setUpdateByName(res.getAccountName());
                caseLibrary.setUpdateTime(date);
                listCaseLibrary.add(caseLibrary);
            }
            return caseLibraryAuthorizeService.insertBatch(listCaseLibrary);
        }

        return true;
    }

    @Override
    public List<CaselibraryAuthorizeVO> getAuthorize(Long caselibraryId) {
        return caseLibraryAuthorizeService.getAuthorize(caselibraryId);
    }

    @Override
    public Page<CaseLibraryVO> getHomeList(Integer pageNo, Integer pageSize) {
        RequestContext res = ContextHolder.get();
        Page<CaseLibraryVO> page = new Page<>(pageNo, pageSize);
        List<CaseLibraryVO> list = caseLibraryMapper.getHomeList(res.getCompanyId(), res.getSiteId(), page);
        if (CollectionUtils.isNotEmpty(list)) {
            page.setRecords(list);
        }
        return page;
    }


    public AccountMapVO getAccountMapVO(boolean isAdmin, Long accountId, Long siteId, List<Long> accountIds, List<Long> orgIds,

                                        String accountName, String orgName) {
        List<Long> ids = new ArrayList<>();
        Map<Long, ReportAccountRespVO> accountMap = new HashMap<>(16);
        List<ReportAccountRespVO> list = null;
        //如果是管理员(肯定是模糊查询)
        if (isAdmin) {
            list = getRangeAccounts(siteId, accountIds, orgIds, accountName, orgName);
        } else {
            //非管理员不一定是模糊查询
            list = getAccountsInManage(accountId, siteId, accountName, orgName);
        }
        //如果有用户信息,放入map中,并加用记id放入list中
        if (CollectionUtils.isNotEmpty(list)) {
            for (ReportAccountRespVO respVO : list) {
                ids.add(respVO.getUserId());
                accountMap.put(respVO.getUserId(), respVO);
            }
            AccountMapVO accountMapVO = new AccountMapVO();
            accountMapVO.setAccountIds(ids);
            accountMapVO.setAccountMap(accountMap);
            return accountMapVO;
        }
        return null;
    }

    //如果是管理员需要模糊查询内部方法
    public List<ReportAccountRespVO> getRangeAccounts(Long siteId, List<Long> accountIds, List<Long> orgIds,

                                                      String accountName, String orgName) {
        ReportRangeAccountReqVO reqVO = new ReportRangeAccountReqVO();
        reqVO.setSiteId(siteId);
        if (CollectionUtils.isNotEmpty(accountIds)) {
            reqVO.setAccountIds(accountIds);
        }
        if (CollectionUtils.isNotEmpty(orgIds)) {
            reqVO.setOrgIds(orgIds);
        }
        if (StringUtils.isNotBlank(accountName)) {
            reqVO.setAccountName(accountName);
        }
        if (StringUtils.isNotBlank(orgName)) {
            reqVO.setOrgName(orgName);
        }
        List<com.yizhi.system.application.vo.ReportAccountRespVO> list = reportClient.getRangeAccounts(reqVO);

        List<ReportAccountRespVO> vos = new ArrayList<>();
        for (com.yizhi.system.application.vo.ReportAccountRespVO it : list) {
            ReportAccountRespVO vo = new ReportAccountRespVO();
            BeanUtils.copyProperties(it,vo);
            vos.add(vo);
        }
        return vos;
    }

    //如果是非管理员查询内部方法(不管是不是模糊查询)
    public List<ReportAccountRespVO> getAccountsInManage(Long accountId, Long siteId, String accountName, String orgName) {
        HQAccountInManageParam reqVO = new HQAccountInManageParam();
        reqVO.setSiteId(siteId);
        reqVO.setModuleType(AuthzConstant.moduleType.caseLibrary);
        reqVO.setSiteId(siteId);
        if (StringUtils.isNotBlank(accountName)) {
            reqVO.setAccountName(accountName);
        }
        if (StringUtils.isNotBlank(orgName)) {
            reqVO.setOrgName(orgName);
        }
        ReportAccountRespInManageVO respInManageVO = reportClient.getRangeAccountsInManage(reqVO);
        if (null != respInManageVO) {
            if (!respInManageVO.getIsAdmin()) {
                List<com.yizhi.system.application.vo.ReportAccountRespVO> list  =  respInManageVO.getList();
                List<ReportAccountRespVO> vos = new ArrayList<>();
                for (com.yizhi.system.application.vo.ReportAccountRespVO it : list) {
                    ReportAccountRespVO vo = new ReportAccountRespVO();
                    BeanUtils.copyProperties(it,vo);
                    vos.add(vo);
                }
                return vos;
            } else {
                return getRangeAccounts(siteId, null, null, accountName, orgName);
            }
        }
        return null;
    }


    @Override
    public Page<DroolsVo> getPageByDrools(String field, String value, Page<DroolsVo> page) {
        if (StringUtils.isBlank(field)) {
            logger.info("列名不能为空！");
            return page;
        }
        if (field.equalsIgnoreCase(TaskParamsEnums.CLASSIFY.getCode())) {
            field = Constant.NAME;
            page = classifyService.getClassifyNameByDrools(field, value, page);
        } else {
            if (field.equalsIgnoreCase(TaskParamsEnums.NAME.getCode())) {
                return getPage(field, value, page);
            } else if (field.equalsIgnoreCase(TaskParamsEnums.KEYWORD.getCode())) {
                field = Constant.NAME;
                return getPage(field, value, page);
            }
        }
        return page;
    }

    public Page getPage(String field, String value, Page<DroolsVo> page) {
        RequestContext requestContext = ContextHolder.get();
        Long siteId = requestContext.getSiteId();
        Long companyId = requestContext.getCompanyId();

        CaseLibrary caseLibrary = new CaseLibrary();
        caseLibrary.setSiteId(siteId);
        caseLibrary.setCompanyId(companyId);
        caseLibrary.setState(Constant.UP);
        EntityWrapper wrapper = new EntityWrapper(caseLibrary);
        wrapper.setSqlSelect("distinct(" + field + ")," + "id ")
                .isNotNull(field)
                .like(field, value)
                .addFilter(field + "!=''")
                .orderBy("create_time", false);

        //转换为驼峰命名
        String upperField = ClassUtil.getFieldName(field);
        List<DroolsVo> voList = null;
        List<CaseLibrary> list = this.baseMapper.selectPage(page, wrapper);
        if (!CollectionUtils.isEmpty(list)) {
            voList = new ArrayList<>(list.size());
            for (CaseLibrary a : list) {
                DroolsVo vo = new DroolsVo();
                vo.setTaskId(a.getId());
                vo.setTaskFieldValue(ClassUtil.invokeMethod(a, upperField));
                vo.setTaskParamsType(field);
                voList.add(vo);
            }
            page.setRecords(voList);
        }
        return page;
    }

    @Override
    public List<Long> getCaseIdsByName(String name) {
        RequestContext requestContext = ContextHolder.get();
        Long siteId = requestContext.getSiteId();
        Long companyId = requestContext.getCompanyId();
        List<Long> ids = null;

        CaseLibrary caseLibrary = new CaseLibrary();
        caseLibrary.setSiteId(siteId);
        caseLibrary.setCompanyId(companyId);
        caseLibrary.setState(Constant.UP);
        EntityWrapper wrapper = new EntityWrapper(caseLibrary);
        wrapper.setSqlSelect("id").like("name", name);
        List<CaseLibrary> list = this.baseMapper.selectList(wrapper);
        if (CollectionUtils.isNotEmpty(list)) {
            ids = new ArrayList<>(list.size());
            for (CaseLibrary a : list) {
                ids.add(a.getId());
            }
        }
        return ids;
    }

    /**
     * 处理多维度打分功能
     *
     * @param caseLibraryVO
     */
    public void dealCaseLibraryDimensionGradeInsert(CaseLibraryVO caseLibraryVO, RequestContext res) {
        // 如果不是 多维度，则删除所有
        if (caseLibraryVO.getScoreRule() != null && caseLibraryVO.getScoreRule() != 4) {
            CaseLibraryDimensionGrade grade = new CaseLibraryDimensionGrade();
            grade.setDeleted(1);
            EntityWrapper<CaseLibraryDimensionGrade> ew = new EntityWrapper<>();
            ew.eq("case_library_id", caseLibraryVO.getId());
            caseLibraryDimensionGradeMapper.update(grade, ew);
            return;
        }

        List<CaseLibraryDimensionGradeVO> vos = caseLibraryVO.getCaseLibraryDimensionGradeVOS();
        if (CollectionUtils.isEmpty(vos)) {
            return;
        }
        Date now = new Date();
        CaseLibraryDimensionGrade caseLibraryDimensionGrade = null;
        List<CaseLibraryDimensionGrade> listToInsert = new ArrayList<>();
        List<CaseLibraryDimensionGrade> listToUpdate = new ArrayList<>();
        List<Long> idsToKeep = new ArrayList<>();
        for (CaseLibraryDimensionGradeVO vo : vos) {
            caseLibraryDimensionGrade = new CaseLibraryDimensionGrade();
//            caseLibraryDimensionGrade.setDescription(vo.getDescription());
            caseLibraryDimensionGrade.setId(vo.getId());
            caseLibraryDimensionGrade.setCaseLibraryId(caseLibraryVO.getId());
            caseLibraryDimensionGrade.setMaxScore(vo.getMaxScore());
            caseLibraryDimensionGrade.setMinScore(vo.getMinScore());
            caseLibraryDimensionGrade.setName(vo.getName());
            caseLibraryDimensionGrade.setSort(vo.getSort());
            caseLibraryDimensionGrade.setCompanyId(res.getCompanyId());
            caseLibraryDimensionGrade.setOrgId(res.getOrgId());
            caseLibraryDimensionGrade.setSiteId(res.getSiteId());
            // 新增 保留
            if (vo.getId() == null) {
                caseLibraryDimensionGrade.setDeleted(0);
                caseLibraryDimensionGrade.setCreateById(res.getAccountId());
                caseLibraryDimensionGrade.setCreateByName(res.getAccountName());
                caseLibraryDimensionGrade.setCreateTime(now);
                listToInsert.add(caseLibraryDimensionGrade);
            }
            // 修改
            else {
                caseLibraryDimensionGrade.setUpdateById(res.getAccountId());
                caseLibraryDimensionGrade.setUpdateByName(res.getAccountName());
                caseLibraryDimensionGrade.setUpdateTime(now);
                listToUpdate.add(caseLibraryDimensionGrade);
            }
            idsToKeep.add(vo.getId());
        }
        // 删除（除了保留的）
        if (CollectionUtils.isNotEmpty(idsToKeep)) {
            CaseLibraryDimensionGrade grade = new CaseLibraryDimensionGrade();
            grade.setDeleted(1);

            EntityWrapper<CaseLibraryDimensionGrade> ew = new EntityWrapper<>();
            ew.eq("case_library_id", caseLibraryVO.getId());
            ew.notIn("id", idsToKeep);

            caseLibraryDimensionGradeMapper.update(grade, ew);
        }
        // 新增
        if (CollectionUtils.isNotEmpty(listToInsert)) {
            caseLibraryDimensionGradeService.insertBatch(listToInsert);
        }
        // 修改
        if (CollectionUtils.isNotEmpty(listToUpdate)) {
            caseLibraryDimensionGradeService.updateBatchById(listToUpdate);
        }
    }
}
