package com.yizhi.library.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.library.application.constant.Constant;
import com.yizhi.library.application.domain.Classify;
import com.yizhi.library.application.mapper.ClassifyMapper;
import com.yizhi.library.application.service.ICaseLibraryRelatedClassifyService;
import com.yizhi.library.application.service.IClassifyService;
import com.yizhi.library.application.service.IStudentCaseRelatedClassifyService;
import com.yizhi.library.application.vo.ClassifyLevelOneVO;
import com.yizhi.library.application.vo.ClassifyListVO;
import com.yizhi.library.application.vo.ClassifyVO;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-07-03
 */
@Service
public class ClassifyServiceImpl extends ServiceImpl<ClassifyMapper, Classify> implements IClassifyService {

    private Logger logger = LoggerFactory.getLogger(ClassifyServiceImpl.class);

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ClassifyMapper classifyMapper;

    @Autowired
    private ICaseLibraryRelatedClassifyService caseLibraryRelatedClassifyService;

    @Autowired
    private IStudentCaseRelatedClassifyService studentCaseRelatedClassifyService;

    @Override
    public Integer insertClassify(ClassifyVO classifyVO) {
        if (null != classifyVO) {
            Classify classify = new Classify();
            BeanUtils.copyProperties(classifyVO,classify);
            RequestContext res = ContextHolder.get();
            Integer sort = classifyMapper.getSort(classify.getParentId(), res.getCompanyId(), res.getSiteId());
            if (sort == null) {
                sort = 0;
            }
            //去除一个站点最多三个站点的限制
//            if (classify.getLevel() == 0 && classify.getParentId() == 0) {
//                if (sort > 2) {
//                    return Constant.INSERT_CLASSIFY_INSUFFICIENT;
//                }
//            }
            //同一个站点下,同一级分类不允许重名
            Classify classify1 = new Classify();
            classify1.setParentId(classify.getParentId());
            classify1.setName(classify.getName());
            classify1.setSiteId(res.getSiteId());
            EntityWrapper<Classify> entityWrapper = new EntityWrapper<>(classify1);
            entityWrapper.notIn("state", 0);
            List<Classify> list = this.selectList(entityWrapper);
            if (CollectionUtils.isNotEmpty(list)) {
                return Constant.INSERT_CLASSIFY_IDENTICAL_NAME;
            }

            classify.setId(idGenerator.generate());
            classify.setSort(sort + 1);
            if (null == classify.getParentId()) {
                classify.setParentId(0L);
            }
            classify.setCreateById(res.getAccountId());
            classify.setCreateByName(res.getAccountName());
            classify.setCreateTime(new Date());
            classify.setUpdateById(res.getAccountId());
            classify.setUpdateByName(res.getAccountName());
            classify.setUpdateTime(new Date());
            classify.setCompanyId(res.getCompanyId());
            classify.setSiteId(res.getSiteId());
            classify.setOrgId(res.getOrgId());
//            classify.setCreateById(0L);
//            classify.setUpdateById(0L);
            if (this.insert(classify)) {
                return Constant.INSERT_CLASSIFY_SUCCESS;
            } else {
                return Constant.INSERT_CLASSIFY_FAIL;
            }
        }
        return Constant.INSERT_CLASSIFY_FAIL;
    }

    @Override
    public List<ClassifyListVO> getClassifyList() {
        List<ClassifyListVO> classifyListVOList = new ArrayList<>();
        RequestContext res = ContextHolder.get();

        //获取维度分类
        Classify classify = new Classify();
        classify.setCompanyId(res.getCompanyId());
        classify.setSiteId(res.getSiteId());
        classify.setParentId(0L);
        EntityWrapper<Classify> entityWrapper = new EntityWrapper<>(classify);
        entityWrapper.notIn("state", 0);
        entityWrapper.orderBy("sort", true);
        List<Classify> classifyList = this.selectList(entityWrapper);
        //最多三个维度遍历获取一级以及二级分类
        if (CollectionUtils.isNotEmpty(classifyList)) {
            for (Classify classify1 : classifyList) {
                ClassifyListVO classifyListVO = new ClassifyListVO();
                BeanUtils.copyProperties(classify1, classifyListVO);
                //所有一级分类
                classify = new Classify();
                classify.setCompanyId(res.getCompanyId());
                classify.setSiteId(res.getSiteId());
                classify.setParentId(classify1.getId());
                classify.setLevel(1);
                EntityWrapper<Classify> entityWrapper1 = new EntityWrapper<>(classify);
                entityWrapper1.orderBy("sort", true);
                entityWrapper1.notIn("state", 0);
                List<Classify> classifyLevelOneVOList = this.selectList(entityWrapper1);
                //获取每个一级分类下的二级分类并组装到list中
                if (CollectionUtils.isNotEmpty(classifyLevelOneVOList)) {
                    List<ClassifyLevelOneVO> classifyLevelOneVOList1 = new ArrayList<>();
                    for (Classify classify2 : classifyLevelOneVOList) {
                        //新建一级分类并把分类信息copy过来,并获取二级分类列表

                        ClassifyLevelOneVO classifyLevelOneVO = new ClassifyLevelOneVO();
                        BeanUtils.copyProperties(classify2, classifyLevelOneVO);

                        if (classify1.getState() == 2) {
                            classifyLevelOneVO.setState(2);
                            classifyLevelOneVO.setShowState(0);
                        } else {
                            classifyLevelOneVO.setShowState(1);
                        }

                        List<Classify> classifyLevelTwoVOList = null;
                        classify = new Classify();
                        classify.setCompanyId(res.getCompanyId());
                        classify.setSiteId(res.getSiteId());
                        classify.setParentId(classify2.getId());
                        classify.setLevel(2);
                        EntityWrapper<Classify> entityWrapper2 = new EntityWrapper<>(classify);
                        entityWrapper2.notIn("state", 0);
                        entityWrapper2.orderBy("sort", true);
                        classifyLevelTwoVOList = this.selectList(entityWrapper2);
                        if (CollectionUtils.isNotEmpty(classifyLevelTwoVOList)) {
                            List<ClassifyVO> classifyVOList = new ArrayList<>();
//                            BeanUtils.copyProperties(classifyLevelTwoVOList, classifyVOList);
                            classifyVOList = JSON.parseArray(JSON.toJSONString(classifyLevelTwoVOList), ClassifyVO.class);
                            for (ClassifyVO classifyVO : classifyVOList) {
                                classifyVO.setKey(classifyVO.getId().toString());
                                if (classifyLevelOneVO.getState() == 2) {
                                    classifyVO.setState(2);
                                    classifyVO.setShowState(0);
                                } else {
                                    classifyVO.setShowState(1);
                                }
                                if (studentCaseRelatedClassifyService.getIsRelated(classifyVO.getId())) {
                                    classifyVO.setCanDelete(0);
                                } else {
                                    classifyVO.setCanDelete(1);
                                }
                            }
                            classifyLevelOneVO.setChildren(classifyVOList);
                            //如果有子分类,不能删除
                            classifyLevelOneVO.setCanDelete(0);
                        } else {
                            //如果没有子分类,查找是否被提交案例关联,关联了不能被删除,没关联可以被删除
                            if (studentCaseRelatedClassifyService.getIsRelated(classify2.getId())) {
                                classifyLevelOneVO.setCanDelete(0);
                            } else {
                                classifyLevelOneVO.setCanDelete(1);
                            }
                        }
                        classifyLevelOneVO.setKey(classify2.getId().toString());
                        //组装一级分类list
                        classifyLevelOneVOList1.add(classifyLevelOneVO);
                    }
                    //如果一级分类list不为空,组装到维度类中
                    if (CollectionUtils.isNotEmpty(classifyLevelOneVOList1)) {
                        classifyListVO.setChildren(classifyLevelOneVOList1);
                        classifyListVO.setCanDelete(0);
                    }
                } else {
                    if (caseLibraryRelatedClassifyService.getIsRelated(classifyListVO.getId())) {
                        classifyListVO.setCanDelete(0);
                    } else {
                        classifyListVO.setCanDelete(1);
                    }
                }
                //将维度类放入结果list中
                classifyListVOList.add(classifyListVO);
            }
            return classifyListVOList;
        }
        return null;
    }

    @Override
    public Classify getPreClassify(Long classifyId, Long parentId, Integer sort, Long companyId, Long siteId) {

        Page<Classify> page = new Page<>(1, 1);

        List<Classify> list = classifyMapper.getPreClassify(parentId, sort, companyId, siteId, page);
        if (CollectionUtils.isNotEmpty(list)) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public Classify getSuffixClassify(Long classifyId, Long parentId, Integer sort, Long companyId, Long siteId) {

        Page<Classify> page = new Page<>(1, 1);

        List<Classify> list = classifyMapper.getSuffixClassify(parentId, sort, companyId, siteId, page);

        if (CollectionUtils.isNotEmpty(list)) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public Page<DroolsVo> getClassifyNameByDrools(String field, String value, Page<DroolsVo> page) {
        if (StringUtils.isEmpty(field)) {
            logger.info("列名不能为空！");
            return page;
        }
        RequestContext requestContext = ContextHolder.get();
        Long siteId = requestContext.getSiteId();
        Long companyId = requestContext.getCompanyId();
        List<DroolsVo> voList = null;

        Classify classify = new Classify();
        classify.setSiteId(siteId);
        classify.setCompanyId(companyId);
        EntityWrapper wrapper = new EntityWrapper(classify);
        wrapper.setSqlSelect("distinct(" + field + "),"+"id ")
                .isNotNull(field)
                .like(field, value)
                .ne("state", Constant.DELETED)
                .addFilter(field + "!=''")
                .orderBy("create_time", false);

        List<Classify> list = this.baseMapper.selectPage(page, wrapper);
        if (!CollectionUtils.isEmpty(list)) {
            voList = new ArrayList<>(list.size());
            for (Classify a : list) {
                DroolsVo vo = new DroolsVo();
                vo.setTaskId(a.getId());
                vo.setTaskFieldValue(a.getName());
                vo.setTaskParamsType(field);
                voList.add(vo);
            }
        }
        page.setRecords(voList);
        return page;
    }
}
