package com.yizhi.library.application.service.impl;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.core.application.cache.CacheNamespace;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.library.application.domain.CommentReply;
import com.yizhi.library.application.mapper.CommentReplyMapper;
import com.yizhi.library.application.service.ICommentReplyService;
import com.yizhi.library.application.util.Util;
import com.yizhi.library.application.vo.CommentReplyVO;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.util.application.json.JsonUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-07-12
 */
@Service
public class CommentReplyServiceImpl extends ServiceImpl<CommentReplyMapper, CommentReply> implements ICommentReplyService {

    @Autowired
    private CommentReplyMapper commentReplyMapper;

    @Autowired
    private AccountClient accountClient;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private Util util;

    @Override
    public Page<CommentReplyVO> getReplyList(Long commentId, Integer pageNo, Integer pageSize) {

        RequestContext res = ContextHolder.get();

        Page<CommentReplyVO> page = new Page<>(pageNo, pageSize);
        List<CommentReplyVO> list = commentReplyMapper.getReplyList(commentId, page);
        //获取评论人和被评论人id并获取信息放入map中
        List<Long> accountIds = new ArrayList<>();
        Map<Long, AccountVO> accountVOMap = new HashMap<>(16);
        if (CollectionUtils.isNotEmpty(list)) {
            for (CommentReplyVO commentReplyVO : list) {
                if (!accountIds.contains(commentReplyVO.getCommenterId())) {
                    accountIds.add(commentReplyVO.getCommenterId());
                }
                if (!accountIds.contains(commentReplyVO.getReplyAccountId())) {
                    accountIds.add(commentReplyVO.getReplyAccountId());
                }
            }
            List<AccountVO> accountVOList = accountClient.idsGet(accountIds);
            if (CollectionUtils.isNotEmpty(accountVOList)) {
                for (AccountVO accountVO : accountVOList) {
                    accountVOMap.put(accountVO.getId(), accountVO);
                }
            }

            //获取敏感词
            Set<String> sensitiveWordList = new HashSet<>();
            String sensitiveValues = (String) redisCache.hget(CacheNamespace.REDIS_SENSITIVE_WORD_INFO, res.getCompanyId().toString());
            if (StringUtils.isNotBlank(sensitiveValues)) {
                sensitiveWordList = JsonUtil.json2Ojbect(sensitiveValues, Set.class);
            }

            //将用户名放入返回结果中
            for (CommentReplyVO commentReplyVO : list) {

                //过滤敏感词
                if (CollectionUtils.isNotEmpty(sensitiveWordList)) {
                    if (null != commentReplyVO.getContent()) {
                        commentReplyVO.setContent(util.filterWord(commentReplyVO.getContent(), sensitiveWordList));
                    }
                }

                if (null != accountVOMap.get(commentReplyVO.getCommenterId())) {
                    commentReplyVO.setCommenterName(accountVOMap.get(commentReplyVO.getCommenterId()).getName());
                    commentReplyVO.setCommenterFullName(
                            accountVOMap.get(commentReplyVO.getCommenterId()).getFullName() == null ? accountVOMap.get(commentReplyVO.getCommenterId()).getName() : accountVOMap.get(commentReplyVO.getCommenterId()).getFullName());
                    commentReplyVO.setHeadPortrait(accountVOMap.get(commentReplyVO.getCommenterId()).getHeadPortrait());
                }
                if (null != accountVOMap.get(commentReplyVO.getReplyAccountId())) {
                    commentReplyVO.setReplyAccountName(accountVOMap.get(commentReplyVO.getReplyAccountId()).getName());
                    commentReplyVO.setReplyAccountFullName(
                            accountVOMap.get(commentReplyVO.getReplyAccountId()).getFullName() == null ?
                                    accountVOMap.get(commentReplyVO.getReplyAccountId()).getName() : accountVOMap.get(commentReplyVO.getReplyAccountId()).getFullName());
                }
            }
            page.setRecords(list);
        }
        return page;
    }

    @Override
    public boolean putDown(Long id) {
        return commentReplyMapper.putDown(id);
    }
}
