package com.yizhi.library.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.cache.CacheNamespace;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.library.application.domain.Comment;
import com.yizhi.library.application.domain.CommentAdmire;
import com.yizhi.library.application.domain.CommentReply;
import com.yizhi.library.application.mapper.CommentMapper;
import com.yizhi.library.application.mapper.CommentReplyMapper;
import com.yizhi.library.application.param.CommentParam;
import com.yizhi.library.application.service.ICommentAdmireService;
import com.yizhi.library.application.service.ICommentReplyService;
import com.yizhi.library.application.service.ICommentService;
import com.yizhi.library.application.util.AccountUtil;
import com.yizhi.library.application.util.Util;
import com.yizhi.library.application.vo.*;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.util.application.json.JsonUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-07-12
 */
@Service
public class CommentServiceImpl extends ServiceImpl<CommentMapper, Comment> implements ICommentService {

    @Autowired
    private ICommentReplyService commentReplyService;

    @Autowired
    private ICommentAdmireService commentAdmireService;

    @Autowired
    private CommentMapper commentMapper;

    @Autowired
    IdGenerator idGenerator;

    @Autowired
    private AccountClient accountClient;

    @Autowired
    private CaseLibraryServiceImpl caseLibraryServiceImpl;

    @Autowired
    private CommentReplyMapper commentReplyMapper;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private Util util;

    @Autowired
    private AccountUtil accountUtil;

    @Override
    public boolean comment(CommentParam commentParam) {

        RequestContext res = ContextHolder.get();

        //插入评论记录
        if (commentParam.getType() == 1) {
            Comment comment = new Comment();
            comment.setId(idGenerator.generate());
            comment.setAccountId(res.getAccountId());
            comment.setStudentCaseId(commentParam.getStudentCaseId());
            comment.setContent(commentParam.getContent());
            comment.setCreateById(res.getAccountId());
            comment.setCreateByName(res.getAccountName());
            comment.setCreateTime(new Date());
            comment.setUpdateById(res.getAccountId());
            comment.setUpdateByName(res.getAccountName());
            comment.setUpdateTime(new Date());
            comment.setCompanyId(res.getCompanyId());
            comment.setSiteId(res.getSiteId());
            comment.setOrgId(res.getOrgId());
            comment.setState(1);
            return this.insert(comment);
            //插入回复记录
        } else if (commentParam.getType() == 2) {
            CommentReply commentReply = new CommentReply();
            commentReply.setId(idGenerator.generate());
            commentReply.setCommentId(commentParam.getCommentId());
            commentReply.setCommenterId(res.getAccountId());
            commentReply.setReplyAccountId(commentParam.getReplyAccountId());
            commentReply.setContent(commentParam.getContent());
            commentReply.setCreateById(res.getAccountId());
            commentReply.setCreateByName(res.getAccountName());
            commentReply.setCreateTime(new Date());
            commentReply.setUpdateById(res.getAccountId());
            commentReply.setUpdateByName(res.getAccountName());
            commentReply.setUpdateTime(new Date());
            commentReply.setState(1);
            return commentReplyService.insert(commentReply);
        }

        return false;
    }

    @Override
    public boolean admireComment(Long commentId) {

        RequestContext res = ContextHolder.get();

        CommentAdmire commentAdmire = new CommentAdmire();
        commentAdmire.setId(idGenerator.generate());
        commentAdmire.setAccountId(res.getAccountId());
        commentAdmire.setCommentId(commentId);
        commentAdmire.setCreateById(res.getAccountId());
        commentAdmire.setCreateByName(res.getAccountName());
        commentAdmire.setCreateTime(new Date());
        commentAdmire.setUpdateById(res.getAccountId());
        commentAdmire.setUpdateByName(res.getAccountName());
        commentAdmire.setUpdateTime(new Date());
        commentAdmire.setState(1);

        return commentAdmireService.insert(commentAdmire);
    }

    @Override
    public boolean cancelAdmireComment(Long commentId) {

        RequestContext res = ContextHolder.get();

        CommentAdmire commentAdmire = new CommentAdmire();
        commentAdmire.setCommentId(commentId);
        commentAdmire.setAccountId(res.getAccountId());
        commentAdmire.setState(1);

        return commentAdmireService.delete(new EntityWrapper<>(commentAdmire));
    }

    @Override
    public Page<CommentVO> getCommentList(Long studentCaseId, Integer pageNo, Integer pageSize) {

        RequestContext res = ContextHolder.get();

        Page<CommentVO> page = new Page<>(pageNo, pageSize);
        List<CommentVO> list = commentMapper.getCommentList(studentCaseId, res.getAccountId(), page);
        List<Long> accountIds = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(list)) {
            for (CommentVO commentVO : list) {
                Page<CommentReplyVO> replyVOPage = new Page<>(1, 5);
                //获取5条回复
                List<CommentReplyVO> children = commentReplyMapper.getReplyList(commentVO.getId(), replyVOPage);
                //需要获得用户信息的评论人id
                if (!accountIds.contains(commentVO.getAccountId())) {
                    accountIds.add(commentVO.getAccountId());
                }
                //需要获得用户信息的回复人id
                if (CollectionUtils.isNotEmpty(children)) {
                    for (CommentReplyVO commentReplyVO : children) {
                        if (!accountIds.contains(commentReplyVO.getCommenterId())) {
                            accountIds.add(commentReplyVO.getCommenterId());
                        }
                    }
                    commentVO.setChildren(children);
                }
            }
        }
        //根据用户ids获取用户信息并放到map中
        List<AccountVO> accountVOList = accountClient.idsGet(accountIds);
        Map<Long, AccountVO> accountVOMap = new HashMap<>(16);
        if (CollectionUtils.isNotEmpty(accountVOList)) {
            for (AccountVO accountVO : accountVOList) {
                accountVOMap.put(accountVO.getId(), accountVO);
            }
        }
        getRecords(page, list, accountVOMap, res.getCompanyId());
        return page;
    }

    @Override
    public Page<CommentVO> getCommentManageList(Long studentCaseId, String name, Integer state, Integer pageNo, Integer pageSize) {

        RequestContext res = ContextHolder.get();
        RangeAccountsVO rangeAccountsVO = accountUtil.getRangeAccounts(res.isAdmin(), res.getAccountId(), res.getSiteId(), name, null);

        List<Long> accountIds = rangeAccountsVO.getAccountIds();

        Map<Long, ReportAccountRespVO> accountMap = rangeAccountsVO.getAccountMap();

        Page<CommentVO> page = new Page<>(pageNo, pageSize);


        List<CommentVO> list = commentMapper.getCommentManageList(studentCaseId, accountIds, state, res.getCompanyId(), res.getSiteId(), page);


        if (CollectionUtils.isNotEmpty(list)) {

            List<Long> accountIds1 = new ArrayList<>();

            if (null == accountIds) {
                accountIds = new ArrayList<>();
            }

            if (null == accountMap) {
                accountMap = new HashMap<>(16);
            }

            //获取敏感词
            Set<String> sensitiveWordList = new HashSet<>();
            String sensitiveValues = (String) redisCache.hget(CacheNamespace.REDIS_SENSITIVE_WORD_INFO, res.getCompanyId().toString());
            if (StringUtils.isNotBlank(sensitiveValues)) {
                sensitiveWordList = JsonUtil.json2Ojbect(sensitiveValues, Set.class);
            }
            for (CommentVO commentVO : list) {
                //过滤敏感词
                if (CollectionUtils.isNotEmpty(sensitiveWordList)) {
                    if (null != commentVO.getContent()) {
                        commentVO.setContent(util.filterWord(commentVO.getContent(), sensitiveWordList));
                    }
                }
                if (!accountIds.contains(commentVO.getAccountId())) {
                    accountIds1.add(commentVO.getAccountId());
                }

                //获取回复列表
                List<CommentReplyVO> commentReplyList = commentMapper.getCommentReplyList(accountIds, state, commentVO.getId());
                if (CollectionUtils.isNotEmpty(commentReplyList)) {
                    for (CommentReplyVO commentReplyVO : commentReplyList) {

                        commentReplyVO.setKey(commentReplyVO.getId().toString());

                        //过滤敏感词
                        if (CollectionUtils.isNotEmpty(sensitiveWordList)) {
                            if (StringUtils.isNotBlank(commentReplyVO.getContent())) {
                                commentReplyVO.setContent(util.filterWord(commentReplyVO.getContent(), sensitiveWordList));
                            }
                        }
                        //需要获得用户信息的回复人id及被回复人id
                        if (!accountIds.contains(commentReplyVO.getCommenterId()) && !accountIds1.contains(commentReplyVO.getCommenterId())) {
                            accountIds1.add(commentReplyVO.getCommenterId());
                        }
                        if (!accountIds.contains(commentReplyVO.getReplyAccountId()) && !accountIds1.contains(commentReplyVO.getReplyAccountId())) {
                            accountIds1.add(commentReplyVO.getReplyAccountId());
                        }
                    }
                    commentVO.setChildren(commentReplyList);
                }
            }

            //获取用户信息
            if (CollectionUtils.isNotEmpty(accountIds1)) {
                accountMap = accountUtil.getAccountMap(accountMap, accountIds1);
            }

            //组装用户信息
            for (CommentVO commentVO : list) {

                commentVO.setKey(commentVO.getId().toString());

                Integer index = list.indexOf(commentVO) + 1;
                Integer sort = ((pageNo - 1) * pageSize) + index;
                commentVO.setIndex(sort);

                ReportAccountRespVO commenter = accountMap.get(commentVO.getAccountId());

                if (null != commenter) {
                    commentVO.setCommenterUserName(commenter.getUserName());
                    commentVO.setCommenterFullName(commenter.getUserFullName());
                }

                //组装回复用户信息
                if (CollectionUtils.isNotEmpty(commentVO.getChildren())) {
                    for (CommentReplyVO commentReplyVO : commentVO.getChildren()) {
                        ReportAccountRespVO commenter1 = accountMap.get(commentReplyVO.getCommenterId());
                        if (null != commenter1) {
                            commentReplyVO.setCommenterName(commenter1.getUserName());
                            commentReplyVO.setCommenterFullName(commenter1.getUserFullName() == null ? "--" : commenter1.getUserFullName());
                        }

                        ReportAccountRespVO replyAccount = accountMap.get(commentReplyVO.getReplyAccountId());
                        if (null != replyAccount) {
                            commentReplyVO.setReplyAccountFullName(replyAccount.getUserFullName() == null ? "--" : replyAccount.getUserFullName());
                        }
                    }
                }
            }


            page.setRecords(list);
        }


        return page;
    }

    @Override
    public List<CommentReplyVO> getCommentReplyList(String name, Integer state, Long commentId) {
        RequestContext res = ContextHolder.get();

        //获取敏感词
        Set<String> sensitiveWordList = new HashSet<>();
        String sensitiveValues = (String) redisCache.hget(CacheNamespace.REDIS_SENSITIVE_WORD_INFO, res.getCompanyId().toString());
        if (StringUtils.isNotBlank(sensitiveValues)) {
            sensitiveWordList = JsonUtil.json2Ojbect(sensitiveValues, Set.class);
        }

        List<Long> accountIds = new ArrayList<>();
        Map<Long, ReportAccountRespVO> accountMap = new HashMap<>(16);

        boolean isGetAccountMap = false;
        //如果是管理员
        if (res.isAdmin()) {
            //如果模糊查询,查出用户信息
            if (StringUtils.isNotBlank(name)) {
                isGetAccountMap = true;
            }
        } else {
            isGetAccountMap = true;
        }
        if (isGetAccountMap) {
            if (StringUtils.isNotBlank(name)) {
                name = name.trim();
            }
            AccountMapVO accountMapVO = caseLibraryServiceImpl.getAccountMapVO(res.isAdmin(), res.getAccountId(),
                    res.getSiteId(), null, null, name, null);
            if (null != accountMapVO) {
                accountIds = accountMapVO.getAccountIds();
                if (CollectionUtils.isEmpty(accountIds)) {
                    return null;
                }
                accountMap = accountMapVO.getAccountMap();
            } else {
                return null;
            }
        }


        List<CommentReplyVO> list = commentMapper.getCommentReplyList(accountIds, state, commentId);
        //管理员非模糊查询需要获得用户信息
        if (!isGetAccountMap) {
            if (CollectionUtils.isNotEmpty(list)) {
                for (CommentReplyVO commentReplyVO : list) {
                    if (!accountIds.contains(commentReplyVO.getCommenterId())) {
                        accountIds.add(commentReplyVO.getCommenterId());
                    }
                    if (!accountIds.contains(commentReplyVO.getReplyAccountId())) {
                        accountIds.add(commentReplyVO.getReplyAccountId());
                    }
                }
            }
            //根据用户ids获取用户信息并放到map中
            List<AccountVO> accountVOList = accountClient.idsGet(accountIds);
            Map<Long, AccountVO> accountVOMap = new HashMap<>(16);
            if (CollectionUtils.isNotEmpty(accountVOList)) {
                for (AccountVO accountVO : accountVOList) {
                    accountVOMap.put(accountVO.getId(), accountVO);
                }
            }

            //将用户信息组装进去
            for (CommentReplyVO commentReplyVO : list) {

                if (CollectionUtils.isNotEmpty(sensitiveWordList)) {
                    if (null != commentReplyVO.getContent()) {
                        commentReplyVO.setContent(util.filterWord(commentReplyVO.getContent(), sensitiveWordList));
                    }
                }

                //评论人
                AccountVO commentAccount = accountVOMap.get(commentReplyVO.getCommenterId());
                //被回复人
                AccountVO replyAccount = accountVOMap.get(commentReplyVO.getReplyAccountId());
                if (null != commentAccount) {
                    commentReplyVO.setCommenterName(commentAccount.getName());
                    commentReplyVO.setCommenterFullName(commentAccount.getFullName());
                }
                if (null != replyAccount) {
                    commentReplyVO.setReplyAccountFullName(replyAccount.getFullName());
                }
            }

            //其他已经获得用户信息,直接组装
        } else {
            if (CollectionUtils.isNotEmpty(list)) {
                List<Long> accountIds1 = new ArrayList<>();
                //评论人信息已经有了
                for (CommentReplyVO commentReplyVO : list) {
                    if (!accountIds.contains(commentReplyVO.getReplyAccountId())) {
                        accountIds1.add(commentReplyVO.getReplyAccountId());
                    }
                }
                List<AccountVO> accountVOList = accountClient.idsGet(accountIds1);
                Map<Long, AccountVO> accountVOMap = new HashMap<>(16);
                if (CollectionUtils.isNotEmpty(accountVOList)) {
                    for (AccountVO accountVO : accountVOList) {
                        accountVOMap.put(accountVO.getId(), accountVO);
                    }
                }
                for (CommentReplyVO commentReplyVO : list) {

                    if (CollectionUtils.isNotEmpty(sensitiveWordList)) {
                        if (null != commentReplyVO.getContent()) {
                            commentReplyVO.setContent(util.filterWord(commentReplyVO.getContent(), sensitiveWordList));
                        }
                    }

                    //评论人
                    ReportAccountRespVO respVO = accountMap.get(commentReplyVO.getCommenterId());
                    //回复人
                    ReportAccountRespVO respVO1 = accountMap.get(commentReplyVO.getReplyAccountId());

                    //如果回复人没找到,再找一次
                    AccountVO accountVO = accountVOMap.get(commentReplyVO.getReplyAccountId());

                    if (null != respVO) {
                        commentReplyVO.setCommenterName(respVO.getUserName() == null ? "" : respVO.getUserName());
                        commentReplyVO.setCommenterFullName(respVO.getUserFullName() == null ? "" : respVO.getUserFullName());
                    }
                    if (null != respVO1) {
                        commentReplyVO.setReplyAccountFullName(respVO1.getUserFullName());
                    } else if (null != accountVO) {
                        commentReplyVO.setReplyAccountFullName(accountVO.getFullName());
                    } else {
                        commentReplyVO.setReplyAccountFullName("--");
                    }
                }
            }
        }

        return list;
    }

    @Override
    public boolean putDown(Long id, Integer type) {
        if (type == 1) {
            return commentMapper.putDown(id);
        } else if (type == 2) {
            return commentReplyService.putDown(id);
        }
        return false;
    }

    @Override
    public boolean deleteCommentOrReply(Long id, Integer type) {
        if (type == 1) {
            this.deleteById(id);
        } else if (type == 2) {
            commentReplyService.deleteById(id);
        }
        return false;
    }

    @Override
    public boolean release(Long id, Integer type) {

        if (type == 1) {
            return commentMapper.release(id);
        } else if (type == 2) {
            return commentReplyMapper.release(id);
        }
        return false;
    }

    @Override
    public List<CommentVO> downloadList(RequestContext res, Long studentCaseId, String name, Integer state) {

        //获取敏感词
        Set<String> sensitiveWordList = new HashSet<>();
        String sensitiveValues = (String) redisCache.hget(CacheNamespace.REDIS_SENSITIVE_WORD_INFO, res.getCompanyId().toString());
        if (StringUtils.isNotBlank(sensitiveValues)) {
            sensitiveWordList = JsonUtil.json2Ojbect(sensitiveValues, Set.class);
        }

        List<Long> accountIds = new ArrayList<>();
        Map<Long, ReportAccountRespVO> accountMap = new HashMap<>(16);

        boolean isGetAccountMap = false;
        //如果是管理员
        if (res.isAdmin()) {
            //如果模糊查询,查出用户信息
            if (StringUtils.isNotBlank(name)) {
                isGetAccountMap = true;
            }
        } else {
            isGetAccountMap = true;
        }
        if (isGetAccountMap) {
            if (StringUtils.isNotBlank(name)) {
                name = name.trim();
            }
            AccountMapVO accountMapVO = caseLibraryServiceImpl.getAccountMapVO(res.isAdmin(), res.getAccountId(),
                    res.getSiteId(), null, null, name, null);
            if (null != accountMapVO) {
                accountIds = accountMapVO.getAccountIds();
                if (CollectionUtils.isEmpty(accountIds)) {
                    return null;
                }
                accountMap = accountMapVO.getAccountMap();
            } else {
                return null;
            }
        }

        Page<CommentVO> page = new Page<>(1, Integer.MAX_VALUE);
        //评论列表
        List<CommentVO> list = commentMapper.getCommentManageList(studentCaseId, accountIds, state, res.getCompanyId(), res.getSiteId(), page);

        //管理员非模糊查询需要获得用户信息
        if (!isGetAccountMap) {
            if (CollectionUtils.isNotEmpty(list)) {
                for (CommentVO commentVO : list) {
                    if (!accountIds.contains(commentVO.getAccountId())) {
                        accountIds.add(commentVO.getAccountId());
                    }
                }
            } else {
                return null;
            }
            //根据用户ids获取用户信息并放到map中
            List<AccountVO> accountVOList = accountClient.idsGet(accountIds);
            Map<Long, AccountVO> accountVOMap = new HashMap<>(16);
            if (CollectionUtils.isNotEmpty(accountVOList)) {
                for (AccountVO accountVO : accountVOList) {
                    accountVOMap.put(accountVO.getId(), accountVO);
                }
            } else {
                return null;
            }
            for (CommentVO commentVO : list) {
                AccountVO accountVO = accountVOMap.get(commentVO.getAccountId());
                if (null != accountVO) {
                    commentVO.setCommenterUserName(accountVO.getName());
                    commentVO.setCommenterFullName(accountVO.getFullName());
                }
            }
            //其他已经获得用户信息,直接组装
        } else {
            if (CollectionUtils.isNotEmpty(list)) {
                for (CommentVO commentVO : list) {
                    ReportAccountRespVO respVO = accountMap.get(commentVO.getAccountId());
                    if (null != respVO) {
                        commentVO.setCommenterUserName(respVO.getUserName() == null ? "" : respVO.getUserName());
                        commentVO.setCommenterFullName(respVO.getUserFullName() == null ? "" : respVO.getUserFullName());
                    }
                }
            } else {
                return null;
            }
        }

        //组装结果
        List<CommentVO> resultList = new ArrayList<>();

        for (CommentVO commentVO : list) {
            //过滤评论敏感词
            if (CollectionUtils.isNotEmpty(sensitiveWordList)) {
                if (null != commentVO.getContent()) {
                    commentVO.setContent(util.filterWord(commentVO.getContent(), sensitiveWordList));
                }
            }

            resultList.add(commentVO);
            //获取该评论的回复并组装
            List<CommentVO> commentReplyList = commentMapper.getCommentReplyDownLoadList(commentVO.getId(), state, accountIds);

            if (CollectionUtils.isNotEmpty(commentReplyList)) {
                if (!isGetAccountMap) {
                    List<Long> accountIds1 = new ArrayList<>();
                    //如果是管理员非模糊查询,
                    for (CommentVO commentVO1 : commentReplyList) {
                        if (!accountIds1.contains(commentVO1.getAccountId())) {
                            accountIds1.add(commentVO1.getAccountId());
                        }
                        if (!accountIds1.contains(commentVO1.getReplyAccountId())) {
                            accountIds1.add(commentVO1.getReplyAccountId());
                        }
                    }

                    List<AccountVO> accountVOList = accountClient.idsGet(accountIds1);
                    Map<Long, AccountVO> accountVOMap = new HashMap<>(16);
                    if (CollectionUtils.isNotEmpty(accountVOList)) {
                        for (AccountVO accountVO : accountVOList) {
                            accountVOMap.put(accountVO.getId(), accountVO);
                        }
                    }
                    for (CommentVO commentVO1 : commentReplyList) {
                        //过滤回复敏感词
                        if (CollectionUtils.isNotEmpty(sensitiveWordList)) {
                            if (null != commentVO1.getContent()) {
                                commentVO1.setContent(util.filterWord(commentVO1.getContent(), sensitiveWordList));
                            }
                        }
                        AccountVO commentAccountVO = accountVOMap.get(commentVO1.getAccountId());
                        AccountVO replyAccountVO = accountVOMap.get(commentVO1.getReplyAccountId());
                        if (null != commentAccountVO) {
                            commentVO1.setCommenterUserName(commentAccountVO.getName());
                            commentVO1.setCommenterFullName(commentAccountVO.getFullName());
                        }
                        if (null != replyAccountVO) {
                            commentVO1.setReplyAccountFullName(replyAccountVO.getFullName());
                        }
                        resultList.add(commentVO1);
                    }
                } else {
                    List<Long> accountIds1 = new ArrayList<>();
                    for (CommentVO commentVO1 : commentReplyList) {
                        if (!accountIds.contains(commentVO1.getReplyAccountId())) {
                            accountIds1.add(commentVO1.getReplyAccountId());
                        }
                    }
                    List<AccountVO> accountList = accountClient.idsGet(accountIds1);
                    Map<Long, AccountVO> accountVOMap = new HashMap<>(16);
                    if (CollectionUtils.isNotEmpty(accountList)) {
                        for (AccountVO accountVO : accountList) {
                            accountVOMap.put(accountVO.getId(), accountVO);
                        }
                    }
                    for (CommentVO commentVO1 : commentReplyList) {

                        //过滤回复敏感词
                        if (CollectionUtils.isNotEmpty(sensitiveWordList)) {
                            if (null != commentVO1.getContent()) {
                                commentVO1.setContent(util.filterWord(commentVO1.getContent(), sensitiveWordList));
                            }
                        }

                        ReportAccountRespVO commentAccountVO = accountMap.get(commentVO1.getAccountId());
                        ReportAccountRespVO replyAccountVO = accountMap.get(commentVO1.getReplyAccountId());
                        AccountVO replyAccountVO1 = null;
                        if (null == replyAccountVO) {
                            replyAccountVO1 = accountVOMap.get(commentVO1.getReplyAccountId());
                        }

                        if (null != commentAccountVO) {
                            commentVO1.setCommenterFullName(commentAccountVO.getUserFullName());
                            commentVO1.setCommenterUserName(commentAccountVO.getUserName());
                        }
                        if (null != replyAccountVO) {
                            commentVO1.setReplyAccountFullName(replyAccountVO.getUserFullName());
                        } else if (null != replyAccountVO1) {
                            commentVO1.setReplyAccountFullName(replyAccountVO1.getFullName());
                        }
                        resultList.add(commentVO1);
                    }
                }

            }
        }
        return resultList;
    }

    private void getRecords(Page<CommentVO> page, List<CommentVO> list, Map<Long, AccountVO> accountVOMap, Long companyId) {
        if (CollectionUtils.isNotEmpty(list)) {
            Set<String> sensitiveWordList = new HashSet<>();
            String sensitiveValues = (String) redisCache.hget(CacheNamespace.REDIS_SENSITIVE_WORD_INFO, companyId.toString());
            if (StringUtils.isNotBlank(sensitiveValues)) {
                sensitiveWordList = JsonUtil.json2Ojbect(sensitiveValues, Set.class);
            }

            for (CommentVO commentVO : list) {
                AccountVO accountVO = accountVOMap.get(commentVO.getAccountId());
                //组装评论人信息
                if (null != accountVO) {
                    commentVO.setCommenterUserName(accountVO.getName());
                    commentVO.setCommenterFullName(accountVO.getFullName());
                    commentVO.setHeadPortrait(accountVO.getHeadPortrait() == null ? "--" : accountVO.getHeadPortrait());
                }
                //敏感词过滤评论内容
                if (CollectionUtils.isNotEmpty(sensitiveWordList)) {
                    if (null != commentVO.getContent()) {
                        commentVO.setContent(util.filterWord(commentVO.getContent(), sensitiveWordList));
                    }
                }
                if (CollectionUtils.isNotEmpty(commentVO.getChildren())) {
                    for (CommentReplyVO commentReplyVO : commentVO.getChildren()) {
                        AccountVO commentAccount = accountVOMap.get(commentReplyVO.getCommenterId());
                        AccountVO replyAccount = accountVOMap.get(commentReplyVO.getReplyAccountId());
                        //组装回复人信息
                        if (null != commentAccount) {
                            commentReplyVO.setCommenterName(commentAccount.getName());
                            commentReplyVO.setCommenterFullName(commentAccount.getFullName());
                        }
                        //组装被回复人信息
                        if (null != replyAccount) {
                            commentReplyVO.setReplyAccountName(replyAccount.getName());
                            commentReplyVO.setReplyAccountFullName(replyAccount.getFullName());
                        }

                        //敏感词过滤回复内容
                        if (null != commentReplyVO.getContent()) {
                            commentReplyVO.setContent(util.filterWord(commentReplyVO.getContent(), sensitiveWordList));
                        }
                    }
                }

            }
        }

        page.setRecords(list);
    }

}
