package com.yizhi.library.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.comment.application.feign.PdfPagesClient;
import com.yizhi.comment.application.vo.PdfVO;
import com.yizhi.comment.application.vo.domain.PdfPagesVo;
import com.yizhi.core.application.cache.distributedlock.impl.RedisDistributedLock;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.enums.TaskParamsEnums;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.library.application.constant.Constant;
import com.yizhi.library.application.constant.MQqueuesEnum;
import com.yizhi.library.application.constant.PointEventEnum;
import com.yizhi.library.application.domain.*;
import com.yizhi.library.application.mapper.StudentCaseDimensionGradeRecordMapper;
import com.yizhi.library.application.mapper.StudentCaseMapper;
import com.yizhi.library.application.param.HomeListParam;
import com.yizhi.library.application.param.SelectionStudentCaseParam;
import com.yizhi.library.application.param.StudentCaseAuditParam;
import com.yizhi.library.application.param.StudentCaseParam;
import com.yizhi.library.application.service.*;
import com.yizhi.library.application.util.AccountUtil;
import com.yizhi.library.application.util.Util;
import com.yizhi.library.application.vo.*;
import com.yizhi.point.application.feign.PointRedisFeignClients;
import com.yizhi.point.application.vo.PointParamVO;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.system.application.vo.ReportRangeAccountReqVO;
import com.yizhi.training.application.feign.TpProjectClient;
import com.yizhi.util.application.clazz.ClassUtil;
import com.yizhi.util.application.constant.TpActivityType;
import com.yizhi.util.application.event.TrainingProjectEvent;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 学生案例表 服务实现类
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-07-09
 */
@Service
public class StudentCaseServiceImpl extends ServiceImpl<StudentCaseMapper, StudentCase> implements IStudentCaseService {

    private static final Logger LOGGER = LoggerFactory.getLogger(StudentCaseServiceImpl.class);

    @Autowired
    private StudentCaseMapper studentCaseMapper;

    @Autowired
    private TpProjectClient projectClient;

    @Autowired
    private ICaseLibraryService caseLibraryService;

    @Autowired
    private IClassifyService classifyService;

    @Autowired
    private ICaseLibraryRelatedClassifyService caseLibraryRelatedClassifyService;

    @Autowired
    private IStudentCaseRelatedClassifyService studentCaseRelatedClassifyService;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private AccountClient accountClient;

    @Autowired
    private ICaseStudyRecodeService caseStudyRecodeService;

    @Autowired
    private AccountUtil accountUtil;

    @Autowired
    private IInformRecordeService informRecordeService;

    @Autowired
    private TpProjectClient tpProjectClient;

    @Autowired
    TaskExecutor taskExecutor;

    @Autowired
    private PointRedisFeignClients pointRedisFeignClients;

    @Autowired
    private AmqpTemplate amqpTemplate;

    @Autowired
    private ReportClient reportClient;

    @Autowired
    private IStudentCaseScoreRecordeService scoreRecordeService;

    @Autowired
    private ICaseLibraryAuthorizeService caseLibraryAuthorizeService;

    @Autowired
    private RedisDistributedLock redisDistributedLock;

    @Autowired
    private IScorerService scorerService;

    @Autowired
    private IStudentCaseScoreRecordeService studentCaseScoreRecordeService;

    @Autowired
    private Util util;

    @Autowired
    private IStudentCaseAccessaryService studentCaseAccessaryService;

    @Autowired
    private CloudEventPublisher cloudEventPublisher;

    @Autowired
    private PdfPagesClient pdfPagesClient;

    @Autowired
    private ICaseLibraryDimensionGradeService caseLibraryDimensionGradeService;

    @Autowired
    private IStudentCaseDimensionGradeRecordService studentCaseDimensionGradeRecordService;

    @Autowired
    private StudentCaseDimensionGradeRecordMapper studentCaseDimensionGradeRecordMapper;

    @Override
    public Page<CaseLibraryVO> myCaseLibraryList(Integer pageNo, Integer pageSize) {
        RequestContext res = ContextHolder.get();

        Page<CaseLibraryVO> page = new Page<>(pageNo, pageSize);

        //找出该用户指定可见的案例活动id
        List<Long> relationIds = res.getRelationIds();
        if (null == relationIds) {
            relationIds = new ArrayList<>();
        }

        List<Long> caseLibraryIds = null;

        if (CollectionUtils.isNotEmpty(relationIds)) {
            caseLibraryIds = caseLibraryAuthorizeService.getAuthorizeIds(res.getAccountId(), relationIds);
        }

        List<Long> submitCaseLibraryIds = studentCaseMapper.getSubmitCaseLibraryIds(res.getCompanyId(), res.getSiteId(), res.getAccountId());
        if (CollectionUtils.isNotEmpty(submitCaseLibraryIds)) {
            if (CollectionUtils.isNotEmpty(caseLibraryIds)) {
                caseLibraryIds.addAll(submitCaseLibraryIds);
            } else {
                caseLibraryIds = new ArrayList<>();
                caseLibraryIds.addAll(submitCaseLibraryIds);
            }
        }


        List<CaseLibraryVO> list = studentCaseMapper.myCaseLibraryList(res.getAccountId(), caseLibraryIds, res.getCompanyId(), res.getSiteId(), page);


        if (CollectionUtils.isNotEmpty(list)) {

            for (CaseLibraryVO caseLibraryVO : list) {
                Date endTime = caseLibraryVO.getEndTime();
                if (System.currentTimeMillis() >= endTime.getTime()) {
                    caseLibraryVO.setIsMustSubmit(2);
                }
            }
            page.setRecords(list);
        }
        return page;
    }

    @Override
    public Integer myCaseLibraryTotal() {
        RequestContext res = ContextHolder.get();

        //找出该用户指定可见的案例活动id
        List<Long> relationIds = res.getRelationIds();
        if (null == relationIds) {
            relationIds = new ArrayList<>();
        }
        relationIds.add(res.getAccountId());

        List<Long> caseLibraryIds = null;

        if (CollectionUtils.isNotEmpty(relationIds)) {
            caseLibraryIds = caseLibraryAuthorizeService.getAuthorizeIds(res.getAccountId(), relationIds);
        }

        List<Long> submitCaseLibraryIds = studentCaseMapper.getSubmitCaseLibraryIds(res.getCompanyId(), res.getSiteId(), res.getAccountId());
        if (CollectionUtils.isNotEmpty(submitCaseLibraryIds)) {
            if (CollectionUtils.isNotEmpty(caseLibraryIds)) {
                caseLibraryIds.addAll(submitCaseLibraryIds);
            } else {
                caseLibraryIds = new ArrayList<>();
                caseLibraryIds.addAll(submitCaseLibraryIds);
            }
        }

        Integer total = studentCaseMapper.myCaseLibraryTotal(res.getAccountId(), caseLibraryIds, res.getCompanyId(), res.getSiteId());

        return total;
    }

    @Override
    public FavoriteVO getStudentCaseDetail(Long studentCaseId) {

        FavoriteVO favoriteVO = studentCaseMapper.getStudentCaseDetail(studentCaseId);
        if (null != favoriteVO && null != favoriteVO.getAuthorId()) {
            AccountVO accountVO = accountClient.findById(favoriteVO.getAuthorId());
            if (null != accountVO) {
                String fullName = accountVO.getFullName();
                favoriteVO.setAuthorFullName(fullName);
                if (org.springframework.util.StringUtils.isEmpty(fullName)){
                    favoriteVO.setAuthorFullName(accountVO.getName());
                }
            }
        }

        return favoriteVO;
    }

    @Override
    public Integer getFavoritesCount(Long companyId, Long siteId, Long accountId) {
        Integer count = studentCaseMapper.getFavoritesCount(companyId, siteId, accountId);
        return count;
    }

    @Override
    public StudentCaseVO prepareStudentCase(Long caseLibraryId) {

        RequestContext res = ContextHolder.get();

        StudentCaseVO studentCaseVO = new StudentCaseVO();
        CaseLibrary caseLibrary = studentCaseMapper.getCaseLibrary(caseLibraryId);

        if (null != caseLibrary) {
            //如果非上架状态直接返回不能提交
            if (caseLibrary.getState() != 2) {
                studentCaseVO.setResultState(0);
                return studentCaseVO;
            }
            studentCaseVO.setCaseLibraryId(caseLibraryId);
            studentCaseVO.setCaseLibraryName(caseLibrary.getName());
            studentCaseVO.setExplanation(caseLibrary.getExplanation());
            studentCaseVO.setTextEditType(caseLibrary.getTextEditType());
            studentCaseVO.setProductCover(caseLibrary.getProductCover());

            List<ClassifyListVO> classifyListVOList = getCaseLibraryClassifyListVO(caseLibraryId, res.getCompanyId(), res.getSiteId());

            if (CollectionUtils.isNotEmpty(classifyListVOList)) {
                studentCaseVO.setClassifyListVOList(classifyListVOList);
            }
        }


        return studentCaseVO;
    }

    @Override
    public boolean submitStudentCase(StudentCaseParam studentCaseParam) {

        RequestContext res = ContextHolder.get();
        if (CollectionUtils.isNotEmpty(studentCaseParam.getKeywordsList())) {
            String keyWords = getKeyWords(studentCaseParam.getKeywordsList());
            if (StringUtils.isNotBlank(keyWords)) {
                studentCaseParam.setKeywords(keyWords);
            }
        }

        StudentCase studentCase = new StudentCase();
        BeanUtils.copyProperties(studentCaseParam, studentCase);
        if (null == studentCase.getId()) {
            studentCase.setId(idGenerator.generate());
        }

        //如果是编辑,附件入库之前先删除对应附件
        if (null != studentCaseParam.getId()) {
            studentCaseAccessaryService.deleteAccessary(studentCase.getId());
        }


        //组装附件url
        if (CollectionUtils.isNotEmpty(studentCaseParam.getAccessoryUrlList())) {

            String accessoryUrl = util.getAccessoryUrl(studentCase.getId(), studentCaseParam.getAccessoryUrlList());
//            for (String url : studentCaseParam.getAccessoryUrlList()) {
//                System.out.println(studentCaseParam.getAccessoryUrlList().indexOf(url));
//                if (studentCaseParam.getAccessoryUrlList().indexOf(url) == 0) {
//                    accessoryUrl += "," + url;
//                } else {
//                    accessoryUrl += url;
//                }
//            }
            studentCase.setAccessoryUrl(accessoryUrl);
        }

        Date date = new Date();
        //如果是新建提交
        if (null == studentCaseParam.getId()) {
            studentCase.setLastModifyTime(date);
            //如果是管理端代提交
            if (null != studentCaseParam.getSubmitterId()) {
                studentCase.setCreateById(studentCaseParam.getSubmitterId());
                studentCase.setAccountId(studentCaseParam.getSubmitterId());
                studentCase.setDiplomaticAgentId(res.getAccountId());
            } else {
                studentCase.setCreateById(res.getAccountId());
                studentCase.setCreateByName(res.getAccountName());
                studentCase.setAccountId(res.getAccountId());
            }
            studentCase.setCreateTime(date);
            studentCase.setUpdateById(res.getAccountId());
            studentCase.setUpdateByName(res.getAccountName());
            studentCase.setUpdateTime(date);
            studentCase.setCompanyId(res.getCompanyId());
            studentCase.setSiteId(res.getSiteId());
            studentCase.setOrgId(res.getOrgId());
            //如果是编辑提交
        } else {
            studentCase.setLastModifyTime(date);
            studentCase.setUpdateById(res.getAccountId());
            studentCase.setUpdateByName(res.getAccountName());
            studentCase.setUpdateTime(date);
        }


        //保存,草稿状态
        if (studentCaseParam.getType() == 0) {
            studentCase.setState(4);
            //提交,待审核状态,代提交
        } else if (studentCaseParam.getType() == 1) {

            studentCase.setState(1);
            studentCase.setAuditState(0);
            studentCase.setSubmitTime(date);
            //管理端修改上架
        } else if (studentCaseParam.getType() == 3) {
            studentCase.setState(2);
        }

        //新建或者更新
        if (!this.insertOrUpdate(studentCase)) {
            return false;
        }

        //如果是提交
        if (studentCaseParam.getType() == 1) {

//            //清除之前评分空记录
//            StudentCaseScoreRecorde studentCaseScoreRecorde = new StudentCaseScoreRecorde();
//            studentCaseScoreRecorde.setStudentCaseId(studentCase.getId());
//            studentCaseScoreRecordeService.delete(new EntityWrapper<>(studentCaseScoreRecorde));
//
//            //插入评分空记录
//            this.insertScorer(studentCase.getCaseLibraryId(), studentCase.getId(), studentCase.getAccountId());

            Long caseLibraryId = studentCaseParam.getCaseLibraryId();
            Long accountId = res.getAccountId();
            Long siteId = res.getSiteId();
            Date now = new Date();
            taskExecutor.asynExecute(new AbstractTaskHandler() {

                @Override
                public void handle() {

                    try {
                        amqpTemplate.convertAndSend(Constant.TRAINING_PROJECT_EVENT_QUEUE,
                                new EventWrapper<TrainingProjectEvent>(caseLibraryId,
                                        TrainingProjectEvent.getInstance(caseLibraryId, Constant.CASELIBRARY_TRANINNING_TYPE, accountId, now, siteId, false
                                        ))
                        );
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            });
        }
        if (null != studentCase.getTextEditType() && studentCase.getTextEditType() == 2) {
            List<PdfVO> pdfVOs = new ArrayList<PdfVO>();
            PdfVO pdfVO = new PdfVO();
            pdfVO.setBizId(studentCase.getId());
            pdfVO.setBizType(TpActivityType.TYPE_SELECTED_CASE);
            pdfVO.setCompanyId(studentCase.getCompanyId());
            pdfVO.setSiteId(studentCase.getSiteId());
            pdfVO.setCreateById(studentCase.getCreateById());
            pdfVO.setOriginalFile(studentCase.getContentFileName());
            pdfVO.setOriginalFileUrl(studentCase.getPdfOssUrl());
            pdfVOs.add(pdfVO);
            try {
                if (CollectionUtils.isNotEmpty(pdfVOs)) {
                    cloudEventPublisher.publish("pdfToPng",
                            new EventWrapper(0L, pdfVOs));
                }
            } catch (Exception e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        List<StudentCaseRelatedClassifyVO> studentCaseRelatedClassifyList = studentCaseParam.getRelatedClassifyList();
        List<StudentCaseRelatedClassify> studentCaseList = new ArrayList<StudentCaseRelatedClassify>();


        if (CollectionUtils.isNotEmpty(studentCaseRelatedClassifyList)) {

            List<Long> ids = new ArrayList<>();

            for (StudentCaseRelatedClassifyVO studentCaseRelatedClassify : studentCaseRelatedClassifyList) {


                if (null == studentCaseRelatedClassify.getId()) {
                    studentCaseRelatedClassify.setId(idGenerator.generate());
                    studentCaseRelatedClassify.setStudentCaseId(studentCase.getId());
                    studentCaseRelatedClassify.setCreateById(res.getAccountId());
                    studentCaseRelatedClassify.setCreateByName(res.getAccountName());
                    studentCaseRelatedClassify.setCreateTime(new Date());
                    studentCaseRelatedClassify.setUpdateById(res.getAccountId());
                    studentCaseRelatedClassify.setUpdateByName(res.getAccountName());
                    studentCaseRelatedClassify.setUpdateTime(new Date());
                    studentCaseRelatedClassify.setCompanyId(res.getCompanyId());
                    studentCaseRelatedClassify.setSiteId(res.getSiteId());
                    studentCaseRelatedClassify.setOrgId(res.getOrgId());
                } else {
                    studentCaseRelatedClassify.setUpdateById(res.getAccountId());
                    studentCaseRelatedClassify.setUpdateByName(res.getAccountName());
                    studentCaseRelatedClassify.setUpdateTime(new Date());
                }

                StudentCaseRelatedClassify studentCas = new StudentCaseRelatedClassify();
                BeanUtils.copyProperties(studentCaseRelatedClassify, studentCas);
                ids.add(studentCas.getId());
                studentCaseList.add(studentCas);
            }
            //将其他关联分类删除
            StudentCaseRelatedClassify studentCaseRelatedClassify = new StudentCaseRelatedClassify();
            studentCaseRelatedClassify.setStudentCaseId(studentCase.getId());
            EntityWrapper<StudentCaseRelatedClassify> entityWrapper = new EntityWrapper<>(studentCaseRelatedClassify);
            entityWrapper.notIn("id", ids);
            studentCaseRelatedClassifyService.delete(entityWrapper);
            studentCaseRelatedClassifyService.insertOrUpdateBatch(studentCaseList);
            return true;
        }

        return true;
    }

    @Override
    public StudentCaseRecordListVO getSubmitRecords(Long caseLibraryId, String endTime) {

        RequestContext res = ContextHolder.get();

        StudentCaseRecordListVO studentCaseRecordListVO = new StudentCaseRecordListVO();
        //获取案例活动信息组装
        CaseLibrary caseLibrary = caseLibraryService.selectById(caseLibraryId);
        if (null != caseLibrary) {
            studentCaseRecordListVO.setCaseLibraryName(caseLibrary.getName());
            studentCaseRecordListVO.setExplanation(caseLibrary.getExplanation());
        }
        //活动是否过期
        Date date = new Date();
        if (caseLibrary.getEndTime().getTime()<date.getTime()){
            studentCaseRecordListVO.setCaseLibraryState(2);
        }else {
            studentCaseRecordListVO.setCaseLibraryState(1);
        }
        StudentCase studentCase = new StudentCase();
        studentCase.setCaseLibraryId(caseLibraryId);
        studentCase.setAccountId(res.getAccountId());
        EntityWrapper<StudentCase> entityWrapper = new EntityWrapper<>(studentCase);
        entityWrapper.orderBy("create_time", false);
        entityWrapper.notIn("state", 0);

        //获取学员案例记录组装
        List<StudentCase> studentCaseList = this.selectList(entityWrapper);
        List<StudentCaseRecordVO> recordVOList = new ArrayList<>();
        Date time = caseLibrary.getEndTime();

        if (CollectionUtils.isNotEmpty(studentCaseList)) {
            for (StudentCase studentCase1 : studentCaseList) {
                StudentCaseRecordVO studentCaseRecordVO = new StudentCaseRecordVO();

                //学员案例信息组装
                studentCaseRecordVO.setStudentCaseId(studentCase1.getId());
                studentCaseRecordVO.setLogoUrl(studentCase1.getLogoUrl());
                studentCaseRecordVO.setTitle(studentCase1.getTitle());
                studentCaseRecordVO.setLastModifyTime(studentCase1.getLastModifyTime());
                studentCaseRecordVO.setEndTime(time);

                //logo左上角状态
                //已过期
//                if (System.currentTimeMillis() > time.getTime()) {
//                    studentCaseRecordVO.setState(0);
//                } else {
                //待提交
                if (studentCase1.getState() == 4) {
                    studentCaseRecordVO.setState(1);
                } else {
                    //提交待审核
                    if (studentCase1.getAuditState() == 0) {
                        studentCaseRecordVO.setState(2);
                        //审核通过
                    } else if (studentCase1.getAuditState() == 1) {
                        //审核通过待评分
                        if (studentCase1.getScoreState() == 0) {
                            studentCaseRecordVO.setState(3);
                            //审核通过已评分
                        } else if (studentCase1.getScoreState() == 1) {
                            studentCaseRecordVO.setState(4);
                        }
                        //审核不通过
                    } else if (studentCase1.getAuditState() == 2) {
                        studentCaseRecordVO.setState(5);
                    }
                }
//                }

                recordVOList.add(studentCaseRecordVO);
            }
            if (System.currentTimeMillis() > time.getTime()) {
                studentCaseRecordListVO.setState(0);
            }
            studentCaseRecordListVO.setStudentCaseRecordList(recordVOList);
        }


        return studentCaseRecordListVO;
    }


    @Override
    public boolean studentCaseRecall(Long studentCaseId) {
        return studentCaseMapper.studentCaseRecall(studentCaseId);
    }

    @Override
    public StudentCaseViewVO studentCaseView(Long studentCaseId, Integer type) {

        RequestContext res = ContextHolder.get();

        StudentCaseViewVO studentCaseViewVO = studentCaseMapper.studentCaseView(studentCaseId);


        if (null == studentCaseViewVO) {
            return null;
        }

        //组装关键字
        if (StringUtils.isNotBlank(studentCaseViewVO.getKeywords())) {
            List<String> keyWordsList = getKeyWordsList(studentCaseViewVO.getKeywords());
            studentCaseViewVO.setKeywordsList(keyWordsList);
        }

        //组装附件url
        String accessoryUrl = studentCaseViewVO.getAccessoryUrl();

        //组装分类
        List<ClassifyListVO> classifyListVOList = getCaseLibraryClassifyListVO(studentCaseViewVO.getCaseLibraryId(), res.getCompanyId(), res.getSiteId());


        if (null == type) {
            List<StudentRelatedClassifyVO> selectedList = new ArrayList<>();

            //获得已选分类
            if (CollectionUtils.isNotEmpty(classifyListVOList)) {
                for (ClassifyListVO classifyListVO : classifyListVOList) {
                    StudentRelatedClassifyVO studentRelatedClassifyVO = new StudentRelatedClassifyVO();
                    studentRelatedClassifyVO = studentCaseRelatedClassifyService.getRelatedClassify(classifyListVO.getId(), studentCaseId);
                    if (null != studentRelatedClassifyVO) {
                        studentRelatedClassifyVO.setRootName(classifyListVO.getName());
                        if (studentRelatedClassifyVO.getLevel() == 1) {
                            studentRelatedClassifyVO.setClassifyParentName(null);
                        }
                        selectedList.add(studentRelatedClassifyVO);
                    }
                }
                //案例活动关联的所有分类列表
                studentCaseViewVO.setClassifyListVOList(classifyListVOList);
                //学员案例已选的关联分类列表
                if (CollectionUtils.isNotEmpty(selectedList)) {
                    studentCaseViewVO.setSelectedList(selectedList);
                }
            }
        } else {
            //获得已选分类
            if (CollectionUtils.isNotEmpty(classifyListVOList)) {
                for (ClassifyListVO classifyListVO : classifyListVOList) {
                    StudentRelatedClassifyVO studentRelatedClassifyVO = new StudentRelatedClassifyVO();
                    studentRelatedClassifyVO = studentCaseRelatedClassifyService.getRelatedClassify(classifyListVO.getId(), studentCaseId);
                    if (null != studentRelatedClassifyVO) {
                        List<Long> selectList = new ArrayList<>();
                        if (studentRelatedClassifyVO.getLevel() == 1) {
                            selectList.add(studentRelatedClassifyVO.getClassifyId());
                        } else if (studentRelatedClassifyVO.getLevel() == 2) {
                            selectList.add(studentRelatedClassifyVO.getClassifyParentId());
                            selectList.add(studentRelatedClassifyVO.getClassifyId());
                        }
                        classifyListVO.setSelectList(selectList);
                    }
                }
                studentCaseViewVO.setClassifyListVOList(classifyListVOList);
            }
        }

        if (StringUtils.isNotBlank(accessoryUrl)) {


            //组装附件类
            List<AccessoryUrlVO> accessoryUrlVOList = util.getAccessoryUrlList(studentCaseId);
            studentCaseViewVO.setAccessoryUrl(null);
            if (CollectionUtils.isNotEmpty(accessoryUrlVOList)) {
                studentCaseViewVO.setAccessoryUrlList(accessoryUrlVOList);
            }
        }

        List<PdfPagesVo> list = pdfPagesClient.list(studentCaseId, TpActivityType.TYPE_SELECTED_CASE);
        List<com.yizhi.library.application.vo.PdfPages> vos = new ArrayList<>();
        for (PdfPagesVo pdfPages : list) {
            com.yizhi.library.application.vo.PdfPages vo = new com.yizhi.library.application.vo.PdfPages();
            vos.add(vo);
        }
        studentCaseViewVO.setPdfPages(vos);
        // 组装多维度打分
        if (studentCaseViewVO.getScoreRule() == 4) {
            CaseLibraryDimensionGrade grade = new CaseLibraryDimensionGrade();
            grade.setCaseLibraryId(studentCaseViewVO.getCaseLibraryId());
            grade.setDeleted(0);
            EntityWrapper<CaseLibraryDimensionGrade> ew = new EntityWrapper<>(grade);
            ew.orderBy("sort");
            List<CaseLibraryDimensionGrade> dimensionGrades = caseLibraryDimensionGradeService.selectList(ew);
            if (CollectionUtils.isNotEmpty(dimensionGrades)) {
                CaseLibraryDimensionGradeVO gradeVO = null;
                List<CaseLibraryDimensionGradeVO> voList = new ArrayList<>(dimensionGrades.size());
                Double score = null;
                for (CaseLibraryDimensionGrade caseLibraryDimensionGrade : dimensionGrades) {
                    score = studentCaseDimensionGradeRecordMapper.getAverageScore(studentCaseViewVO.getAccountId(), caseLibraryDimensionGrade.getId(), studentCaseId);
                    gradeVO = new CaseLibraryDimensionGradeVO();
                    gradeVO.setId(caseLibraryDimensionGrade.getId());
                    gradeVO.setMaxScore(caseLibraryDimensionGrade.getMaxScore());
                    gradeVO.setMinScore(caseLibraryDimensionGrade.getMinScore());
                    gradeVO.setName(caseLibraryDimensionGrade.getName());
                    gradeVO.setSort(caseLibraryDimensionGrade.getSort());
                    gradeVO.setScore(NumberUtils.toInt(String.valueOf(score)));
                    voList.add(gradeVO);
                    if (studentCaseViewVO.getDimensionScore() == null) {
                        studentCaseViewVO.setDimensionScore(score);
                    } else {
                        studentCaseViewVO.setDimensionScore(score == null ? studentCaseViewVO.getDimensionScore() : score + studentCaseViewVO.getDimensionScore());
                    }
                }
                studentCaseViewVO.setCaseLibraryDimensionGradeVOS(voList);
            }
        }
        return studentCaseViewVO;
    }

    @Override
    public StudentCaseStudyVO studyStudentCase(Long studentCaseId, Integer type, Integer number, String name) {

        RequestContext res = ContextHolder.get();

        StudentCaseStudyVO caseStudyVO = null;
        //学习案例
        if (type == 0) {
            caseStudyVO = studentCaseMapper.studyStudentCase(studentCaseId, type, res.getAccountId());
            if (caseStudyVO == null) {
                caseStudyVO.setResultState(0);
            }
            if (caseStudyVO.getState() == 0 || caseStudyVO.getState() == 3) {
                caseStudyVO.setResultState(0);
                return caseStudyVO;
            }
            //审核案例
        } else {
            caseStudyVO = studentCaseMapper.studentCaseManageAuditOrScoreView(studentCaseId);
            if (null != caseStudyVO) {

                //审核
                if (type == 1) {
                    //获取上一篇,下一篇id
                    List<Long> prefixIds = studentCaseMapper.getStudentAuditPrefix(studentCaseId, caseStudyVO.getSubmitTime(), res.getAccountId(), res.getCompanyId(), res.getSiteId());
                    if (CollectionUtils.isNotEmpty(prefixIds)) {
                        caseStudyVO.setPrefixId(prefixIds.get(0));
                    }

                    List<Long> suffixIds = studentCaseMapper.getStudentAuditSuffix(studentCaseId, caseStudyVO.getSubmitTime(), res.getAccountId(), res.getCompanyId(), res.getSiteId());

                    if (CollectionUtils.isNotEmpty(suffixIds)) {
                        caseStudyVO.setSuffixId(suffixIds.get(0));
                    }
                } else if (type == 2) {
                    //评分
                    //获取上一篇,下一篇id
                    List<Long> prefixIds = studentCaseMapper.getStudentScorePrefix(studentCaseId, caseStudyVO.getSubmitTime(), res.getAccountId(), res.getCompanyId(), res.getSiteId());
                    if (CollectionUtils.isNotEmpty(prefixIds)) {
                        caseStudyVO.setPrefixId(prefixIds.get(0));
                    }

                    List<Long> suffixIds = studentCaseMapper.getStudentScoreSuffix(studentCaseId, caseStudyVO.getSubmitTime(), res.getAccountId(), res.getCompanyId(), res.getSiteId());
                    if (CollectionUtils.isNotEmpty(suffixIds)) {
                        caseStudyVO.setSuffixId(suffixIds.get(0));
                    }
                }

            }
        }


        //获取作者姓名
        AccountVO accountVO = accountClient.findById(caseStudyVO.getAuthorId());
        if (null != accountVO) {
            caseStudyVO.setAuthorName(accountVO.getFullName() == null ? "" : accountVO.getFullName());
            caseStudyVO.setAuthorUserName(accountVO.getName());
        }

        //组装附件list
        if (null != caseStudyVO && StringUtils.isNotBlank(caseStudyVO.getAccessoryUrl())) {
            List<AccessoryUrlVO> accessoryUrlVOList = util.getAccessoryUrlList(studentCaseId);
            if (CollectionUtils.isNotEmpty(accessoryUrlVOList)) {
                caseStudyVO.setAccessoryUrlList(accessoryUrlVOList);
                caseStudyVO.setAccessoryUrl(null);
            }
        }


        //如果是学习案例插入学习记录
        if (type == 0) {
            CaseStudyRecode caseStudyRecode = new CaseStudyRecode();
            caseStudyRecode.setId(idGenerator.generate());
            caseStudyRecode.setStudentCaseId(studentCaseId);
            caseStudyRecode.setAccountId(res.getAccountId());
            caseStudyRecode.setFinishState(1);
            caseStudyRecode.setFinishTime(new Date());
            caseStudyRecode.setCreateById(res.getAccountId());
            caseStudyRecode.setCreateByName(res.getAccountName());
            caseStudyRecode.setCreateTime(new Date());
            caseStudyRecode.setUpdateById(res.getAccountId());
            caseStudyRecode.setUpdateByName(res.getAccountName());
            caseStudyRecode.setUpdateTime(new Date());
            caseStudyRecode.setCompanyId(res.getCompanyId());
            caseStudyRecode.setSiteId(res.getSiteId());
            caseStudyRecode.setOrgId(res.getOrgId());
            //插入学习记录
            caseStudyRecodeService.insert(caseStudyRecode);
            //更新学习数

            String lockName = "study_student_case_" + studentCaseId;
            //获取redis锁
            if (redisDistributedLock.lock(lockName)) {
                try {
                    studentCaseMapper.updateStudyNum(studentCaseId);
                } catch (Exception e) {
                    LOGGER.error("案例学习人数更新异常" + studentCaseId, e);
                } finally {
                    redisDistributedLock.releaseLock(lockName);
                    LOGGER.info("释放reids锁" + lockName);
                }
            }

            Date now = new Date();

            taskExecutor.asynExecute(new AbstractTaskHandler() {
                @Override
                public void handle() {

                    try {
                        amqpTemplate.convertAndSend(Constant.TRAINING_PROJECT_EVENT_QUEUE,
                                new EventWrapper<TrainingProjectEvent>(studentCaseId,
                                        TrainingProjectEvent.getInstance(studentCaseId, Constant.STUDENTCASE_TRANINNING_TYPE, res.getAccountId(), now, res.getSiteId(), true
                                        ))
                        );
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            });
        }

        if (caseStudyVO.getScoreRule() != null && caseStudyVO.getScoreRule() == 4) {
            CaseLibraryDimensionGrade grade = new CaseLibraryDimensionGrade();
            grade.setCaseLibraryId(caseStudyVO.getCaseLibraryId());
            grade.setDeleted(0);
            EntityWrapper ew = new EntityWrapper<>(grade);
            ew.orderBy("sort");
            Double score;
            List<CaseLibraryDimensionGrade> dimensionGrades = caseLibraryDimensionGradeService.selectList(ew);
            if (CollectionUtils.isNotEmpty(dimensionGrades)) {
                for (CaseLibraryDimensionGrade caseLibraryDimensionGrade : dimensionGrades) {
                    score = studentCaseDimensionGradeRecordMapper.getAverageScore(caseStudyVO.getAuthorId(), caseLibraryDimensionGrade.getId(), caseStudyVO.getStudentCaseId());
                    if (caseStudyVO.getDimensionScore() == null) {
                        caseStudyVO.setDimensionScore(score);
                    } else {
                        caseStudyVO.setDimensionScore(score == null ? caseStudyVO.getDimensionScore() : score + caseStudyVO.getDimensionScore());
                    }
                }
            }
        }
        return caseStudyVO;
    }

    @Override
    public boolean studentCaseAudit(StudentCaseAuditParam studentCaseAuditParam) {

        RequestContext res = ContextHolder.get();

        StudentCase studentCase = new StudentCase();
        studentCase.setId(studentCaseAuditParam.getStudentCaseId());
        studentCase.setAuditState(studentCaseAuditParam.getAuditState());
        studentCase.setAuditEvaluate(studentCaseAuditParam.getAudiEvaluate());
        studentCase.setAuditTime(new Date());
        Integer point = null;
        Long trainingProjectId = null;
        Long accountId = null;
        Long caseLibraryId = null;
        String caseLibraryName = null;
        if (studentCaseAuditParam.getAuditState() == 1) {

            //获取案例活动设置前端显示类型,积分,用户id 1:审核通过后自动上架,2:审核通过后手动上架
            Map<String, Object> map = studentCaseMapper.getExamine(studentCaseAuditParam.getStudentCaseId());

            //获取是否自动上架
            Integer examine = (Integer) map.get("examine");
            //获取培训项目id,向培训项目发送消息
            trainingProjectId = (Long) map.get("trainingProjectId");
            //获取用户id
            accountId = (Long) map.get("accountId");
            //获取积分
            point = (Integer) map.get("point");
            //获取学员案例状态
            Integer state = (Integer) map.get("state");
            //学员案例评分状态
            Integer scoreState = (Integer) map.get("scoreState");
            //案例活动id
            caseLibraryId = (Long) map.get("caseLibraryId");
            //案例活动名称
            caseLibraryName = (String) map.get("name");

            //获取现在案例状态,如果是上架状态,则还是上架状态
//            StudentCase studentCase1 = this.selectById(studentCaseAuditParam.getStudentCaseId());

            //如果有评分状态则不改,没有评分状态,则为待评分状态
            if (scoreState == null) {
                studentCase.setScoreState(0);
            }

            if (state == 2) {
                studentCase.setState(2);
            } else {
                if (examine == 1) {
                    studentCase.setState(2);
                } else if (examine == 2) {
                    studentCase.setState(3);
                }
            }
        } else {
            //如果重新审核不通过,分数清零,评分状态改为null
            studentCase.setScoreState(null);
            studentCase.setScore(null);
            studentCase.setState(3);
        }

        boolean result = this.updateById(studentCase);

        //审核通过发放积分以及给培训项目发送消息
        if (studentCaseAuditParam.getAuditState() == 1) {

            Date time = new Date();

            //如果有积分,发放积分
            if (result && point != null && point > 0) {
                String finalCaseLibraryName = caseLibraryName;
                Long finalCaseLibraryId = caseLibraryId;
                Integer finalPoint = point;
                Long finalAccountId = accountId;
                taskExecutor.asynExecute(new AbstractTaskHandler() {
                    @Override
                    public void handle() {

                        PointParamVO pointParamVO = new PointParamVO();
                        //事件名称
                        pointParamVO.setEventName(PointEventEnum.POINTCASELIBRARY.getKey());
                        //事件类型
                        pointParamVO.setActivityType("案例活动审核通过积分发放");
                        //学习活动来源
                        pointParamVO.setActivitySource("案例活动");
                        //学习活动名称
                        pointParamVO.setActivityName(finalCaseLibraryName);
                        //积分创建时间
                        pointParamVO.setCreatePointTime(time);
                        //积分来源ID，（业务ID主键）
                        pointParamVO.setSourceId(finalCaseLibraryId);
                        //积分数量，  正数 加积分，  负数减积分
                        pointParamVO.setOperatingPoint(finalPoint);
                        //学员ID
                        pointParamVO.setAccountId(finalAccountId);
                        //学员名称
                        pointParamVO.setAccountName("");
                        //企业ID
                        pointParamVO.setCompanyId(res.getCompanyId());
                        //站点ID
                        pointParamVO.setSiteId(res.getSiteId());
                        //组织ID
                        pointParamVO.setOrgId(res.getOrgId());

                        //调用积分接口添加积分
                        String sitePointId = pointRedisFeignClients.addPointRedis(pointParamVO);

                        //发送积分获取消息
                        if (null != sitePointId) {
                            amqpTemplate.convertAndSend(MQqueuesEnum.CASELIBRARY.getKey(), sitePointId);
                        }
                    }
                });
            }

            //向培训项目发消息,完成该活动
            Long finalCaseLibraryId1 = caseLibraryId;
            Long finalAccountId1 = accountId;
            Date now = new Date();
            Long siteId = res.getSiteId();
            taskExecutor.asynExecute(new AbstractTaskHandler() {
                @Override
                public void handle() {

                    try {
                        amqpTemplate.convertAndSend(Constant.TRAINING_PROJECT_EVENT_QUEUE,
                                new EventWrapper<TrainingProjectEvent>(finalCaseLibraryId1,
                                        TrainingProjectEvent.getInstance(finalCaseLibraryId1, Constant.CASELIBRARY_TRANINNING_TYPE, finalAccountId1, now, siteId, true
                                        ))
                        );
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            });
        }


        return result;
    }

    @Override
    public boolean studentCaseScore(StudentCaseAuditParam studentCaseAuditParam) {
        LOGGER.info("studentCaseAuditParam: " + JSON.toJSONString(studentCaseAuditParam));
        boolean isAdmin = null != studentCaseAuditParam.getIsAdmin() && studentCaseAuditParam.getIsAdmin() == 1;
        if (isAdmin) {
            if (CollectionUtils.isNotEmpty(studentCaseAuditParam.getIdList())) {
                List<StudentCase> list = studentCaseMapper.selectBatchIds(studentCaseAuditParam.getIdList());
                StudentCase studentCase4Update = new StudentCase();
                for (StudentCase studentCase : list) {
                    CaseLibrary caseLibrary = caseLibraryService.selectById(studentCase.getCaseLibraryId());
                    Integer scoreRule = caseLibrary.getScoreRule();
                    // 多维度打分
                    dealScore(studentCaseAuditParam, scoreRule, studentCase, caseLibrary, isAdmin);
                    studentCase4Update.setId(studentCase.getId());
                    studentCase4Update.setScoreState(1);
                    this.baseMapper.updateById(studentCase4Update);
                }
            }
            return studentCaseMapper.studentCaseScore(studentCaseAuditParam.getStudentCaseId(), studentCaseAuditParam.getScore());
        } else {

            RequestContext res = ContextHolder.get();
            StudentCase studentCase = studentCaseMapper.selectById(studentCaseAuditParam.getStudentCaseId());
            CaseLibrary caseLibrary = caseLibraryService.selectById(studentCase.getCaseLibraryId());
            Integer scoreRule = caseLibrary.getScoreRule();

            //插入或更新评分记录
            this.insertOrUpdateScorer(studentCase.getCaseLibraryId(), studentCaseAuditParam.getStudentCaseId(),
                    res.getAccountId(), studentCaseAuditParam.getScore(), scoreRule);

            //首次评分
            if (scoreRule == 3) {


                studentCase.setScore(studentCaseAuditParam.getScore());
                studentCase.setScoreState(1);
                studentCase.setIsFinishedScore(1);
                //平均分
            } else if (scoreRule == 1) {
//                //未打分数
//                Integer count = scoreRecordeService.getUnscoredCount(studentCaseAuditParam.getStudentCaseId());
                //平均分
                Integer score = scoreRecordeService.getAverageScore(studentCase.getCaseLibraryId(), studentCase.getAccountId(), studentCaseAuditParam.getStudentCaseId());
                if (null != score) {
                    studentCase.setScore(score);
                    studentCase.setScoreState(1);
                }
                //最高分
            } else if (scoreRule == 2) {
//                //未打分数
//                Integer count = scoreRecordeService.getUnscoredCount(studentCaseAuditParam.getStudentCaseId());
                //最高分
                Integer score = scoreRecordeService.getMaxScore(studentCase.getCaseLibraryId(), studentCase.getAccountId(), studentCaseAuditParam.getStudentCaseId());
                if (null != score) {
                    studentCase.setScore(score);
                    studentCase.setScoreState(1);
                }
            } else if (scoreRule == 4) {
                // 多维度打分
                studentCase.setScoreState(1);
                dealScore(studentCaseAuditParam, scoreRule, studentCase, caseLibrary, isAdmin);
            }
            this.updateById(studentCase);
            return true;
        }

    }

    @Override
    public Page<StudentCaseStudyVO> getAuditList(Integer type, String name, Integer pageNo, Integer pageSize) {

        RequestContext res = ContextHolder.get();

        Page<StudentCaseStudyVO> page = new Page<>(pageNo, pageSize);
        List<StudentCaseStudyVO> list = new ArrayList<>();

        if (type == 1) {
            list = studentCaseMapper.getAuditCaseVO(res.getAccountId(), res.getCompanyId(), res.getSiteId(), name, pageSize, page);
        } else if (type == 2) {
            list = studentCaseMapper.getScoreCaseVO(res.getAccountId(), res.getCompanyId(), res.getSiteId(), name, pageSize, page);
        }
        if (CollectionUtils.isNotEmpty(list)) {

            Map<Long, AccountVO> accountVOMap = new HashMap<>(16);
            //获取 用户信息
            List<Long> accountIds = list.stream().map(StudentCaseStudyVO::getAuthorId).collect(Collectors.toList());
            List<AccountVO> accountVOList = accountClient.idsGet(accountIds);
            if (CollectionUtils.isNotEmpty(accountVOList)) {
                for (AccountVO accountVO : accountVOList) {
                    accountVOMap.put(accountVO.getId(), accountVO);
                }
            }
            for (StudentCaseStudyVO studyVO : list) {
                AccountVO accountVO = accountVOMap.get(studyVO.getAuthorId());
                CaseLibrary caseLibrary = caseLibraryService.selectById(studyVO.getCaseId());
                if (null != caseLibrary) {
                    studyVO.setCaseName(caseLibrary.getName());
                }
                if (null != accountVO) {
                    studyVO.setAuthorName(accountVO.getFullName());
                }
            }

            page.setRecords(list);
        }


        return page;
    }

    @Override
    public boolean batchAudit(StudentCaseAuditParam studentCaseAuditParam) {

        if (CollectionUtils.isNotEmpty(studentCaseAuditParam.getIdList())) {
            //批量通过审核
            if (studentCaseAuditParam.getAuditState() == 1) {

                for (Long id : studentCaseAuditParam.getIdList()) {
                    StudentCaseAuditParam studentCaseAuditParam1 = new StudentCaseAuditParam();
                    studentCaseAuditParam1.setStudentCaseId(id);
                    studentCaseAuditParam1.setAuditState(studentCaseAuditParam.getAuditState());
                    studentCaseAuditParam1.setAudiEvaluate(studentCaseAuditParam.getAudiEvaluate());
                    studentCaseAudit(studentCaseAuditParam1);
                }
                return true;
                //批量审核不通过
            } else if (studentCaseAuditParam.getAuditState() == 2) {
                return studentCaseMapper.batchAuditNoPass(studentCaseAuditParam.getAuditState(), studentCaseAuditParam.getAudiEvaluate(),
                        studentCaseAuditParam.getIdList());
            }

        }
        return false;
    }

    @Override
    public boolean batchScore(StudentCaseAuditParam studentCaseAuditParam) {
        LOGGER.info("studentCaseAuditParam: " + JSON.toJSONString(studentCaseAuditParam));
        boolean isAdmin = null != studentCaseAuditParam.getIsAdmin() && studentCaseAuditParam.getIsAdmin() == 1;
        if (isAdmin) {
            if (CollectionUtils.isNotEmpty(studentCaseAuditParam.getIdList())) {
                List<StudentCase> list = studentCaseMapper.selectBatchIds(studentCaseAuditParam.getIdList());
                StudentCase studentCase4Update = new StudentCase();
                for (StudentCase studentCase : list) {
                    CaseLibrary caseLibrary = caseLibraryService.selectById(studentCase.getCaseLibraryId());
                    Integer scoreRule = caseLibrary.getScoreRule();
                    // 多维度打分
                    dealScore(studentCaseAuditParam, scoreRule, studentCase, caseLibrary, isAdmin);
                    studentCase4Update.setId(studentCase.getId());
                    studentCase4Update.setScoreState(1);
                    this.baseMapper.updateById(studentCase4Update);
                }
            }
            return studentCaseMapper.batchAdminScore(studentCaseAuditParam.getScore(), studentCaseAuditParam.getIdList());
        } else {
            List<StudentCase> list = studentCaseMapper.selectBatchIds(studentCaseAuditParam.getIdList());
            if (CollectionUtils.isNotEmpty(list)) {

                RequestContext res = ContextHolder.get();

                for (StudentCase studentCase : list) {
                    CaseLibrary caseLibrary = caseLibraryService.selectById(studentCase.getCaseLibraryId());
                    Integer scoreRule = caseLibrary.getScoreRule();
                    //插入或更新评分记录
//                    scoreRecordeService.updateScoreRcorde(studentCase.getId(), res.getAccountId(), studentCaseAuditParam.getScore());
                    this.insertOrUpdateScorer(studentCase.getCaseLibraryId(), studentCase.getId(), res.getAccountId(), studentCaseAuditParam.getScore(), scoreRule);

                    //首次评分
                    if (scoreRule == 3) {

                        studentCase.setScore(studentCaseAuditParam.getScore());
                        studentCase.setScoreState(1);
                        studentCase.setIsFinishedScore(1);
                        //平均分
                    } else if (scoreRule == 1) {
                        //未打分数
//                        Integer count = scoreRecordeService.getUnscoredCount(studentCase.getId());
                        //平均分
                        Integer score = scoreRecordeService.getAverageScore(studentCase.getCaseLibraryId(), studentCase.getAccountId(), studentCase.getId());
                        if (null != score) {
                            studentCase.setScore(score);
                            studentCase.setScoreState(1);
//                            if (count == 0) {
//                                studentCase.setIsFinishedScore(1);
//                            } else {
//                                studentCase.setIsFinishedScore(0);
//                            }
                        }
                        //最高分
                    } else if (scoreRule == 2) {
                        //未打分数
//                        Integer count = scoreRecordeService.getUnscoredCount(studentCase.getId());
                        //最高分
                        Integer score = scoreRecordeService.getMaxScore(studentCase.getCaseLibraryId(), studentCase.getAccountId(), studentCase.getId());
                        if (null != score) {
                            studentCase.setScore(score);
                            studentCase.setScoreState(1);
//                            if (count == 0) {
//                                studentCase.setIsFinishedScore(1);
//                            } else {
//                                studentCase.setIsFinishedScore(0);
//                            }
                        }
                    } else if (scoreRule == 4) {
                        // 多维度打分
                        studentCase.setScoreState(1);
                        dealScore(studentCaseAuditParam, scoreRule, studentCase, caseLibrary, isAdmin);
                    }
                }
                //更新打分
                this.updateBatchById(list);

            }
            return true;
        }
    }

    @Override
    public List<StudentCaseVO> studentCaseBatchExport(List<Long> idList) {
        return studentCaseMapper.studentCaseBatchExport(idList);
    }

    @Override
    public Page<StudentCaseVO> getStudentCaseHomeList(HomeListParam homeListParam) {
        RequestContext res = ContextHolder.get();

        //如果有模糊查询
        List<Long> accountIds = null;
        String key = null;
        if (StringUtils.isNotBlank(homeListParam.getKey())) {
            key = homeListParam.getKey().trim();
            RangeAccountsVO rangeAccountsVO = accountUtil.getRangeAccounts
                    (res.isAdmin(), res.getAccountId(), res.getSiteId(), key, null);
            if (rangeAccountsVO != null) {
                accountIds = rangeAccountsVO.getAccountIds();
            }
        }
        Page<StudentCaseVO> page = new Page<>(homeListParam.getPageNo(), homeListParam.getPageSize());
        List<StudentCaseVO> list = studentCaseMapper.getStudentCaseHomeList(accountIds, key, res.getCompanyId(), res.getSiteId(), homeListParam.getStudentCaseIds(), page);
        if (CollectionUtils.isNotEmpty(list)) {
            page.setRecords(list);
        }
        return page;
    }

    @Override
    public Page<StudentCaseManageListVO> studentCaseManageList(Long caseLibraryId, String studentCaseName, String accountName,
                                                               Integer state, String orgName, Integer pageNo, Integer pageSize,
                                                               RequestContext res, Integer type) {

        Page<StudentCaseManageListVO> page = new Page<>(pageNo, pageSize);

        //通过内部方法获取用户信息
        RangeAccountsVO rangeAccountsVO = accountUtil.getRangeAccounts(res.isAdmin(), res.getAccountId(), res.getSiteId(), accountName, orgName);

        Boolean isAllAccount = rangeAccountsVO.getIsAllAccount();
        List<Long> accountIds = rangeAccountsVO.getAccountIds();
        Map<Long, ReportAccountRespVO> accountMap = rangeAccountsVO.getAccountMap();

        //需要补充查询的人员id
        List<Long> accountIds1 = null;

        Integer auditState = null;
        Integer scoreState = null;

        //待审核
        if (null != state) {
            //待审核
            if (state == 1) {
                auditState = 0;
                //审核不通过
            } else if (state == 2) {
                auditState = 2;
                //待评分
            } else if (state == 3) {
                scoreState = 0;
                //已评分
            } else if (state == 4) {
                scoreState = 1;
            }
        }


        if (StringUtils.isNotBlank(studentCaseName)) {
            studentCaseName = studentCaseName.trim();
        }

        List<StudentCaseManageListVO> list = null;
        //全站点
        if (isAllAccount) {
            if (type == 1) {
                list = studentCaseMapper.studentCaseManageList(caseLibraryId, accountIds, studentCaseName, auditState,
                        scoreState, res.getCompanyId(), res.getSiteId(), page);
            } else if (type == 2) {
                list = studentCaseMapper.studentCaseManageExport(caseLibraryId, accountIds, studentCaseName, auditState,
                        scoreState, res.getCompanyId(), res.getSiteId(), page);
            }

            if (CollectionUtils.isNotEmpty(list)) {
                accountIds1 = new ArrayList<>();
                CaseLibrary cl = caseLibraryService.selectById(caseLibraryId);
                for (StudentCaseManageListVO studentCaseManageListVO : list) {

                    //组装临时索引(审核/评分 上一篇,下一篇需要)
                    Integer index = list.indexOf(studentCaseManageListVO) + 1;
                    studentCaseManageListVO.setNumber((pageNo - 1) * pageSize + index);

                    //作者需要获得信息的id
                    if (!accountIds1.contains(studentCaseManageListVO.getAccountId())) {
                        accountIds1.add(studentCaseManageListVO.getAccountId());
                    }
                    //审核人需要获得信息的id
                    if (CollectionUtils.isNotEmpty(studentCaseManageListVO.getAuditorIds())) {
                        for (Long auditorId : studentCaseManageListVO.getAuditorIds()) {
                            if (!accountIds1.contains(auditorId)) {
                                accountIds1.add(auditorId);
                            }
                        }
                    }
                    //评分人需要获得信息的id
                    if (CollectionUtils.isNotEmpty(studentCaseManageListVO.getScorerIds())) {
                        for (Long scorerId : studentCaseManageListVO.getScorerIds()) {
                            if (!accountIds1.contains(scorerId)) {
                                accountIds1.add(scorerId);
                            }
                        }
                    }

                    // 设置 scoreRule
                    if (cl != null) {
                        studentCaseManageListVO.setScoreRule(cl.getScoreRule());
                    }
                }

                if (CollectionUtils.isNotEmpty(accountIds1)) {
                    //获取用户信息
                    if (null == accountMap) {
                        accountMap = new HashMap<>(16);
                    }
                    accountMap = accountUtil.getAccountMap(accountMap, accountIds1);
                }

                list = accountUtil.assembleStudentCaseManageList(accountMap, list);

            }
        } else {
            //非全站点,没找到用户
            if (CollectionUtils.isEmpty(accountIds)) {
                return page;
            } else {
                list = studentCaseMapper.studentCaseManageList(caseLibraryId, accountIds, studentCaseName, auditState,
                        scoreState, res.getCompanyId(), res.getSiteId(), page);
                if (CollectionUtils.isNotEmpty(list)) {
                    accountIds1 = new ArrayList<>();
                    //作者信息不需要获取了,获取审核人以及评分人信息
                    for (StudentCaseManageListVO studentCaseManageListVO : list) {

                        //组装临时索引(审核/评分 上一篇,下一篇需要)
                        Integer index = list.indexOf(studentCaseManageListVO) + 1;
                        studentCaseManageListVO.setNumber((pageNo - 1) * pageSize + index);

                        //获取审核人id
                        if (CollectionUtils.isNotEmpty(studentCaseManageListVO.getAuditorIds())) {
                            for (Long auditorId : studentCaseManageListVO.getAuditorIds()) {
                                if (!accountIds.contains(auditorId)) {
                                    accountIds1.add(auditorId);
                                }
                            }
                        }

                        //获取评分人id
                        if (CollectionUtils.isNotEmpty(studentCaseManageListVO.getScorerIds())) {
                            for (Long scoreId : studentCaseManageListVO.getScorerIds()) {
                                if (!accountIds.contains(scoreId)) {
                                    accountIds1.add(scoreId);
                                }
                            }
                        }

                        if (CollectionUtils.isNotEmpty(accountIds1)) {
                            accountMap = accountUtil.getAccountMap(accountMap, accountIds1);
                        }

                        list = accountUtil.assembleStudentCaseManageList(accountMap, list);
                    }
                }
            }
        }

        if (CollectionUtils.isNotEmpty(list)) {
            page.setRecords(list);
        }
        return page;
    }

    @Override
    public StudentCaseStudyVO studentCaseManageAuditOrScoreView(Long studentCaseId, Long caseLibraryId, String studentCaseName, String accountName, Integer state, String orgName, Integer number) {

        RequestContext res = ContextHolder.get();

        //通过内部方法获取用户信息
        RangeAccountsVO rangeAccountsVO = accountUtil.getRangeAccounts(res.isAdmin(), res.getAccountId(), res.getSiteId(), accountName, orgName);

        List<Long> accountIds = rangeAccountsVO.getAccountIds();

        Map<Long, ReportAccountRespVO> accountMap = rangeAccountsVO.getAccountMap();


        Integer auditState = null;
        Integer scoreState = null;

        //状态
        if (null != state) {
            //待审核
            if (state == 1) {
                auditState = 0;
                //审核不通过
            } else if (state == 2) {
                auditState = 2;
                //待评分
            } else if (state == 3) {
                scoreState = 0;
                //已评分
            } else if (state == 4) {
                scoreState = 1;
                //所有列表中点击评分状态
            } else if (state == 5) {
                auditState = 1;
            }
        }


        StudentCaseStudyVO studyVO = studentCaseMapper.studentCaseManageAuditOrScoreView(studentCaseId);

        if (null != studyVO) {

            //获取作者信息
            if (CollectionUtils.isNotEmpty(accountIds) && null != accountMap) {
                if (accountIds.contains(studyVO.getAuthorId())) {
                    ReportAccountRespVO account = accountMap.get(studyVO.getAuthorId());
                    if (null != account) {
                        studyVO.setAuthorName(account.getUserFullName() == null ? "" : account.getUserFullName());
                    }
                }
            } else {
                AccountVO accountVO = accountClient.findById(studyVO.getAuthorId());
                if (null != accountVO) {
                    studyVO.setAuthorName(accountVO.getFullName() == null ? "" : accountVO.getFullName());
                }
            }

            //组装附件list
            if (StringUtils.isNotBlank(studyVO.getAccessoryUrl())) {
                List<AccessoryUrlVO> accessoryUrlVOList = util.getAccessoryUrlList(studentCaseId);
                if (CollectionUtils.isNotEmpty(accessoryUrlVOList)) {
                    studyVO.setAccessoryUrlList(accessoryUrlVOList);
                    studyVO.setAccessoryUrl(null);
                }
            }

            //获取上一篇,下一篇id
            List<Long> prefixIds = studentCaseMapper.getManageAuditOrScorePrefix(caseLibraryId, studentCaseId, studyVO.getSubmitTime(), auditState, scoreState, accountIds, res.getCompanyId(), res.getSiteId());

            List<Long> suffixIds = studentCaseMapper.getManageAuditOrScoreSuffix(caseLibraryId, studentCaseId, studyVO.getSubmitTime(), auditState, scoreState, accountIds, res.getCompanyId(), res.getSiteId());

            if (CollectionUtils.isNotEmpty(prefixIds)) {
                studyVO.setPrefixId(prefixIds.get(0));
            }

            if (CollectionUtils.isNotEmpty(suffixIds)) {
                studyVO.setSuffixId(suffixIds.get(0));
            }

            // 组装多维度 打分
            if (studyVO.getScoreRule() != null && studyVO.getScoreRule() == 4) {
                CaseLibraryDimensionGrade grade = new CaseLibraryDimensionGrade();
                grade.setCaseLibraryId(studyVO.getCaseLibraryId());
                grade.setDeleted(0);
                EntityWrapper<CaseLibraryDimensionGrade> ew = new EntityWrapper<>(grade);
                ew.orderBy("sort");
                List<CaseLibraryDimensionGrade> dimensionGrades = caseLibraryDimensionGradeService.selectList(ew);
                if (CollectionUtils.isNotEmpty(dimensionGrades)) {
                    CaseLibraryDimensionGradeVO gradeVO = null;
                    List<CaseLibraryDimensionGradeVO> voList = new ArrayList<>(dimensionGrades.size());
                    Double score = null;
                    for (CaseLibraryDimensionGrade caseLibraryDimensionGrade : dimensionGrades) {
                        score = studentCaseDimensionGradeRecordMapper.getAverageScore(studyVO.getAuthorId(), caseLibraryDimensionGrade.getId(), studentCaseId);
                        gradeVO = new CaseLibraryDimensionGradeVO();
                        gradeVO.setId(caseLibraryDimensionGrade.getId());
                        gradeVO.setMaxScore(caseLibraryDimensionGrade.getMaxScore());
                        gradeVO.setMinScore(caseLibraryDimensionGrade.getMinScore());
                        gradeVO.setName(caseLibraryDimensionGrade.getName());
                        gradeVO.setSort(caseLibraryDimensionGrade.getSort());
                        gradeVO.setScore(NumberUtils.toInt(String.valueOf(score)));
                        voList.add(gradeVO);
                        if (studyVO.getDimensionScore() == null) {
                            studyVO.setDimensionScore(score);
                        } else {
                            studyVO.setDimensionScore(score == null ? studyVO.getDimensionScore() : score + studyVO.getDimensionScore());
                        }
                    }
                    studyVO.setCaseLibraryDimensionGradeVOS(voList);
                }
            }
        } else {
            studyVO = new StudentCaseStudyVO();
        }


        return studyVO;
    }

    @Override
    public List<StudentCaseManageListVO> exportAuditList(List<Long> studentCaseIds, RequestContext res) {
        List<StudentCaseManageListVO> list = studentCaseMapper.exportAuditList(studentCaseIds, res.getCompanyId(), res.getSiteId());

        if (CollectionUtils.isNotEmpty(list)) {

            List<Long> accountIds1 = new ArrayList<>();
            Map<Long, ReportAccountRespVO> accountMap = null;

            for (StudentCaseManageListVO studentCaseManageListVO : list) {

                //作者需要获得信息的id
                if (!accountIds1.contains(studentCaseManageListVO.getAccountId())) {
                    accountIds1.add(studentCaseManageListVO.getAccountId());
                }
                //审核人需要获得信息的id
                if (CollectionUtils.isNotEmpty(studentCaseManageListVO.getAuditorIds())) {
                    for (Long auditorId : studentCaseManageListVO.getAuditorIds()) {
                        if (!accountIds1.contains(auditorId)) {
                            accountIds1.add(auditorId);
                        }
                    }
                }

            }

            if (CollectionUtils.isNotEmpty(accountIds1)) {
                //获取用户信息
                if (null == accountMap) {
                    accountMap = new HashMap<>(16);
                }
                accountMap = accountUtil.getAccountMap(accountMap, accountIds1);
            }

            //组装用户信息
            list = accountUtil.assembleStudentCaseManageList(accountMap, list);
            return list;
        }
        return null;
    }

    @Override
    public List<StudentCaseManageListVO> exportScoreList(List<Long> studentCaseIds, RequestContext res) {
        List<StudentCaseManageListVO> list = studentCaseMapper.exportScoreList(studentCaseIds, res.getCompanyId(), res.getSiteId());

        if (CollectionUtils.isNotEmpty(list)) {

            List<Long> accountIds1 = new ArrayList<>();
            Map<Long, ReportAccountRespVO> accountMap = null;

            for (StudentCaseManageListVO studentCaseManageListVO : list) {

                //作者需要获得信息的id
                if (!accountIds1.contains(studentCaseManageListVO.getAccountId())) {
                    accountIds1.add(studentCaseManageListVO.getAccountId());
                }
                //评分人需要获得信息的id
                if (CollectionUtils.isNotEmpty(studentCaseManageListVO.getScorerIds())) {
                    for (Long scorerId : studentCaseManageListVO.getScorerIds()) {
                        if (!accountIds1.contains(scorerId)) {
                            accountIds1.add(scorerId);
                        }
                    }
                }

            }

            if (CollectionUtils.isNotEmpty(accountIds1)) {
                //获取用户信息
                if (null == accountMap) {
                    accountMap = new HashMap<>(16);
                }
                accountMap = accountUtil.getAccountMap(accountMap, accountIds1);
            }

            //组装用户信息
            list = accountUtil.assembleStudentCaseManageList(accountMap, list);
            return list;
        }
        return null;
    }

    @Override
    public Page<SelectionCaseListVO> getSelectionList(String studentCaseName, String accountName, Integer state, Integer pageNo, Integer pageSize, RequestContext res) {

        Page<SelectionCaseListVO> page = new Page<>(pageNo, pageSize);

        //获取是否是全平台人员
        RangeAccountsVO rangeAccountsVO = accountUtil.getRangeAccounts(res.isAdmin(), res.getAccountId(), res.getSiteId(), accountName, null);
        List<Long> accountIds = rangeAccountsVO.getAccountIds();
        Map<Long, ReportAccountRespVO> accountMap = rangeAccountsVO.getAccountMap();

        List<SelectionCaseListVO> list = null;

        //不是全平台人员
        if (!rangeAccountsVO.getIsAllAccount()) {
            if (CollectionUtils.isNotEmpty(accountIds)) {
                list = studentCaseMapper.getSelectionList(studentCaseName, state, accountIds, res.getCompanyId(), res.getSiteId(), page);
            } else {
                return page;
            }
            //全平台人员
        } else {
            list = studentCaseMapper.getSelectionList(studentCaseName, state, null, res.getCompanyId(), res.getSiteId(), page);
            //全平台的需要获取用户信息
            if (CollectionUtils.isNotEmpty(list)) {
                if (null == accountIds) {
                    accountIds = new ArrayList<>();
                }
                for (SelectionCaseListVO listVO : list) {
                    if (!accountIds.contains(listVO.getAccountId())) {
                        accountIds.add(listVO.getAccountId());
                    }
                }

                if (null == accountMap) {
                    accountMap = new HashMap<>(16);
                }
                accountMap = accountUtil.getAccountMap(accountMap, accountIds);
            }
        }
        //组装用户信息
        if (CollectionUtils.isNotEmpty(list)) {
            CaseLibraryDimensionGrade grade = new CaseLibraryDimensionGrade();
            grade.setDeleted(0);
            EntityWrapper<CaseLibraryDimensionGrade> ew = null;
            Double score;

            for (SelectionCaseListVO listVO : list) {
                ReportAccountRespVO account = accountMap.get(listVO.getAccountId());
                if (null != account) {
                    listVO.setAccountName(account.getUserName());
                    listVO.setAccountFullName(account.getUserFullName() == null ? "" : account.getUserFullName());
                }

                if (listVO.getScoreRule() != null && listVO.getScoreRule() == 4) {
                    grade.setCaseLibraryId(listVO.getCaseLibraryId());
                    ew = new EntityWrapper<>(grade);
                    ew.orderBy("sort");

                    List<CaseLibraryDimensionGrade> dimensionGrades = caseLibraryDimensionGradeService.selectList(ew);
                    if (CollectionUtils.isNotEmpty(dimensionGrades)) {
                        for (CaseLibraryDimensionGrade caseLibraryDimensionGrade : dimensionGrades) {
                            score = studentCaseDimensionGradeRecordMapper.getAverageScore(listVO.getAccountId(), caseLibraryDimensionGrade.getId(), listVO.getId());
                            if (listVO.getDimensionScore() == null) {
                                listVO.setDimensionScore(score);
                            } else {
                                listVO.setDimensionScore(score == null ? listVO.getDimensionScore() : score + listVO.getDimensionScore());
                            }
                        }
                    }
                }
            }
            page.setRecords(list);
        }

        return page;
    }

    @Override
    public Page<StudentCaseManageListVO> getInformList(String studentCaseName, String accountName, Integer pageNo, Integer pageSize, RequestContext res) {
        Page<StudentCaseManageListVO> page = new Page<>(pageNo, pageSize);

        //获取是否是全平台人员
        RangeAccountsVO rangeAccountsVO = accountUtil.getRangeAccounts(res.isAdmin(), res.getAccountId(), res.getSiteId(), accountName, null);
        List<Long> accountIds = rangeAccountsVO.getAccountIds();
        Map<Long, ReportAccountRespVO> accountMap = rangeAccountsVO.getAccountMap();

        List<StudentCaseManageListVO> list = null;

        //不是全平台人员
        if (!rangeAccountsVO.getIsAllAccount()) {
            if (CollectionUtils.isNotEmpty(accountIds)) {
                list = studentCaseMapper.getInformList(studentCaseName, accountIds, res.getCompanyId(), res.getSiteId(), page);
            } else {
                return page;
            }
            //全平台人员
        } else {
            list = studentCaseMapper.getInformList(studentCaseName, null, res.getCompanyId(), res.getSiteId(), page);
        }

        if (CollectionUtils.isNotEmpty(list)) {
            if (null == accountMap) {
                accountMap = new HashMap<>(16);
            }
            List<Long> accountIds1 = new ArrayList<>();
            for (StudentCaseManageListVO listVO : list) {
                if (CollectionUtils.isEmpty(accountIds)) {
                    if (!accountIds1.contains(listVO.getAccountId())) {
                        accountIds1.add(listVO.getAccountId());
                    }
                }

                if (!accountIds1.contains(listVO.getInformerId())) {
                    accountIds1.add(listVO.getInformerId());
                }
            }

            accountMap = accountUtil.getAccountMap(accountMap, accountIds1);
        }

        //组装用户信息
        if (CollectionUtils.isNotEmpty(list)) {
            for (StudentCaseManageListVO listVO : list) {

                //第几条数据,上一条,下一条需要
                Integer index = list.indexOf(listVO) + 1;
                listVO.setNumber((pageNo - 1) * pageSize + index);

                ReportAccountRespVO account = accountMap.get(listVO.getAccountId());
                ReportAccountRespVO informAccount = accountMap.get(listVO.getInformerId());
                if (null != account) {
                    listVO.setAccountName(account.getUserName());
                    listVO.setAccountFullName(account.getUserFullName() == null ? "" : account.getUserFullName());
                }
                if (null != informAccount) {
                    listVO.setInformerFullName(informAccount.getUserFullName());
                }
            }
            page.setRecords(list);
        }

        return page;
    }

    @Override
    public List<StudentCaseManageListVO> getInformExportList(String studentCaseName, String accountName, RequestContext res) {
        //获取是否是全平台人员
        RangeAccountsVO rangeAccountsVO = accountUtil.getRangeAccounts(res.isAdmin(), res.getAccountId(), res.getSiteId(), accountName, null);
        List<Long> accountIds = rangeAccountsVO.getAccountIds();
        Map<Long, ReportAccountRespVO> accountMap = rangeAccountsVO.getAccountMap();

        List<StudentCaseManageListVO> list = null;

        //不是全平台人员
        if (!rangeAccountsVO.getIsAllAccount()) {
            if (CollectionUtils.isNotEmpty(accountIds)) {
                list = studentCaseMapper.getInformExportList(studentCaseName, accountIds, res.getCompanyId(), res.getSiteId());
            } else {
                return list;
            }
            //全平台人员
        } else {
            list = studentCaseMapper.getInformExportList(studentCaseName, null, res.getCompanyId(), res.getSiteId());
        }

        if (CollectionUtils.isNotEmpty(list)) {
            if (null == accountMap) {
                accountMap = new HashMap<>(16);
            }
            List<Long> accountIds1 = new ArrayList<>();
            for (StudentCaseManageListVO listVO : list) {
                //如果是全站点用户信息需要查找
                if (CollectionUtils.isEmpty(accountIds)) {
                    if (!accountIds1.contains(listVO.getAccountId())) {
                        accountIds1.add(listVO.getAccountId());
                    }
                }
                //举报人信息需要查找
                if (!accountIds1.contains(listVO.getInformerId())) {
                    accountIds1.add(listVO.getInformerId());
                }

                //审核人信息需要查找
                if (CollectionUtils.isNotEmpty(listVO.getAuditorIds())) {
                    for (Long auditorId : listVO.getAuditorIds()) {
                        if (!accountIds1.contains(auditorId)) {
                            accountIds1.add(auditorId);
                        }
                    }
                }

                //评分人信息需要查找
                if (CollectionUtils.isNotEmpty(listVO.getScorerIds())) {
                    for (Long scorerId : listVO.getScorerIds()) {
                        if (!accountIds1.contains(scorerId)) {
                            accountIds1.add(scorerId);
                        }
                    }
                }
            }

            accountMap = accountUtil.getAccountMap(accountMap, accountIds1);
        }

        //组装用户,举报人,审核人,评分人信息
        if (CollectionUtils.isNotEmpty(list)) {
            for (StudentCaseManageListVO listVO : list) {

                ReportAccountRespVO account = accountMap.get(listVO.getAccountId());
                ReportAccountRespVO informAccount = accountMap.get(listVO.getInformerId());
                //作者
                if (null != account) {
                    listVO.setAccountName(account.getUserName());
                    listVO.setAccountFullName(account.getUserFullName() == null ? "" : account.getUserFullName());
                }
                //举报人
                if (null != informAccount) {
                    listVO.setInformerFullName(informAccount.getUserFullName());
                }
                //审核人
                if (CollectionUtils.isNotEmpty(listVO.getAuditorIds())) {
                    String auditorNames = "";
                    for (int i = 0; i < listVO.getAuditorIds().size(); i++) {
                        Long auditorId = listVO.getAuditorIds().get(i);
                        ReportAccountRespVO auditor = accountMap.get(auditorId);
                        if (null != auditor && StringUtils.isNotBlank(auditor.getUserFullName())) {
                            if (i == 0) {
                                auditorNames += auditor.getUserFullName();
                            } else {
                                auditorNames += "," + auditor.getUserFullName();
                            }
                        }
                    }
                    if (StringUtils.isNotBlank(auditorNames)) {
                        listVO.setAuditorFullNames(auditorNames);
                    }

                }

                //评分人
                if (CollectionUtils.isNotEmpty(listVO.getScorerIds())) {
                    String scorerNames = "";
                    for (int i = 0; i < listVO.getScorerIds().size(); i++) {
                        Long scoreId = listVO.getScorerIds().get(i);
                        ReportAccountRespVO scorer = accountMap.get(scoreId);
                        if (null != scorer && StringUtils.isNotBlank(scorer.getUserFullName())) {
                            if (i == 0) {
                                scorerNames += scorer.getUserFullName();
                            } else {
                                scorerNames += "," + scorer.getUserFullName();
                            }
                        }
                    }
                    if (StringUtils.isNotBlank(scorerNames)) {
                        listVO.setAuditorFullNames(scorerNames);
                    }

                }
            }
        }

        return list;
    }

    @Override
    public StudentCaseStudyVO informListView(Long informRecordId, RequestContext res) {

        //获取是否是全平台人员
        RangeAccountsVO rangeAccountsVO = accountUtil.getRangeAccounts(res.isAdmin(), res.getAccountId(), res.getSiteId(), null, null);
        List<Long> accountIds = rangeAccountsVO.getAccountIds();
        Map<Long, ReportAccountRespVO> accountMap = rangeAccountsVO.getAccountMap();

        StudentCaseStudyVO studyVO = null;

        studyVO = studentCaseMapper.informListView(informRecordId);


        if (null != studyVO) {


            //获得前一篇,后一篇id
            List<Long> prefixIds = studentCaseMapper.getManageInformPrefix(informRecordId, studyVO.getInformTime(), accountIds, res.getCompanyId(), res.getSiteId());
            if (CollectionUtils.isNotEmpty(prefixIds)) {
                studyVO.setPrefixId(prefixIds.get(0));
            }

            List<Long> suffixIds = studentCaseMapper.getManageInformSuffix(informRecordId, studyVO.getInformTime(), accountIds, res.getCompanyId(), res.getSiteId());
            if (CollectionUtils.isNotEmpty(suffixIds)) {
                studyVO.setSuffixId(suffixIds.get(0));
            }

            //组装附件list
            if (StringUtils.isNotBlank(studyVO.getAccessoryUrl())) {
                studyVO.setAccessoryUrlList(util.getAccessoryUrlList(studyVO.getStudentCaseId()));
            }

            //获得举报说明列表
            List<InformVO> informVOList = informRecordeService.getInformList(studyVO.getStudentCaseId());
            //获取举报人和作者信息
            List<Long> accountIds1 = new ArrayList<>();
            if (null == accountMap) {
                accountMap = new HashMap<>(16);
                accountIds1.add(studyVO.getAuthorId());
            }
            if (CollectionUtils.isNotEmpty(informVOList)) {
                for (InformVO informVO : informVOList) {
                    if (!accountIds1.contains(informVO.getInformerId())) {
                        accountIds1.add(informVO.getInformerId());
                    }
                }
            }
            accountMap = accountUtil.getAccountMap(accountMap, accountIds1);

            ReportAccountRespVO author = accountMap.get(studyVO.getAuthorId());
            if (null != author) {
                studyVO.setAuthorName(author.getUserFullName());
            }

            if (CollectionUtils.isNotEmpty(informVOList)) {
                for (InformVO informVO : informVOList) {
                    //组装举报截图
                    if (StringUtils.isNotBlank(informVO.getPictureUrl())) {
                        String[] strings = informVO.getPictureUrl().split(",");
                        List<String> stringList = Arrays.asList(strings);
                        informVO.setPictureUrlList(stringList);
                        informVO.setPictureUrl(null);
                    }
                    ReportAccountRespVO informer = accountMap.get(informVO.getInformerId());
                    if (null != informer) {
                        informVO.setInformerFullName(informer.getUserFullName());
                    }
                }
                studyVO.setInformVOList(informVOList);
            }


            return studyVO;

        }

        return new StudentCaseStudyVO();
    }

    @Override
    public Map<String, Object> getManageListCount(Long caseLibraryId) {

        Map<String, Object> map = new HashMap<>(3);

        RequestContext res = ContextHolder.get();

        //通过内部方法获取用户信息
        RangeAccountsVO rangeAccountsVO1 = accountUtil.getRangeAccounts(res.isAdmin(), res.getAccountId(), res.getSiteId(), null, null);

        List<Long> accountIds2 = rangeAccountsVO1.getAccountIds();
        Boolean isAllAccount = rangeAccountsVO1.getIsAllAccount();
        if (!isAllAccount) {
            if (CollectionUtils.isEmpty(accountIds2)) {
                map.put("total", 0);
                map.put("waitAuditNumber", 0);
                map.put("waitScoreNumber", 0);
                return map;
            }
        }


        //获取待审核记录数

        Integer total = studentCaseMapper.getStudentCaseManageCount(caseLibraryId, accountIds2, null, null,
                null, res.getCompanyId(), res.getSiteId());
        Integer waitAuditNumber = studentCaseMapper.getStudentCaseManageCount(caseLibraryId, accountIds2, null, 0,
                null, res.getCompanyId(), res.getSiteId());
        //获取待评分记录数
        Integer waitScoreNumber = studentCaseMapper.getStudentCaseManageCount(caseLibraryId, accountIds2, null, null,
                0, res.getCompanyId(), res.getSiteId());
        map.put("total", total);
        map.put("waitAuditNumber", waitAuditNumber);
        map.put("waitScoreNumber", waitScoreNumber);
        return map;
    }

    @Override
    public Map<String, Object> getLibraryProjectName(Long caseLibraryId, Long studentLibraryId) {

        Map<String, Object> map = null;

        if (null != caseLibraryId) {
            map = studentCaseMapper.getProjectByCaseLibrary(caseLibraryId);
            //案例活动报表需要关联分类维度
            List<CaseLibraryRelatedClassify> caseLibraryRelatedClassifyList =
                    caseLibraryRelatedClassifyService.getRelatedClassify(caseLibraryId);
            map.put("relatedClassify", caseLibraryRelatedClassifyList);
        } else if (null != studentLibraryId) {
            map = studentCaseMapper.getProjectByStudentCase(studentLibraryId);
        }
        if (null == map) {
            map = new HashMap<>(1);
        }

        return map;
    }

    @Override
    public boolean deleteStudentCase(Long studentCaseId) {
        return studentCaseMapper.deleteStudentCase(studentCaseId);
    }

    @Override
    public Page<FavoriteVO> getFavorites(Integer pageNo, Integer pageSize, String name) {

        RequestContext res = ContextHolder.get();

        Page<FavoriteVO> page = new Page<>(pageNo, pageSize);
        List<FavoriteVO> list = studentCaseMapper.getFavorites(res.getCompanyId(), res.getSiteId(), name, res.getAccountId(), page);
        if (CollectionUtils.isNotEmpty(list)) {
            List<Long> accountIds = new ArrayList<>();
            Map<Long, ReportAccountRespVO> accountMap = new HashMap<>(20);
            for (FavoriteVO favoriteVO : list) {
                if (!accountIds.contains(favoriteVO.getAuthorId())) {
                    accountIds.add(favoriteVO.getAuthorId());
                }
            }
            //获取作者信息
            if (CollectionUtils.isNotEmpty(accountIds)) {
                accountMap = accountUtil.getAccountMap(accountMap, accountIds);
                for (FavoriteVO favoriteVO : list) {
                    ReportAccountRespVO account = accountMap.get(favoriteVO.getAuthorId());
                    if (null != account) {
                        favoriteVO.setAuthorFullName(account.getUserFullName() == null ? "" : account.getUserFullName());
                    }
                }
            }

            page.setRecords(list);
        }


        return page;
    }

    @Override
    public Page<StudentCaseStudyVO> getProjectList(String name, Integer pageNo, Integer pageSize) {

        RequestContext res = ContextHolder.get();
        Page<StudentCaseStudyVO> page = new Page<>(pageNo, pageSize);
        List<StudentCaseStudyVO> list = studentCaseMapper.getProjectList(name, res.getCompanyId(), res.getSiteId(), page);
        if (CollectionUtils.isNotEmpty(list)) {
            List<Long> accountIds = new ArrayList<>();
            Map<Long, ReportAccountRespVO> accountMap = new HashMap<>(16);
            for (StudentCaseStudyVO studyVO : list) {
                if (!accountIds.contains(studyVO.getAuthorId())) {
                    accountIds.add(studyVO.getAuthorId());
                }
            }
            accountMap = accountUtil.getAccountMap(accountMap, accountIds);
            for (StudentCaseStudyVO studyVO : list) {
                ReportAccountRespVO account = accountMap.get(studyVO.getAuthorId());
                if (null != account) {
                    studyVO.setAuthorUserName(account.getUserName());
                    studyVO.setAuthorName(account.getUserFullName() == null ? "" : account.getUserFullName());
                    studyVO.setOrgName(account.getOrgName() == null ? "" : account.getOrgName());
                }
            }
            page.setRecords(list);
        }

        return page;
    }

    @Override
    public Page<FavoriteVO> getSelectionStudentCaseList(SelectionStudentCaseParam selectionStudentCaseParam) {

        RequestContext res = ContextHolder.get();

        Page<FavoriteVO> page = new Page<>(selectionStudentCaseParam.getPageNo(), selectionStudentCaseParam.getPageSize());


        //作者或者案例姓名模糊查询
        String key = selectionStudentCaseParam.getKey();
        Integer sortType = selectionStudentCaseParam.getSortType();
        if (StringUtils.isNotBlank(key)) {
            key = key.trim();
        }

        List<Long> accountIds = new ArrayList<>();
        Map<Long, com.yizhi.library.application.vo.ReportAccountRespVO> accountMap = new HashMap<>(16);

        //分类筛选id
        List<Long> classifyIds = selectionStudentCaseParam.getClassifyIdList();
        List<Long> studentCaseIds = null;

        //排序类型 1:提交时间由近到远 2:提交时间由远到近 3:学习量由多到少 4:点赞数由多到少 5:评分由高到低

        //如果关键字不为空,先模糊查询作者
        if (StringUtils.isNotBlank(key)) {
            getAccountByName(key, res.getSiteId(), accountIds, accountMap);
        }

        //如果有分类筛选
        if (CollectionUtils.isNotEmpty(classifyIds)) {
            studentCaseIds = studentCaseMapper.getStudentCaseIdsByClassifyIds(res.getCompanyId(), res.getSiteId(), classifyIds);
            if (CollectionUtils.isEmpty(studentCaseIds)) {
                return page;
            }
        }

        //获取指定范围的该用户可见的精选案例
        List<Long> rangeIds = studentCaseMapper.getRangeIds(res.getRelationIds());

        List<FavoriteVO> list = studentCaseMapper.getSelectionStudentCaseList(key, accountIds, rangeIds, studentCaseIds, sortType, res.getCompanyId(), res.getSiteId(), page);

        //获取作者信息
        if (CollectionUtils.isNotEmpty(list)) {
            List<Long> accountIds1 = new ArrayList<>();
            for (FavoriteVO favoriteVO : list) {
                if (!accountIds.contains(favoriteVO.getAuthorId()) && !accountIds1.contains(favoriteVO.getAuthorId())) {
                    accountIds1.add(favoriteVO.getAuthorId());
                }
            }
            if (CollectionUtils.isNotEmpty(accountIds1)) {
                accountMap = accountUtil.getAccountMap(accountMap, accountIds1);
            }
            Long accountId = res.getAccountId();
            for (FavoriteVO favoriteVO : list) {
                Long authorId = favoriteVO.getAuthorId();
                if (null != authorId && authorId.equals(accountId)){
                    favoriteVO.setSelf(true);
                }
                com.yizhi.library.application.vo.ReportAccountRespVO account = accountMap.get(favoriteVO.getAuthorId());
                if (null != account) {
                    String fullName = account.getUserFullName();
                    favoriteVO.setAuthorFullName(fullName);
                    if (org.springframework.util.StringUtils.isEmpty(fullName)){
                        favoriteVO.setAuthorFullName(account.getUserName());
                    }
                }
            }
            page.setRecords(list);
        }


        return page;
    }

    /**
     * 内部封装,获取案例活动关联维度以及下级所有分类列表
     *
     * @param caseLibraryId
     * @return
     */
    public List<ClassifyListVO> getCaseLibraryClassifyListVO(Long caseLibraryId, Long companyId, Long siteId) {
        //获取维度列表
        List<CaseLibraryRelatedClassify> caseLibraryRelatedClassifyList =
                caseLibraryRelatedClassifyService.getRelatedClassify(caseLibraryId);

        List<ClassifyListVO> classifyListVOList = new ArrayList<>();

        if (CollectionUtils.isNotEmpty(caseLibraryRelatedClassifyList)) {
            for (CaseLibraryRelatedClassify caseLibraryRelatedClassify1 : caseLibraryRelatedClassifyList) {
                ClassifyListVO classifyListVO = new ClassifyListVO();
                classifyListVO.setLevel(0);
                classifyListVO.setName(caseLibraryRelatedClassify1.getClassifyName());
                classifyListVO.setId(caseLibraryRelatedClassify1.getClassifyId());
                List<ClassifyLevelOneVO> classifyLevelOneVOList = new ArrayList<>();
                //获取一级分类组装
                Classify classify = new Classify();
                classify.setParentId(caseLibraryRelatedClassify1.getClassifyId());
                classify.setCompanyId(companyId);
                classify.setSiteId(siteId);
                classify.setState(1);
                classify.setLevel(1);
                List<Classify> levelOneList = classifyService.selectList(new EntityWrapper<>(classify));
                //获取二级分类组装
                if (CollectionUtils.isNotEmpty(levelOneList)) {
                    for (Classify classify1 : levelOneList) {
                        ClassifyLevelOneVO classifyLevelOneVO = new ClassifyLevelOneVO();
                        BeanUtils.copyProperties(classify1, classifyLevelOneVO);
                        classifyLevelOneVO.setKey(classify1.getId().toString());

                        //少波需要前端组件拿固定key
                        classifyLevelOneVO.setLabel(classifyLevelOneVO.getName());
                        classifyLevelOneVO.setValue(classifyLevelOneVO.getId());

                        classify = new Classify();
                        classify.setParentId(classify1.getId());
                        classify.setCompanyId(companyId);
                        classify.setSiteId(siteId);
                        classify.setState(1);
                        classify.setLevel(2);
                        List<Classify> levelTwoList = classifyService.selectList(new EntityWrapper<>(classify));
                        if (CollectionUtils.isNotEmpty(levelTwoList)) {
                            List<ClassifyVO> classifyVOList = new ArrayList<>();
//                            BeanUtils.copyProperties(levelTwoList, classifyListVOList);
                            classifyVOList = JSON.parseArray(JSON.toJSONString(levelTwoList), ClassifyVO.class);
                            for (ClassifyVO classifyVO : classifyVOList) {
                                classifyVO.setKey(classifyVO.getId().toString());
                                classifyVO.setLabel(classifyVO.getName());
                                classifyVO.setValue(classifyVO.getId());
                            }
                            classifyLevelOneVO.setChildren(classifyVOList);
                        }
                        classifyLevelOneVOList.add(classifyLevelOneVO);
                    }

                }
                classifyListVO.setChildren(classifyLevelOneVOList);
                classifyListVOList.add(classifyListVO);
            }
        }
        return classifyListVOList;
    }

    public void getAccountByName(String name, Long siteId, List<Long> accountIds, Map<Long, com.yizhi.library.application.vo.ReportAccountRespVO> accountMap) {

        ReportRangeAccountReqVO reqVO = new ReportRangeAccountReqVO();
        reqVO.setAccountName(name);
        reqVO.setSiteId(siteId);
        List<com.yizhi.system.application.vo.ReportAccountRespVO> accountRespVOS = reportClient.getRangeAccounts(reqVO);
        if (CollectionUtils.isNotEmpty(accountRespVOS)) {
            for (com.yizhi.system.application.vo.ReportAccountRespVO account : accountRespVOS) {
                com.yizhi.library.application.vo.ReportAccountRespVO vo = new com.yizhi.library.application.vo.ReportAccountRespVO();
                BeanUtils.copyProperties(account,vo);
                accountIds.add(account.getUserId());
                accountMap.put(account.getUserId(), vo);
            }
        }
    }

    //组装关键字
    public String getKeyWords(List<String> keyWordsList) {
        StringBuilder stringBuilder = new StringBuilder();
        if (CollectionUtils.isNotEmpty(keyWordsList)) {
            for (String s : keyWordsList) {
                if (keyWordsList.indexOf(s) == 0) {
                    stringBuilder.append(s);
                } else {
                    stringBuilder.append(",").append(s);
                }
            }
        }
        return stringBuilder.toString();
    }

    //组装关键字数组
    public List<String> getKeyWordsList(String keyWords) {
        String[] strings = keyWords.split(",");
        List<String> keyWordsList = Arrays.asList(strings);
        return keyWordsList;
    }

    //插入或者更新评分记录
    public Boolean insertOrUpdateScorer(Long caseLibraryId, Long studentCaseId, Long accountId, Integer score, Integer scoreRule) {

        Date date = new Date();

        StudentCaseScoreRecorde scoreRecorde = new StudentCaseScoreRecorde();

        if (scoreRule == 4) {
            score = 0;
        }

        scoreRecorde.setStudentCaseId(studentCaseId);
        scoreRecorde.setScorerId(accountId);
        StudentCaseScoreRecorde scoreRecorde1 = scoreRecordeService.selectOne(new EntityWrapper<>(scoreRecorde));
        if (null != scoreRecorde1) {
            scoreRecorde1.setScore(score);
            scoreRecorde1.setCreateTime(date);
            scoreRecordeService.updateById(scoreRecorde1);
        } else {
            scoreRecorde.setId(idGenerator.generate());
            scoreRecorde.setScore(score);
            scoreRecorde.setCreateTime(date);
            scoreRecordeService.insert(scoreRecorde);
        }

        return true;
    }

    @Override
    public Page<DroolsVo> getPageByDrools(String field, String value, Page<DroolsVo> page) {
        if (StringUtils.isBlank(field)) {
            LOGGER.info("列名不能为空！");
            return page;
        }
        if (field.equalsIgnoreCase(TaskParamsEnums.CLASSIFY.getCode())) {
            field = Constant.NAME;
            page = classifyService.getClassifyNameByDrools(field, value, page);
        } else {
            if (field.equalsIgnoreCase(TaskParamsEnums.NAME.getCode())) {
                field = Constant.TITLE;
                return getPage(field, value, page);
            } else if (field.equalsIgnoreCase(TaskParamsEnums.KEYWORD.getCode())) {
                field = Constant.CASE_LIBRARY_KEYWORD;
                return getPage(field, value, page);
            } else if (field.equalsIgnoreCase(TaskParamsEnums.BELONG_CASE_ACTIVITY.getCode())) {
                return caseLibraryService.getPageByDrools(Constant.NAME, value, page);
            }
        }
        return page;
    }

    @Override
    public void checkAdmireNum() {
        studentCaseMapper.initializeAdmireNum();
//        studentCaseMapper.checkAdmireNum();
    }

    public Page getPage(String field, String value, Page<DroolsVo> page) {
        RequestContext requestContext = ContextHolder.get();
        Long siteId = requestContext.getSiteId();
        Long companyId = requestContext.getCompanyId();

        StudentCase caseLibrary = new StudentCase();
        caseLibrary.setSiteId(siteId);
        caseLibrary.setCompanyId(companyId);
        caseLibrary.setState(Constant.UP);
        EntityWrapper wrapper = new EntityWrapper(caseLibrary);
        wrapper.setSqlSelect("id," + field)
                .isNotNull(field)
                .like(field, value)
                .addFilter(field + "!=''")
                .orderBy("create_time", false);

        //转换为驼峰命名
        String upperField = ClassUtil.getFieldName(field);
        List<DroolsVo> voList = null;
        List<StudentCase> list = this.baseMapper.selectPage(page, wrapper);
        if (!CollectionUtils.isEmpty(list)) {
            voList = new ArrayList<>(list.size());
            for (StudentCase a : list) {
                DroolsVo vo = new DroolsVo();
                vo.setTaskId(a.getId());
                vo.setTaskFieldValue(ClassUtil.invokeMethod(a, upperField));
                vo.setTaskParamsType(field);
                voList.add(vo);
            }
            page.setRecords(voList);
        }
        return page;
    }

    /**
     * 处理分数
     *
     * @param studentCaseAuditParam
     * @param scoreRule             分数规则
     * @param studentCase
     * @param caseLibrary           案例库
     * @param isAdmin               是否为管理员评分（1：是  0：否） 是则将该评分记录的其他评分分数逻辑删除
     */
    private void dealScore(StudentCaseAuditParam studentCaseAuditParam, Integer scoreRule, StudentCase studentCase, CaseLibrary caseLibrary, boolean isAdmin) {
        RequestContext context = ContextHolder.get();
        Date now = new Date();
        List<StudentCaseDimensionGradeRecordVO> studentCaseDimensionGradeRecordVOS = studentCaseAuditParam.getStudentCaseDimensionGradeRecordVOS();
        if (scoreRule == 4 && CollectionUtils.isNotEmpty(studentCaseDimensionGradeRecordVOS)) {
            List<StudentCaseDimensionGradeRecord> listToInsert = new ArrayList<>(studentCaseDimensionGradeRecordVOS.size());
            StudentCaseDimensionGradeRecord gradeRecord = null;
            for (StudentCaseDimensionGradeRecordVO studentCaseDimensionGradeRecordVO : studentCaseDimensionGradeRecordVOS) {
                if (isAdmin) {
                    //管理员多维度评分 逻辑删除非管理员的多维度评分
                    Integer i = studentCaseDimensionGradeRecordMapper.deleteNotAdminRecord(studentCaseDimensionGradeRecordVO.getCaseLibraryDimensionGradeId());
                }
                gradeRecord = new StudentCaseDimensionGradeRecord();

                gradeRecord.setAccountId(studentCase.getAccountId());
                gradeRecord.setCaseLibraryId(caseLibrary.getId());
                gradeRecord.setCompanyId(context.getCompanyId());
                gradeRecord.setCreateById(context.getAccountId());
                gradeRecord.setCreateByName(context.getAccountName());
                gradeRecord.setCreateTime(now);
                gradeRecord.setDeleted(0);
                gradeRecord.setId(idGenerator.generate());
                gradeRecord.setOrgId(context.getOrgId());
                gradeRecord.setScore(studentCaseDimensionGradeRecordVO.getScore());
                gradeRecord.setSiteId(context.getSiteId());
                gradeRecord.setScorerId(context.getAccountId());
                gradeRecord.setStudentCaseId(studentCase.getId());
                gradeRecord.setCaseLibraryDimensionGradeId(studentCaseDimensionGradeRecordVO.getCaseLibraryDimensionGradeId());
                listToInsert.add(gradeRecord);
            }
            if (CollectionUtils.isNotEmpty(listToInsert)) {
                studentCaseDimensionGradeRecordService.insertBatch(listToInsert);
            }
        }
    }

}
