package com.yizhi.library.application.util;

import com.yizhi.library.application.vo.RangeAccountsVO;
import com.yizhi.library.application.vo.StudentCaseManageListVO;
import com.yizhi.system.application.constant.AuthzConstant;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.vo.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Ding
 * @className AccountUtil
 * @description TODO
 * @date 2019/7/25
 **/
@Component
public class AccountUtil {

    @Autowired
    private ReportClient reportClient;

    @Autowired
    private AccountClient accountClient;


    /**
     * 获取管辖区/管理员模糊查询人员信息
     *
     * @param isAdmin
     * @param accountId
     * @param siteId
     * @param accountName
     * @param orgName
     * @return
     */
    public RangeAccountsVO getRangeAccounts(Boolean isAdmin, Long accountId, Long siteId, String accountName, String orgName) {
        Boolean isAllAccounts = false;
        //返回实体类
        RangeAccountsVO rangeAccountsVO = new RangeAccountsVO();
        List<com.yizhi.system.application.vo.ReportAccountRespVO> accountList = null;
        List<Long> accountIds = null;
        Map<Long, com.yizhi.library.application.vo.ReportAccountRespVO> accountMap = new HashMap<>(16);
        if (isAdmin) {
            //管理员模糊查询
            if (StringUtils.isNotBlank(accountName) || StringUtils.isNotBlank(orgName)) {
                ReportRangeAccountReqVO reqVO = new ReportRangeAccountReqVO();
                if (StringUtils.isNotBlank(accountName)) {
                    reqVO.setAccountName(accountName.trim());
                }
                if (StringUtils.isNotBlank(orgName)) {
                    reqVO.setAccountName(orgName.trim());
                }
                reqVO.setSiteId(siteId);
                accountList = reportClient.getRangeAccounts(reqVO);
                //如果人员不为空,装入map中,并返回
                if (CollectionUtils.isNotEmpty(accountList)) {
                    accountIds = new ArrayList<>();
                    for (ReportAccountRespVO respVO : accountList) {
                        accountIds.add(respVO.getUserId());

                        com.yizhi.library.application.vo.ReportAccountRespVO vo = new com.yizhi.library.application.vo.ReportAccountRespVO();
                        BeanUtils.copyProperties(respVO,vo);
                        accountMap.put(respVO.getUserId(), vo);
                    }
                    rangeAccountsVO.setAccountIds(accountIds);
                    rangeAccountsVO.setAccountMap(accountMap);
                    rangeAccountsVO.setIsAllAccount(false);
                    return rangeAccountsVO;
                } else {
                    rangeAccountsVO.setIsAllAccount(false);
                    return rangeAccountsVO;
                }

            } else {
                rangeAccountsVO.setIsAllAccount(true);
                return rangeAccountsVO;
            }
        } else {
            //一般管理员
            HQAccountInManageParam reqVO = new HQAccountInManageParam();
            reqVO.setModuleType(AuthzConstant.moduleType.caseLibrary);
            reqVO.setAccountId(accountId);
            reqVO.setSiteId(siteId);
            if (StringUtils.isNotBlank(accountName)) {
                reqVO.setAccountName(accountName.trim());
            }
            if (StringUtils.isNotBlank(orgName)) {
                reqVO.setAccountName(orgName.trim());
            }
            ReportAccountRespInManageVO resp = reportClient.getRangeAccountsInManage(reqVO);
            //如果不是管理的根目录
            if (!resp.getIsAdmin()) {
                rangeAccountsVO.setIsAllAccount(false);
                if (CollectionUtils.isNotEmpty(resp.getList())) {
                    accountList = resp.getList();
                    accountIds = new ArrayList<>();
                    for (ReportAccountRespVO respVO : accountList) {
                        accountIds.add(respVO.getUserId());

                        com.yizhi.library.application.vo.ReportAccountRespVO vo = new com.yizhi.library.application.vo.ReportAccountRespVO();
                        BeanUtils.copyProperties(respVO,vo);
                        accountMap.put(respVO.getUserId(), vo);
                        accountMap.put(respVO.getUserId(), vo);
                    }
                    rangeAccountsVO.setAccountMap(accountMap);
                    rangeAccountsVO.setAccountIds(accountIds);
                }
                return rangeAccountsVO;

            } else {
                //如果是根目录,按管理员处理
                return getRangeAccounts(true, accountId, siteId, accountName, orgName);
            }

        }
    }

    /**
     * 查询用户信息并放入用户map中
     *
     * @param map
     * @param accountIds
     * @return
     */
    public Map<Long, com.yizhi.library.application.vo.ReportAccountRespVO> getAccountMap(Map<Long, com.yizhi.library.application.vo.ReportAccountRespVO> map, List<Long> accountIds) {
        List<AccountVO> list = accountClient.idsGet(accountIds);

        if (CollectionUtils.isNotEmpty(list)) {
            for (AccountVO accountVO : list) {
                com.yizhi.library.application.vo.ReportAccountRespVO respVO = new com.yizhi.library.application.vo.ReportAccountRespVO();
                respVO.setUserId(accountVO.getId());
                respVO.setUserName(accountVO.getName());
                respVO.setUserFullName(accountVO.getFullName());
                respVO.setOrgId(accountVO.getOrgId());
                respVO.setOrgName(accountVO.getOrgName());
                map.put(respVO.getUserId(), respVO);
            }
        }
        return map;
    }


    /**
     * 组装所有学员案例管理列表信息
     *
     * @return
     */
    public List<StudentCaseManageListVO> assembleStudentCaseManageList(Map<Long, com.yizhi.library.application.vo.ReportAccountRespVO> accountMap, List<StudentCaseManageListVO> list) {

        for (StudentCaseManageListVO studentCaseManageListVO : list) {
            com.yizhi.library.application.vo.ReportAccountRespVO account = accountMap.get(studentCaseManageListVO.getAccountId());

            //案例状态判断 以及操作状态判断
            if (null != studentCaseManageListVO.getScoreState()) {
                if (studentCaseManageListVO.getScoreState() == 0) {
                    studentCaseManageListVO.setState("待评分");
                } else if (studentCaseManageListVO.getScoreState() == 1) {
                    studentCaseManageListVO.setState("已评分");
                }
                studentCaseManageListVO.setHandleType(1);
            } else if (null != studentCaseManageListVO.getAuditState()) {
                if (studentCaseManageListVO.getAuditState() == 0) {
                    studentCaseManageListVO.setState("待审核");
                } else if (studentCaseManageListVO.getAuditState() == 2) {
                    studentCaseManageListVO.setState("不通过");
                }
                studentCaseManageListVO.setHandleType(2);
            }

            //作者信息组装
            if (null != account) {
                studentCaseManageListVO.setAccountName(account.getUserName());
                studentCaseManageListVO.setAccountFullName(account.getUserFullName() == null ? "" : account.getUserFullName());
                studentCaseManageListVO.setOrgId(account.getOrgId());
                studentCaseManageListVO.setOrgName(account.getOrgName() == null ? "" : account.getOrgName());
            }
            //审核人信息组装
            if (CollectionUtils.isNotEmpty(studentCaseManageListVO.getAuditorIds())) {
                String auditorString = "";
                for (int i = 0; i < studentCaseManageListVO.getAuditorIds().size(); i++) {
                    Long auditorId = studentCaseManageListVO.getAuditorIds().get(i);
                    com.yizhi.library.application.vo.ReportAccountRespVO auditor = accountMap.get(auditorId);
                    if (null != auditor) {
                        if (StringUtils.isNotBlank(auditor.getUserFullName())) {
                            if (StringUtils.isBlank(auditorString)) {
                                auditorString += auditor.getUserFullName();
                            } else {
                                auditorString += "," + auditor.getUserFullName();
                            }
                        }

                    }
                }
                studentCaseManageListVO.setAuditorFullNames(auditorString);
            }
            //评论人信息组装
            if (CollectionUtils.isNotEmpty(studentCaseManageListVO.getScorerIds())) {
                String scorerNames = "";
                for (int i = 0; i < studentCaseManageListVO.getScorerIds().size(); i++) {
                    Long scorerId = studentCaseManageListVO.getScorerIds().get(i);
                    com.yizhi.library.application.vo.ReportAccountRespVO scorer = accountMap.get(scorerId);
                    if (null != scorer) {
                        if (StringUtils.isNotBlank(scorer.getUserFullName())) {
                            if (StringUtils.isBlank(scorerNames)) {
                                scorerNames += scorer.getUserFullName();
                            } else {
                                scorerNames += "," + scorer.getUserFullName();
                            }
                        }

                    }
                }
                studentCaseManageListVO.setScorerFullNames(scorerNames);
            }
        }
        return list;
    }


}
