package com.yizhi.library.application.util;

import com.alibaba.fastjson.JSON;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;

/**
 * @author Ding
 * @className DownloadOssUrl
 * @description TODO
 * @date 2019/7/8
 **/

public class DownloadOssUrl {

    private static final Logger LOGGER = LoggerFactory.getLogger(DownloadOssUrl.class);

    public static Workbook getWorkBook(String ossUrl) {
        try {

            URL url = null;
            HttpURLConnection connection = null;
            InputStream inputStream = null;
            Workbook workbook = null;
            //解析url

            String fileName = ossUrl.substring(ossUrl.lastIndexOf("/") + 1);
            String ossUrlPrefix = ossUrl.substring(0, ossUrl.lastIndexOf("/") + 1);
            ossUrl = ossUrlPrefix + URLEncoder.encode(fileName, "utf-8");

            url = new URL(ossUrl);
            connection = (HttpURLConnection) url.openConnection();
            //防止屏蔽程序抓取而返回403错误
            connection.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt)");
            inputStream = connection.getInputStream();
            workbook = WorkbookFactory.create(inputStream);
            return workbook;
        } catch (MalformedURLException e) {
            LOGGER.error("创建ossurl 连接错误", e);
            return null;
        } catch (IOException e) {
            LOGGER.error("创建ossurl 连接错误", e);
            return null;
        } catch (InvalidFormatException e) {
            LOGGER.error("创建Workbook 对象错误", e);
            return null;
        }
    }

    public static String downLoadFromUrl(String urlStr, String fileName, String outputFilePath) {

        File file = null;
//        FileInfo fileInfo = new FileInfo();
        try {
            URL url = new URL(urlStr);
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            //防止屏蔽程序抓取而返回403错误
            conn.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt)");
            conn.setRequestProperty("Charsert", "UTF-8");
            conn.setRequestProperty("Accept-Charset", "UTF-8");
            //得到输入流
            InputStream inputStream = conn.getInputStream();
            //获取自己数组
            byte[] buffer = new byte[1024];
            int len = 0;
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            while ((len = inputStream.read(buffer)) != -1) {
                bos.write(buffer, 0, len);
            }
            bos.close();

            byte[] getData = bos.toByteArray();
            //文件保存位置
            File saveDir = new File(outputFilePath);
            if (!saveDir.exists()) {
                saveDir.mkdirs();
            }
            file = new File(saveDir + File.separator + fileName);
            FileOutputStream fos = new FileOutputStream(file);
            fos.write(getData);
            if (fos != null) {
                fos.close();
            }
            if (inputStream != null) {
                inputStream.close();
            }
        } catch (Exception e) {
            LOGGER.info("fileName={},outputFilePath={}", fileName, outputFilePath);
            LOGGER.info("urlStr={},异常={}", urlStr, JSON.toJSONString(e));
        }
        return file.getAbsolutePath();
    }
}
