package com.yizhi.library.application.util;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.comment.application.feign.PdfPagesClient;
import com.yizhi.comment.application.vo.PdfVO;
import com.yizhi.comment.application.vo.domain.PdfPagesVo;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.library.application.domain.CaseLibraryRelatedClassify;
import com.yizhi.library.application.domain.StudentCaseAccessary;
import com.yizhi.library.application.enums.DocumentType;
import com.yizhi.library.application.vo.AccessoryUrlVO;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.library.application.service.IStudentCaseAccessaryService;
import com.yizhi.library.application.service.IStudentCaseService;
import com.yizhi.training.application.feign.TpProjectClient;
import com.yizhi.training.application.vo.domain.TrainingProjectVo;
import com.yizhi.util.application.constant.TpActivityType;
import com.yizhi.util.application.event.AssignmentMediaTranscodingEvent;
import com.yizhi.util.application.sensitive.SensitivewordFilter;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import java.io.File;
import java.util.*;

/**
 * @author Ding
 * @className Util
 * @description TODO
 * @date 2019/7/12
 **/
@Component
public class Util {

    @Autowired
    private IStudentCaseService studentCaseService;

    @Autowired
    private TpProjectClient tpProjectClient;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private IStudentCaseAccessaryService studentCaseAccessaryService;

    @Autowired
    private TaskExecutor taskExecutor;

    @Autowired
    private WorkUtil workUtil;

    @Autowired
    private CloudEventPublisher cloudEventPublisher;

    @Autowired
    private PdfPagesClient pdfPagesClient;
    /**
     * 内部封装截取url组装成附件类list
     *
     * @param studentCaseId
     * @return
     */
    public List<AccessoryUrlVO> getAccessoryUrlList(Long studentCaseId) {
//        String[] strings = url.split(";");
        List<AccessoryUrlVO> accessoryUrlVOList = new ArrayList<>();
        Long time = System.currentTimeMillis();

        StudentCaseAccessary studentCaseAccessary = new StudentCaseAccessary();
        studentCaseAccessary.setStudentCaseId(studentCaseId);
        studentCaseAccessary.setState(1);
        List<StudentCaseAccessary> accessaryList = studentCaseAccessaryService.selectList(new EntityWrapper<>(studentCaseAccessary));

        if (CollectionUtils.isNotEmpty(accessaryList)) {
            for (StudentCaseAccessary studentCaseAccessary1 : accessaryList) {
            	List<PdfPagesVo> list = pdfPagesClient.list(studentCaseAccessary1.getId(), TpActivityType.TYPE_SELECTED_CASE);
                Integer i = accessaryList.indexOf(studentCaseAccessary1);
                AccessoryUrlVO accessoryUrlVO = new AccessoryUrlVO();
                //如果有转码url用转码url
                if (null != studentCaseAccessary1.getVideoSd()) {
                    accessoryUrlVO.setUrl(studentCaseAccessary1.getVideoSd());
                } else {
                    //没有转码url就用原文件url
                    accessoryUrlVO.setUrl(studentCaseAccessary1.getUrl());
                }
                accessoryUrlVO.setFileSize(studentCaseAccessary1.getFileSize());
                accessoryUrlVO.setName(studentCaseAccessary1.getName());
                accessoryUrlVO.setUid(time + "a" + i);
                List<com.yizhi.library.application.vo.PdfPages> vos = new ArrayList<>();

                for (PdfPagesVo pdfPages : list) {
                    com.yizhi.library.application.vo.PdfPages vo = new com.yizhi.library.application.vo.PdfPages();
                    BeanUtils.copyProperties(pdfPages,vo);
                    vos.add(vo);
                }
                accessoryUrlVO.setPdfPages(vos);
                accessoryUrlVOList.add(accessoryUrlVO);
            }
        }

        return accessoryUrlVOList;
    }

    /**
     * 将多个附件url组成一个字符串
     *
     * @param accessoryUrlVOList
     * @return
     */
    public String getAccessoryUrl(Long studentCaseId, List<AccessoryUrlVO> accessoryUrlVOList) {

        RequestContext res = ContextHolder.get();
        Date date = new Date();
        String accessoryUrl = "";

        List<StudentCaseAccessary> accessaryList = new ArrayList<>();
		List<PdfVO> pdfVOs = new ArrayList<PdfVO>();
        for (int i = 0; i < accessoryUrlVOList.size(); i++) {
            PdfVO pdfVO = new PdfVO();
            //组装附件类,并入库
            StudentCaseAccessary studentCaseAccessary = new StudentCaseAccessary();
            studentCaseAccessary.setId(idGenerator.generate());
            studentCaseAccessary.setStudentCaseId(studentCaseId);
            studentCaseAccessary.setName(accessoryUrlVOList.get(i).getName());
            studentCaseAccessary.setFileSize(accessoryUrlVOList.get(i).getFileSize());
            studentCaseAccessary.setUrl(accessoryUrlVOList.get(i).getUrl());
            studentCaseAccessary.setCreateById(res.getAccountId());
            studentCaseAccessary.setCreateByName(res.getAccountName());
            studentCaseAccessary.setCreateTime(date);
            studentCaseAccessary.setUpdateById(res.getAccountId());
            studentCaseAccessary.setUpdateByName(res.getAccountName());
            studentCaseAccessary.setUpdateTime(date);
            studentCaseAccessary.setCompanyId(res.getCompanyId());
            studentCaseAccessary.setSiteId(res.getSiteId());
            studentCaseAccessary.setOrgId(res.getOrgId());
            pdfVO.setBizId(studentCaseAccessary.getId());
            pdfVO.setBizType(TpActivityType.TYPE_SELECTED_CASE);
            pdfVO.setCompanyId(studentCaseAccessary.getCompanyId());
            pdfVO.setSiteId(studentCaseAccessary.getSiteId());
            pdfVO.setCreateById(studentCaseAccessary.getCreateById());
            pdfVO.setOriginalFile(studentCaseAccessary.getName());
            pdfVO.setOriginalFileUrl(studentCaseAccessary.getUrl());
            pdfVOs.add(pdfVO);
            accessaryList.add(studentCaseAccessary);
            
            if (StringUtils.isEmpty(accessoryUrl)) {
                accessoryUrl += accessoryUrlVOList.get(i).getName() + ",";
                accessoryUrl += accessoryUrlVOList.get(i).getFileSize() + ",";
                accessoryUrl += accessoryUrlVOList.get(i).getUrl();
            } else {
                accessoryUrl += ";" + accessoryUrlVOList.get(i).getName() + ",";
                accessoryUrl += accessoryUrlVOList.get(i).getFileSize() + ",";
                accessoryUrl += accessoryUrlVOList.get(i).getUrl();
            }
        }
        studentCaseAccessaryService.insertBatch(accessaryList);
		try {
			if (CollectionUtils.isNotEmpty(pdfVOs)) {
				cloudEventPublisher.publish("pdfToPng",
						new EventWrapper(0L, pdfVOs));
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        taskExecutor.asynExecute(new AbstractTaskHandler() {

            @Override
            public void handle() {

                try {

                    if (CollectionUtils.isNotEmpty(accessaryList)) {
                        for (StudentCaseAccessary studentCaseAccessary : accessaryList) {

                            String suffixName = studentCaseAccessary.getName().substring(studentCaseAccessary.getName().lastIndexOf(".") + 1);

                            if (workUtil.documentType(suffixName).equals(DocumentType.VIDEO.getType()) || workUtil.documentType(suffixName).equals(DocumentType.AUDIO.getType())) {
                                AssignmentMediaTranscodingEvent event = new AssignmentMediaTranscodingEvent();
                                List<AssignmentMediaTranscodingEvent.FileDTO> fileDTOS = new ArrayList<>();
                                fileDTOS.add(event.new FileDTO(studentCaseAccessary.getId(), studentCaseAccessary.getUrl()));
                                event.setType(TpActivityType.TYPE_SELECTED_CASE);
                                event.setFiles(fileDTOS);
                                cloudEventPublisher.publish("assignmentMediaTranscoding",
                                        new EventWrapper(0L, event));
                            }
                        }
                    }


                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });

        return accessoryUrl;
    }

    /**
     * 过滤敏感词
     *
     * @param content           评论内容
     * @param sensitiveWordList 已设置的敏感词集合
     * @return
     */
    public String filterWord(String content, Set<String> sensitiveWordList) {
        if (sensitiveWordList != null) {
            if (!StringUtils.isBlank(content)) {
                return SensitivewordFilter.sensitiveFilter(content, sensitiveWordList);
            }
        }
        return content;
    }

    //删除法目录方法
    public static boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            //递归删除目录中的子目录下
            for (int i = 0; i < children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }
        // 目录此时为空，可以删除
        return dir.delete();
    }

    //多个审核人,评分人转成字符串
    public static String listToString(List<String> stringList) {

        String resultString = "";

        for (int i = 0; i < stringList.size(); i++) {
            String s = stringList.get(i);
            if (i == 0) {
                resultString += s;
            } else {
                resultString += "," + s;
            }
        }
        return resultString;
    }

    //所属部门及上级部门转成字符串
    public static String orgNamesToString(List<String> orgNames) {

        String resultString = "";
        for (int i = 0; i < orgNames.size(); i++) {
            String s = orgNames.get(i);
            if (i == 0) {
                resultString += s;
            } else {
                resultString += "/" + s;
            }
        }
        return resultString;
    }

    public static Long getLongFromCell(Cell cell) {
        Long id = null;
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            id = Long.valueOf(cell.getStringCellValue());
        } else if (cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            cell.setCellType(CellType.STRING);
            id = Long.valueOf(cell.getStringCellValue());
        }
        return id;
    }

    public static Integer getIntegerFromCell(Cell cell) {
        Integer result = null;
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            result = Integer.valueOf(cell.getStringCellValue());
        } else if (cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            cell.setCellType(CellType.STRING);
            result = Integer.valueOf(cell.getStringCellValue());
        }
        return result;
    }

    public static String getStringFromCell(Cell cell) {

        String result = null;
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            result = cell.getStringCellValue();
        } else if (cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            cell.setCellType(CellType.STRING);
            result = cell.getStringCellValue();
        }

        return result;
    }

    public Map<String, Object> getCaseLibraryAndProjectName(Long caseLibraryId, Long studentCaseId) {

        //获取案例活动及项目活动名称
        Map<String, Object> map1 = studentCaseService.getLibraryProjectName(caseLibraryId, studentCaseId);

        String caseLibraryName = (String) map1.get("caseLibraryName");

        List<CaseLibraryRelatedClassify> caseLibraryRelatedClassifyList = (List<CaseLibraryRelatedClassify>) map1.get("relatedClassify");
        Long trainingProjectId = (Long) map1.get("trainingProjectId");

        String trainingProjectName = null;
        if (null != trainingProjectId) {
            List<Long> idList = new ArrayList<>();
            idList.add(trainingProjectId);
            Map<Long, TrainingProjectVo> trainingProjectMap = tpProjectClient.getCaseLibraryProject(idList);
            trainingProjectName = trainingProjectMap.get(trainingProjectId).getName();
        }

        map1 = new HashMap<>(2);
        map1.put("caseLibraryName", caseLibraryName == null ? "" : caseLibraryName);
        map1.put("trainingProjectName", trainingProjectName == null ? "" : trainingProjectName);
        map1.put("relatedClassify", caseLibraryRelatedClassifyList);

        return map1;
    }

}
